# iPBAvizu using PyMOL (PyMol 1.45.05.45, iPBA 1.01, iPBA_Vizu 1.00)
#######################################################################
# iPBAvizu version 1.00 - 01/02/13                                    #
# developped by Agnel Praveen Joseph, Guilhem Faure, N. Srinivasan,   #
# Sylvain Leonard, Joseph Rebehmed, Pierrick Craveur,                 #
# Jean-Christophe Gelly & Alexandre G. de Brevern                     #
# (c) 2010-2013 INSERM UMR_S 665, DSIMB, Univ Paris Diderot           #
# (DSIMB and INDIA)
#######################################################################
# please cite (1) Joseph, Srinivasan & de Brevern (2011) Biochimie    #
# and (2) Faure, Joseph, Srinivasan, Craveur, Rebehmed  & de Brevern  #
#######################################################################
# THE AUTHOR(S) DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN
# NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# ----------------------------------------------------------------------

import os
import string
import sys
import tempfile
import shutil
import glob
import sys
import subprocess
import Pmw
import Tkinter
import tkFileDialog
import tkMessageBox


class Setup:
    
    def __init__(self):
        
        #=======================================================================
        # open a window
        #=======================================================================
        root = Tkinter.Tk()
        self.root = root
        root.title("Configuration of iPBAVizu")        
        #=======================================================================
        # entry for address
        #=======================================================================
        label = Tkinter.Label(root, text="Install directory: ")
        label.grid(row=0, column=0)
        self.installdir = Tkinter.StringVar()
        entry = Tkinter.Entry(root, textvariable=self.installdir)
        entry.grid(row=0, column=1)
        # initialization
        self.installdir.set("/usr/local")
        s = entry.get()
        
        #=======================================================================
        # button to explore directory
        #=======================================================================
        b_browser = Tkinter.Button(root, text="Browser", command=self.openbrowser)
        b_browser.grid(row=0, column=2)
        
        
        b_install = Tkinter.Button(root, text="Install", command=self.install)
        b_install.grid(row=1, column=0)
        
        
        root.mainloop()
        
        return None
        
    def install(self,):
        """
        brief launch the install with the installdir entry by user
        """
        
        
        if not os.path.exists(self.installdir.get()):
            self.error()
            return
            
        #path = os.path.join(self.installdir.getvalue(), "iPBA")
        path = self.installdir.get()
            
            
            
        self.configure(path)
        self.copy(path)
        self.check(path)
        
        return
    

    def copy(self, path):
        """
        brief copy the program in local
        param path address of install of iPBA
        """
        
        err = os.popen("cp -r %s %s"%("ipba_prog", path)).read()
	os.system("chmod -R 777 %s"%(os.path.join(path, "ipba_prog")))
        
        return None
    
        
    def configure(self, path):
        """
        brief write the environnment for iPBA in Main iPBA and write the address of iPBA in the plugin
        brief 1/ we add import sys and sys.path environnement for iPBA because pymol destroy the local env
        brief 2/ we add the address of iPBA main install in the plugin
        param path address of install of iPBA 
        """
        
        #=======================================================================
        # we create a file with only the import sys and initialization of sys.path
        #=======================================================================
        # Get the python environnment for PyMoL
        env = string.join(sys.path)
        fout = open("/tmp/env_iPBA.py", "w")
        fout.write("#!/usr/bin/env python\nimport sys\nsys.path = %s\n"%("['%s','"%(os.path.join(path, "ipba_prog"))+string.join(env.split(), "','")+"']\n"))
        fout.close()
        
        #=======================================================================
        # the file without the config of sys.path
        #=======================================================================
        noconfigfile = "ipba_prog/MainParserPDB_align.py_noconfig"
        
        #=======================================================================
        # we concatenate the init file and the file without config
        #=======================================================================
        fout = open("ipba_prog/MainParserPDB_align.py", "w")
        shutil.copyfileobj(open("/tmp/env_iPBA.py", 'r'), fout)
        shutil.copyfileobj(open(noconfigfile, 'r'), fout)
        fout.close()
        
        
        #=======================================================================
        # add the address of iPBA of install of iPBAVizu 
        #=======================================================================
        fout = open("/tmp/envad_iPBAVizu.py", "w")
        fout.write("#!/usr/bin/env python\n__ADDRESSPBA__ = \"%s\"\n\n"%(os.path.join(path, "ipba_prog", "MainParserPDB_align.py")))
        fout.close()
        
        #=======================================================================
        # the file without the config of __ADDRESSPBA__
        #=======================================================================
        noconfigfile = "iPBAVizu.py_noconfig"
        
        #=======================================================================
        # we concatenate the init file and the file without config
        #=======================================================================
        fout = open("iPBAVizu.py", "w")
        shutil.copyfileobj(open("/tmp/envad_iPBAVizu.py", 'r'), fout)
        shutil.copyfileobj(open(noconfigfile, 'r'), fout)
        fout.close()
        
        #=======================================================================
        # to avoid ambiguity we copy the iPBAVizu plugin in the path repertory chose to install
        # Improvement alpha test 02/04/2012
        #=======================================================================
        if path == os.getcwd(): # same repertory than installer
            return None
        shutil.copyfile("iPBAVizu.py", os.path.join(path,"iPBAVizu.py"))
        
        return None    
    
    
    
    def openbrowser(self, ):
        """
        Open a browser file
        fill self.file
        """
        self.dir = tkFileDialog.askdirectory(initialdir = "/usr/local", title="Choose a director to install iPBA")
        self.installdir.set(self.dir)
        return None
    
    
    
    def check(self, path):
        """
        """
        
        if os.path.exists(os.path.join(path, "ipba_prog", "MainParserPDB_align.py")) and \
                          os.path.exists(os.path.join("iPBAVizu.py")):
            self.end()
        else:
            self.error()
        
        
    
    def error(self, ):
        """
        """
        tkMessageBox.showwarning("Problem during Install", "check you have the permission in the work directory")
        
        return
        
    
    def end(self):
        """
        """
        
        
        tkMessageBox.showinfo("Install Completed", "Install completed, you can now launch PyMol\nand install iPBAVizu as a plugin")
        
        #=======================================================================
        # when we click OK after install complete, all the program is closed
        # Improvement alpha test 02/04/2012
        #=======================================================================
        self.root.quit()
        
        return
    
    




Setup()

