from math import *


__doc__ = """Module pour le calcul d'angles diedre"""

#------------------------------------------------------------------------------------------------------------------------------------
def CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD) :
	"""Fonction de calcul d'un angle diedre defini par 4 points A,B,C,D : les 4 carbones alpha successifs
	ABC sont dans le plan 1 et BCD dans le plan 2, l'angle entre 2 plan est l'angle entre les vecteurs normaux a ces plans*/"""

	
	#Coordonnees des vecteurs definis par les 4 points
	BA = [CoordAtomA[0]-CoordAtomB[0],CoordAtomA[1]-CoordAtomB[1],CoordAtomA[2]-CoordAtomB[2]]
	BC = [CoordAtomC[0]-CoordAtomB[0],CoordAtomC[1]-CoordAtomB[1],CoordAtomC[2]-CoordAtomB[2]]
	CB = [CoordAtomB[0]-CoordAtomC[0],CoordAtomB[1]-CoordAtomC[1],CoordAtomB[2]-CoordAtomC[2]]
	CD = [CoordAtomD[0]-CoordAtomC[0],CoordAtomD[1]-CoordAtomC[1],CoordAtomD[2]-CoordAtomC[2]]

	#Premier produit vectoriel
	PV_BABC = [0.0]*3
	PV_BABC[0]= BA[1]*BC[2]- BA[2]*BC[1]
	PV_BABC[1]= BA[2]*BC[0]- BA[0]*BC[2]
	PV_BABC[2]= BA[0]*BC[1]- BA[1]*BC[0]


	#Deuxieme produit vectoriel
	PV_CBCD = [0.0]*3
	PV_CBCD[0]= CB[1]*CD[2]- CB[2]*CD[1]
	PV_CBCD[1]= CB[2]*CD[0]- CB[0]*CD[2]
	PV_CBCD[2]= CB[0]*CD[1]- CB[1]*CD[0]

	#Normes des vecteurs perpendiculaires a chaque plan
	Norm_PV_BABC = 0.0
	Norm_PV_CBCD = 0.0
	for i in range(3):
		Norm_PV_BABC +=  PV_BABC[i]*PV_BABC[i]
		Norm_PV_CBCD +=  PV_CBCD[i]*PV_CBCD[i]
	
	Norm_PV_BABC = sqrt(Norm_PV_BABC)
	Norm_PV_CBCD = sqrt(Norm_PV_CBCD)

	#Calcul du produit scalaire des 2 vecteurs perpendiculaires a chaque plan
	PScalaire = 0.0
	for i in range(3): 
		PScalaire +=  PV_BABC[i]*PV_CBCD[i];

	#Angle entre les 2 vecteurs perpendiculaires a chaque plan
	cosDIEDRE = PScalaire/(Norm_PV_BABC*Norm_PV_CBCD)
	if(fabs(cosDIEDRE)>1.0) :cosDIEDRE=1.0 #petit pb d'arrondi qui donne -1.00000012 
	DIEDRE = acos(cosDIEDRE) #en radian
	DIEDRE = DIEDRE*180.0 /pi  #conversion en degre

	#Levee de l'indetermination sur le signe de l'angle
	#--> calcul du produit vectorielle de PV_BABC et PV_CBCD
	PV_pv1pv2 = [0.0]*3
	PV_pv1pv2[0]= PV_BABC[1]*PV_CBCD[2]- PV_BABC[2]*PV_CBCD[1]
	PV_pv1pv2[1]= PV_BABC[2]*PV_CBCD[0]- PV_BABC[0]*PV_CBCD[2]
	PV_pv1pv2[2]= PV_BABC[0]*PV_CBCD[1]- PV_BABC[1]*PV_CBCD[0]
	#-->Est-ce que l'angle "s'ouvre" (positif) dans le sens de la sequence (triedre direct) (???)
	#Produit scalaire du vecteur perpendiculaire aux PVs et du vecteur central dans le sens de la sequence
	if ((PV_pv1pv2[0]*BC[0]+PV_pv1pv2[1]*BC[1]+PV_pv1pv2[2]*BC[2])<0) : DIEDRE = -DIEDRE #sin <0

	
	return DIEDRE
#------------------------------------------------------------------------------------------------------------------------------------


# TEST ----------------------		
def main ():
	CoordAtomA = [32.035 , 37.952 , 52.834  ]
	CoordAtomB = [32.939 , 38.301  ,56.494]
	CoordAtomC = [ 30.110 , 36.890,  58.586 ]
	CoordAtomD = [30.127 , 37.743 , 62.283  ]
	print CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD) 

	CoordAtomA = [31.860 , 37.760 , 52.970]
	CoordAtomB = [32.820 , 38.240 , 56.570]
	CoordAtomC = [29.990 , 36.900 , 58.780]
	CoordAtomD = [30.280 , 37.900 , 62.390 ]
	print CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD) 


	
	CoordAtomA = [32.939 , 38.301  ,56.494]
	CoordAtomB = [ 30.110 , 36.890,  58.586 ]
	CoordAtomC = [30.127 , 37.743 , 62.283  ]
	CoordAtomD = [30.434 , 38.166,  65.069  ]
	print CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD) 


	CoordAtomA = [32.820 , 38.240 , 56.570]
	CoordAtomB = [29.990 , 36.900 , 58.780]
	CoordAtomC = [30.280 , 37.900 , 62.390 ]
	CoordAtomD = [30.240 , 38.330,  65.320 ]
	print CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD) 
	

if __name__ == "__main__":
	main()



