
import re
from os import popen
from os import system
from os import getcwd
from os import chdir
from os import mkdir
import sys
import math
import ResiduObj
import cPickle

__doc__ = """Module destine a l'assignation de SII par STRIDE de proteines PDB parsees et enregistrees en cPickle."""

ProgSTRIDE = "stride"

delta = 0.1

#-------------------------------------------------------------------------------------------------------------
class InfoSTRIDEetEnregistrementDiff (Exception):
	def __init__(self ):
		print >>sys.stderr, "Les infos donnees par Stride sont differentes de celles enregitrees"
#-------------------------------------------------------------------------------------------------------------



def Lancer_STRIDE (nomFichierPDB, file_stride):
	"""Methode pour lancer la commance STRIDE. N'ecrit pas un fichier : return le resultat sous forme de stringIO"""
	commande = "%s '%s' -f'%s' 2> /dev/null"%(ProgSTRIDE,nomFichierPDB, file_stride)
#	print commande
	popen(commande) 


def LectureSortieSTRIDE (SortieSTRIDE):
	"""Methode pour la lecture et la recuperation des infos donnees par STRIDE"""

	ListeInfoSTRIDE = {}
	flag_nofile = 0
	#Lecture de la sortie de stride
	try: Fichstride = open(SortieSTRIDE)
	except IOError, msg: 
		print msg
		print "warning!! Secondary Struct assignment by Stride failed!!!"
		flag_nofile = 1
	if flag_nofile != 1:
		Lignesstride = Fichstride.readlines()
		Fichstride.close()
	
	#Recup des infos
		for ligne in Lignesstride :
#				print ligne
			if ligne[0:3] == "ASG":
				Listeligne = []
				#Listeligne.append(int(ligne[11:15])) #Numero du residu dans le PDB fourni
				Listeligne.append(ligne[5:8]) #Nom du residu
				Listeligne.append(ligne[9]) #Nom de la chaine
				Listeligne.append(ligne[24].upper()) #SII
				Listeligne.append(float(ligne[42:49])) #angle Phi
				Listeligne.append(float(ligne[52:59])) #angle Psi
				ListeInfoSTRIDE[ligne[11:16]] = Listeligne#Numero du residu dans le PDB fourni
				#print Listeligne
		if len(ListeInfoSTRIDE) == 0:
			print"warning!! SII assignment records missing"
			
		
		
	return ListeInfoSTRIDE
	
	
def Recup_AssignationSTRIDE (NomProt,model, IDchaine, RepertoirePDBPartieProt, dir_stride):
	"""Methode pour lancer STRIDE et recuperer les lignes d'assgination"""
	
	#Nom du PDB de la partie de la prot donnee dans sa conformation desiree
	if IDchaine == " ": CharChaine = "_"
	else : CharChaine = IDchaine
	NomFichierPDB = RepertoirePDBPartieProt+"/"+NomProt+"_"+`model`+"_"+IDchaine+".pdb"
#	print NomFichierPDB
	
	file_stride = dir_stride+"/"+NomProt+"_"+`model`+"_"+IDchaine+".str"
	stride_filename = NomProt+"_"+`model`+"_"+IDchaine+".str"
	#Lancement de STRIDE
	Lancer_STRIDE (NomFichierPDB, file_stride)
	

	#Lecture de la sortie de stride et renvoie des lignes d'assignation
	List_stride = LectureSortieSTRIDE (file_stride)
	if not dir_stride.split("/")[-1] == "temp_stride":
		cur_dir = getcwd()
		chdir(dir_stride)
		command = "tar cf - "+stride_filename+"| gzip -c >"+stride_filename+".tar.gz"
		system(command)
		chdir(cur_dir)
		system("rm -f "+file_stride)
	else: system("rm -f "+file_stride)
	return List_stride

def Modelset_SIIParRes_STRIDE (ObjetModele,NomPDB,IDchaine,RepertoirePDBPartieProt ,verif=True):
	"""Methode permettant d'assigner temporairement la PREMIERE (max occupancy moyenne) conformation possible d'un modele residu par residu avec Stride.
	L'option verif donne la possibilite de verifier la concordance des angles phi/psi enregistres et ceux calcules par Stride 
	On renvoie une liste avec l'assignation de la sequence complete. 
	On efface finalement cette assignation temporaire."""
	dictGlobalStride = {}
	#Recuperation des assignations STRIDE de chaque Partie du modele
	if ObjetModele.ListeIndicesPartiesProt == None :
		ObjetModele.getIndicesDebFinPartie()
	listeIndicesDebFinPartie = ObjetModele.ListeIndicesPartiesProt 
	for numPart in range(len(listeIndicesDebFinPartie)):
		dictGlobalStride.update(Recup_AssignationSTRIDE (NomPDB,IDchaine, numPart, listeIndicesDebFinPartie[numPart][0], listeIndicesDebFinPartie[numPart][1],0, RepertoirePDBPartieProt))
	
	#print dictGlobalStride
	#Pour la verification de la concordance des angles Phi/psi
	comptErreur = 0
	
	#Parcours de la sequence globale
	for i in range(len(ObjetModele.Seq_Struct)):
		res = ObjetModele.Seq_Struct[i]
		
		#Si assignation Stride non disponible : Partie de sequence trop courte (< 5 res), break,residu incomplet
		if not dictGlobalStride.has_key(i) :
			res.setAttributTEMP ("#")
		else :
			#Verification minimum : on verifie que le nom du residu correspond
			#print res.Type, i
			if res.Type == dictGlobalStride[i][0] :
				#Enregistrement temporaire de l'assignation STRIDE
				res.setAttributTEMP (dictGlobalStride[i][2])
			else :
				raise InfoSTRIDEetEnregistrementDiff()
			#Verification de la concordance des angles Phi/psi
			if verif == True :
				AnglePhienr = res.AnglesPhiPsi['Phi'][0]
				#print AnglePhienr
				AnglePsienr = res.AnglesPhiPsi['Psi'][0]
				#print AnglePsienr
				AnglePhistride = dictGlobalStride[i][3]
				if AnglePhistride == 360.0 : AnglePhistride = 999
				#print AnglePhistride
				AnglePsistride = dictGlobalStride[i][4]
				if AnglePsistride ==360.0  : AnglePsistride = 999
				#print AnglePsistride
				if (math.fabs(AnglePhienr) > math.fabs(AnglePhistride)+delta  or math.fabs(AnglePhienr) < math.fabs(AnglePhistride)-delta) : #angle phi different dans STRIDE et enregistrement (fabs : car Stride peut parfois mal lire un fichier ??? et donner des Phi/Psi opposes)
					comptErreur += 1
					#print "DIFF !!!"
				if (math.fabs(AnglePsienr) > math.fabs(AnglePsistride)+delta  or math.fabs(AnglePsienr )< math.fabs(AnglePsistride)-delta)  : #angle psi different dans STRIDE et enregistrement
					comptErreur += 1
					#print "DIFF !!"
					
	
	#Comptabilisation du nombre d'erreurs
	if comptErreur/len(ObjetModele.Seq_Struct) > 0.05 :
		raise InfoSTRIDEetEnregistrementDiff()
	#Recuperation de la liste global d'assignation
	assignationsSTRIDE = [res.getAttributTEMP(0) for res in ObjetModele.Seq_Struct]
	#liberation de l'attribut temporaire
	[res.Del_AttributTEMP() for res in ObjetModele.Seq_Struct]
	
	return assignationsSTRIDE


# TEST ----------------------		
def main ():
	
	RepertoirePDBPartieProt = "PDBsClean_Parties"
	system("tar -zxf '"+RepertoirePDBPartieProt+"/"+"pdb2b97.ent"+"_Chaine"+"A"+"_Mod"+'0'+".tgz' ")
	lignes = Recup_AssignationSTRIDE ("pdb2b97.ent","A", 0, 0,69,0, "PDBsClean_Parties")
	system("rm -rf "+RepertoirePDBPartieProt+"/*"+"2b97"+"*.pdb")	
#	print lignes
	
#	PDB ="pdb2b97.ent"
#	Chaine = "A"
	PDB ="pdb2c49.ent"
	Chaine = "A"
	PDB ="pdb2cb5.ent"
	Chaine = "B"

	
	fp = open("PDBsPickle/"+PDB,'r')
	prot = cPickle.load(fp)
	fp.close()
	print prot.Chaines[Chaine ].ModelsStruct[0].memoNombreConformationsALT_CA
	system("tar -zxf '"+RepertoirePDBPartieProt+"/"+PDB+"_Chaine"+Chaine+"_Mod"+'0'+".tgz' ")
	print Modelset_SIIParRes_STRIDE (prot.Chaines[Chaine ].ModelsStruct[0],PDB,Chaine,RepertoirePDBPartieProt ,verif=True)
	system("rm -rf "+RepertoirePDBPartieProt+"/*"+PDB+"*.pdb")	
	
	
	
if __name__ == "__main__":
	main()
	
