import sys
import Definitions
import ModelObj
import ResiduObj
import needle
import re
import textwrap
from math import *

__doc__= """Module de definir un objet Chaine proteique"""


#-------------------------------------------------------------------------------------------------------------
class AtomeNonDefini (Exception):
	def __init__(self,TypeAtome ):
		print >>sys.stderr, "L'atome %s n'est pas defini"%(TypeAtome )
#-------------------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------------------
class ChaineProt :
	

	def __init__(self, NomPDB, IDchaine):
		"""Initialisation d'une objet Chaine Proteique"""
		#self.NomPDBclean = "%s_clean.pdb"%NomPDB
		self.IDchaine = IDchaine
		self.Seq_3lettres = []
		self.Seq_1lettre = ""
		self.ModelsStruct = []
		self.SCOP = {}  # attention !! Les positions enregistrees quand il y a plusieurs domaines de classes SCOP differentes sont celles donnees par SCOP (ie : position dans la structure)
						#Mais Quand un seul domaine : les positions enregistrees englobe toute la sequence connue !
		
		Liste_Seq1Lettre = []

	def setSequence_3lettres (self, ListeSeq3lettres):
		"""Initialisation du champ sequence 3 lettres: liste des aa codes en 3 lettres"""
		self.Seq_3lettres = ListeSeq3lettres
	
	def setSequence_1lettres (self, StringSeq1lettre):
		"""Initialisation du champ sequence 1 lettre: string des aa codes en 1 lettres"""
		self.Seq_1lettre = StringSeq1lettre
		
	def setSCOP (self, classe, deb, fin, numDomaine):
		"""Initialisation du champ SCOP.Les classes sont a, b, a/b, a+b et _. Elles servent de cle.
		deb et fin correspondent a la numerotation dans la sequence A PARTIR de 1, mis sous forme de liste.
		Comme plusieurs domaines de la meme classe possible, a priori,ajout du num de domaine a la cle."""
		self.SCOP[classe+str(numDomaine)] = [deb, fin]
		

	def Seq3lettresTO1lettre (self, ListeSeq3lettres):
		"""Methode pour transformer une seq sous forme de liste d'aa codes en 3 lettres 
		en une seq sous forme de string d'aa codes en une lettre """
		Liste_Seq1Lettre = []
		for aa in ListeSeq3lettres:
			if not Definitions.EquiAA_3to1lettres[aa] == 'X':
				try:
 					Liste_Seq1Lettre.append(Definitions.EquiAA_3to1lettres[aa])
				except KeyError:
					pass	
		List_copy_1letter = Liste_Seq1Lettre
		for res in Liste_Seq1Lettre:
			if res == 'X':
				Liste_Seq1Lettre.remove('X')		
		return ''.join(Liste_Seq1Lettre)
		
	def add_ModelsStruct (self):
		"""Methode pour enregistrer les differents models proteiques disponibles pour une chaine"""
		NewModelChaine = ModelObj.Model()
		self.ModelsStruct.append(NewModelChaine)

	def ChaineTOListOfString_PDB (self,stringList=[] ):
		"""Impression d'une chaine prot au format PDB"""
			
		
		[model.ModelTOListOfString_PDB (self.IDchaine ,stringList ) for model in self.ModelsStruct][0]
		#print stringList
		stringList.append("TER\n")
		#print stringList
	
	def ChaineTOListOfString_Banque (self,stringList=[], CodePDB=None,TypeFich="BanqueGenerale"):
		"""Impression d'une chaine prot au format Banque"""
			
		if TypeFich=="BanqueGenerale":
			#On n'imprime que le premier modele
			self.ModelsStruct[0].ModelTOListOfString_Banque (stringList,CodePDB=CodePDB, IdChaine=self.IDchaine , TypeFich=TypeFich )
		elif TypeFich=="ParChaine" :
			#On imprime tous les modeles partie par partie
			NumModel = 0
			for model in self.ModelsStruct:
				 ListeIndicesParties=model.ListeIndicesPartiesProt
				 #print ListeIndicesParties
				 for numPartie in range(len(ListeIndicesParties)):
				 	model.ModelTOListOfString_Banque (stringList,NumModel= NumModel ,PartieProt = numPartie,PositionsPartiesProt=ListeIndicesParties[numPartie], TypeFich=TypeFich )
				 NumModel += 1
		print stringList
		return stringList	
			
	def ChaineTOListOfString_Header (self, CodePDB, MethodeExp, Resolution):
		"""Impression du header se raportant a une chaine"""
		ListeLignes = []
		ligne1 = "%s%s\n"%(CodePDB,self.IDchaine)
		ListeLignes.append(ligne1)
		if Resolution != None:
			ligne2 = "%s	reso = %f\n"%(MethodeExp, Resolution)
		else : 
			ligne2 = "%s\n"%(MethodeExp)
		ListeLignes.append(ligne2)
		ligne3 = "Nombre Modeles : %d\n"%(len(self.ModelsStruct))
		ListeLignes.append(ligne3)
		
		for NumModel in range(len(self.ModelsStruct)) :
			
			for NumPart in range(len(self.ModelsStruct[NumModel].ListeIndicesPartiesProt)):
				#print "Indice Part",len(self.ModelsStruct[NumModel].ListeIndicesPartiesProt)
				#print "Mem conf",len(self.ModelsStruct[NumModel].memoNombreConformationsALT_CA)
				#print self.ModelsStruct[NumModel].memoNombreConformationsALT_CA
				#print self.ModelsStruct[NumModel].memoNombreConformationsALT_CA[NumPart]
				ligne = "Model%2d Part%2d	Deb%4d  Fin%4d	NbConf%2d\n"%(NumModel,NumPart,self.ModelsStruct[NumModel].ListeIndicesPartiesProt[NumPart][0],self.ModelsStruct[NumModel].ListeIndicesPartiesProt[NumPart][1],self.ModelsStruct[NumModel].memoNombreConformationsALT_CA[NumPart])
				ListeLignes.append(ligne)
				
			#Indice des positions alt
			ligne = "Indices Positions alt : \n"
			ListeLignes.append(ligne)
			for numRes in range(len(self.ModelsStruct[NumModel].Seq_Struct)):
				if self.ModelsStruct[NumModel].Seq_Struct[numRes].AltLocationOfCa > 0:
					ListeLignes.append("%d\n"%(numRes))
				
		
		return ListeLignes
		
		
	def ChaineToFasta (self,CodePDB):
		"""Methode pour obtenir la sequence complete de la chaine sous forme de lignes au format fasta"""
		lignes = [">%s%s|length%d\n"%(CodePDB,self.IDchaine,len(self.Seq_1lettre))]
		lignes += textwrap.fill(self.Seq_1lettre,60).splitlines(True)
		lignes[-1]+= "\n"
		return lignes
		
	def ChaineBreaksToFasta (self,CodePDB):
		"""Methode pour obtenir les lignes des sequences des breaks dans la chaine"""
		#Parcours des modeles
#		for NumModel in range(len(self.ModelsStruct)) :
		seqBrk = ""
		break_count = 0
		lignes = []
		dict_break_count = {}
		res_in_break = 0
		#Si le modele a des breaks
		if self.ModelsStruct[-1].FlagExistanceBreak != 0:
			#Parcours de la structure
			for i in range(len(self.ModelsStruct[-1].Seq_Struct)):
				#Qd j'ai un break
				if self.ModelsStruct[-1].Seq_Struct[i].__class__ == ResiduObj.Break :
					seqBrk += Definitions.EquiAA_3to1lettres [self.ModelsStruct[-1].Seq_Struct[i].Type]
					if i == len(self.ModelsStruct[-1].Seq_Struct)-1:
						break_count += 1
						res_in_break += len(seqBrk)
						lignes += [">%s%s|lengthBrk%d|Indices%d-%d\n"%(CodePDB,self.IDchaine,len(seqBrk),i-len(seqBrk)+1,i)]
						lignes += textwrap.fill(seqBrk,60).splitlines(True)
						lignes[-1]+= "\n"
                        	                dict_break_count[break_count] = [res_in_break]

				elif seqBrk != "":
					break_count += 1
					res_in_break += len(seqBrk)
					lignes += [">%s%s|lengthBrk%d|Indices%d-%d\n"%(CodePDB,self.IDchaine,len(seqBrk),i-len(seqBrk),i-1)]
					lignes += textwrap.fill(seqBrk,60).splitlines(True)
					lignes[-1]+= "\n"
					seqBrk = ""
                                       	dict_break_count[break_count] = [res_in_break]
	
					
		#print lignes
		return [lignes, dict_break_count]	
#-------------------------------------------------------------------------------------------------------------



# TEST ----------------------		
def main ():
	print "PDB1"
	try:
# CREATING A CLASS INSTANCE C1 FOR Chaineprot
		C1 = ChaineProt("1IOW",' ')
		print C1.IDchaine
		#print C1.NomPDBclean
		
	except Exception, msg:
		print >>sys.stderr, msg
		pass

	print "PDB2"
	try:
		C2 = ChaineProt("1EE8", 'A')
		print C2.IDchaine
		print C2.NomPDBclean
		
	except Exception:
		print >>sys.stderr, msg
		pass

if __name__ == "__main__":
	main()
