

import AtomeObj

__doc__ = """Module pour construire et gerer une structure de donnees de type arbre pour gerer les conformations alternatives lors du parsage des PDBs"""


#-----------------------------------------------------------------------------------------------------------------------
class Arbre :
	def __init__(self):
		self.NumAltLoc = None
		self.Occupancy = None
		self.ArbresFils = []
		

	def setNumAltLoc (self, valeur):
		self.NumAltLoc = valeur
		
	def setOccupancy (self, occ):
		self.Occupancy = occ

	def AddArbreFils (self,arbre ) :
		self.ArbresFils.append(arbre)
		
#-----------------------------------------------------------------------------------------------------------------------



#-----------------------------------------------------------------------------------------------------------------------
def ConstruireArbreConformationALTLOC (ListeAtomes,i,NbreAltlocAtomeRangCourant=0,ArbreCourant = Arbre() ):
	"""Methode permettant d'ajouter des fils (positions d'atomes) a un arbre lors de la contruction d'un arbre representant toutes les conformations possibles ALTLOC"""
		
#	print ListeAtomes
	#print i
	
	if i != len(ListeAtomes):
		
		NewAtome = ListeAtomes[i]
		PositionsAltloc = NewAtome.Coord.keys()
		NbreAltlocNewAtome= len(PositionsAltloc)
#		print "NbreAltlocNewAtome =",NbreAltlocNewAtome
	
		
		#L'atome a ajoute est le debut d'un segment avec (ou sans) ALTLOC
		if NbreAltlocAtomeRangCourant != NbreAltlocNewAtome:
			for j in range(NbreAltlocNewAtome):
				NewArbre = Arbre()
				NewArbre.setNumAltLoc (PositionsAltloc[j])
				NewArbre.setOccupancy (NewAtome.Occupancy[j])
				ArbreCourant.AddArbreFils(NewArbre)
#				print NewArbre.NumAltLoc #****
#				print ArbreCourant.ArbresFils
				ConstruireArbreConformationALTLOC (ListeAtomes,i+1,NbreAltlocAtomeRangCourant=NbreAltlocNewAtome,ArbreCourant =NewArbre)
			
			NbreAltlocAtomeRangCourant = NbreAltlocNewAtome
		
		else : #l'atome a ajoute est dans la continuite d'un segment tout ALTLOC ou tout avec une seule position
#			print "CAS 2" #***
#			for i in range(NbreAltlocAtomeRangCourant):
				#print i
			NewArbre = Arbre()
			NewArbre.setNumAltLoc (PositionsAltloc[ArbreCourant.NumAltLoc])
			NewArbre.setOccupancy (NewAtome.Occupancy[ArbreCourant.NumAltLoc])
			ArbreCourant.AddArbreFils(NewArbre)
#			print NewArbre.NumAltLoc #****
			ConstruireArbreConformationALTLOC (ListeAtomes, i+1,NbreAltlocAtomeRangCourant=NbreAltlocNewAtome,ArbreCourant =NewArbre)
#			print "\n" #***
#	except StopIteration:
	#	break
#-----------------------------------------------------------------------------------------------------------------------




#-----------------------------------------------------------------------------------------------------------------------
def _ParcoursEnProfondeur_Listes (Arbre, ListeAltloc , listeTemp=[]):
	"""Methode privee (de GetCheminsPossibles) renvoyant la liste des listes de noeuds (plus precisement leur cle altloc) correspondant a un parcours en profondeur (== parcours Prefixe)
	Une liste differente est commencee a chaque fois que le bas de l'arbre est atteint"""
	if Arbre.ArbresFils !=[]:
		
		for arbre in Arbre.ArbresFils:
			listeTemp.append(arbre.NumAltLoc)
			_ParcoursEnProfondeur_Listes (arbre,ListeAltloc,listeTemp)
	else :
		ListeAltloc.append(listeTemp[:])
		#print listeTemp
		while listeTemp != []:
			listeTemp.pop()

#-----------------------------------------------------------------------------------------------------------------------


#-----------------------------------------------------------------------------------------------------------------------

def GetCheminsPossibles (listeAtomes):
	"""Methode permettant d'obtenir toutes les combinaisons possibles des conformations ALTLOC 
	ainsi que l'Occupancy moyenne associee. """
	
	#Construction de l'arbre permettant de trouver toutes les conformations(chemins)possibles
	ArbreAtomes =  Arbre()
	#print "Avant Arbre"
	ConstruireArbreConformationALTLOC (listeAtomes,0, ArbreCourant = ArbreAtomes )
	#PrintArbreParcoursLargeur (ArbreAtomes) #***
	#print "Apres Arbre"
	
	
	#ListeAltloc est obtenu par _ParcoursEnProfondeur_Listes : 
	#c'est une liste issu du parcours en profondeur de l'arbre avec la creation d'une nouvelle sous-liste lorsque le bas de l'arbre est atteint.
	ListeAltloc = []
	_ParcoursEnProfondeur_Listes (ArbreAtomes, ListeAltloc, listeTemp=[])
	#print ListeAltloc #***
	
	#Completion des listes pour obtenir les chemins entiers
	ListeAltloc_Complete = []
	ListeAltloc_Complete.append(ListeAltloc[0])
	for i in range(len(ListeAltloc))[1:]:
		#print ListeAltloc[i], ListeAltloc_Complete[i-1]
		if len(ListeAltloc[i]) < len(ListeAltloc_Complete[i-1]):
			#print "Bout 1 =",ListeAltloc_Complete[i-1][:-len(ListeAltloc[i])]
			Newliste = ListeAltloc_Complete[i-1][:-len(ListeAltloc[i])]+ListeAltloc[i]
			#print "Newliste=",Newliste
			ListeAltloc_Complete.append(Newliste)
		else :
			ListeAltloc_Complete.append(ListeAltloc[i])
	
	#print ListeAltloc_Complete
	
	#Calcul des Occupancy moyennes par fragment
	ListeAltloc_OccupancyMoy =[]
	for chemin in ListeAltloc_Complete:
		OccMoy = 0.0
		for loc,atome in zip(chemin,listeAtomes):
			OccMoy += atome.Occupancy[loc]
		OccMoy = OccMoy/len(listeAtomes)
		ListeAltloc_OccupancyMoy.append(OccMoy)
	
	#print ListeAltloc_OccupancyMoy
	
	
	return ListeAltloc_Complete, ListeAltloc_OccupancyMoy
	
	
	
#-----------------------------------------------------------------------------------------------------------------------



#-----------------------------------------------------------------------------------------------------------------------
def PrintArbrePrefixe (Arbre):
	if Arbre.ArbresFils!=[]:
		
		for arbre in Arbre.ArbresFils:
			print arbre.NumAltLoc,arbre.Occupancy
			PrintArbrePrefixe (arbre)
		#if len( Arbre.ArbresFils)== 1:
		#	print "-"
		#	#print arbre.NumAltLoc
	else :
		print "-"
		
#-----------------------------------------------------------------------------------------------------------------------


#-----------------------------------------------------------------------------------------------------------------------
def PrintArbreParcoursLargeur (Arbre):
	f=[]
	f.append(Arbre)
	while(f != []):
		#print f
		noeud = f.pop(0)
		if noeud == "-":
			print noeud
		else :
			print noeud.NumAltLoc, noeud.Occupancy
			for fils in noeud.ArbresFils:
				f.append(fils)
			if len(noeud.ArbresFils)!= 0:
				f.append("-")
		
#-----------------------------------------------------------------------------------------------------------------------





# TEST ----------------------
def main ():


	n1 = Arbre()
	n1.setNumAltLoc(1)
	n2 = Arbre()
	n2.setNumAltLoc(2)
	n3 = Arbre()
	n3.setNumAltLoc(3)
	n4 = Arbre()
	n4.setNumAltLoc(4)
	n5 = Arbre()
	n5.setNumAltLoc(5)
	n6 = Arbre()
	n6.setNumAltLoc(6)
	n7 = Arbre()
	n7.setNumAltLoc(7)
	n8 = Arbre()
	n8.setNumAltLoc(8)
	n9 = Arbre()
	n9.setNumAltLoc(9)
	A = Arbre()
	A.AddArbreFils(n1)
	A.AddArbreFils(n2)
	n1.AddArbreFils(n3)
	n1.AddArbreFils(n4)
	n2.AddArbreFils(n5)
	n3.AddArbreFils(n6)
	n3.AddArbreFils(n7)
	n4.AddArbreFils(n8)
	n6.AddArbreFils(n9)
	#PrintArbrePrefixe (A)
	#PrintArbreParcoursLargeur (A)


	#Arbre 1
	print "Arbre 1"
	A1 = AtomeObj.Atome('CA')
	A1.set_Coord([])
	A1.set_Occupancy(0.80)
	A1.AltLocation = 1
	A1.set_Coord([])
	A1.set_Occupancy (0.20)
	
	A2 = AtomeObj.Atome('CA')
	A2.set_Coord([])
	A2.set_Occupancy(0.80)
	#A2.AltLocation = 1
	#A2.set_Coord([])
	#A2.set_Occupancy (0.20)
	
	A3 = AtomeObj.Atome('CA')
	A3.set_Coord([])
	A3.set_Occupancy(1.0)
	A3.AltLocation = 1
	A3.set_Coord([])
	A3.set_Occupancy (0.20)
	A3.AltLocation = 2
	A3.set_Coord([])
	A3.set_Occupancy (0.20)
	
	A4 = AtomeObj.Atome('CA')
	A4.set_Coord([])
	A4.set_Occupancy(0.80)
	#A4.AltLocation = 1
	#A4.set_Coord([])
	#A4.set_Occupancy (0.20)
	
	A5 = AtomeObj.Atome('CA')
	A5.set_Coord([])
	A5.set_Occupancy(0.80)
	A5.AltLocation = 1
	A5.set_Coord([])
	A5.set_Occupancy (0.20)
	
	
	listeAtomes=[A1,A2,A3,A4,A5]
	ArbreEssai =  Arbre()
	ConstruireArbreConformationALTLOC (listeAtomes,0, ArbreCourant = ArbreEssai )
	print "Affichage"
	print "Parcours Prefixe"
	PrintArbrePrefixe (ArbreEssai)
	#print ArbreEssai.NumAltLoc
	#print ArbreEssai.ArbresFils
	print "Parcours en Largeur"
	PrintArbreParcoursLargeur (ArbreEssai)
	
	#ListeAltloc = []
	#_ParcoursEnProfondeur_Listes (ArbreEssai, ListeAltloc, listeTemp=[])
	#print ListeAltloc
	
	#GetCheminsPossibles (ArbreEssai)
	ListesAltloc, ListeOccMoy = GetCheminsPossibles (listeAtomes)
	print ListesAltloc
	print ListeOccMoy

	#Arbre 2
	print "Arbre 2"
	A1 = AtomeObj.Atome('CA')
	A1.set_Coord([])
	A1.set_Occupancy(0.80)
	A1.AltLocation = 1
	A1.set_Coord([])
	A1.set_Occupancy (0.20)
	
	A2 = AtomeObj.Atome('CA')
	A2.set_Coord([])
	A2.set_Occupancy(0.80)
	A2.AltLocation = 1
	A2.set_Coord([])
	A2.set_Occupancy (0.20)
	
	A3 = AtomeObj.Atome('CA')
	A3.set_Coord([])
	A3.set_Occupancy(1.0)
	
	A4 = AtomeObj.Atome('CA')
	A4.set_Coord([])
	A4.set_Occupancy(0.80)
	A4.AltLocation = 1
	A4.set_Coord([])
	A4.set_Occupancy (0.20)
	
	A5 = AtomeObj.Atome('CA')
	A5.set_Coord([])
	A5.set_Occupancy(0.80)
	A5.AltLocation = 1
	A5.set_Coord([])
	A5.set_Occupancy (0.20)
	
	
	listeAtomes=[A1,A2,A3,A4,A5]
	ArbreEssai =  Arbre()
	ConstruireArbreConformationALTLOC (listeAtomes,0, ArbreCourant = ArbreEssai )
	print "Affichage"
	print "Parcours Prefixe"
	PrintArbrePrefixe (ArbreEssai)
	#print ArbreEssai.NumAltLoc
	#print ArbreEssai.ArbresFils
	print "Parcours en Largeur"
	PrintArbreParcoursLargeur (ArbreEssai)
	
	#ListeAltloc = []
	#_ParcoursEnProfondeur_Listes (ArbreEssai, ListeAltloc, listeTemp=[])
	#print ListeAltloc
	
	#GetCheminsPossibles (ArbreEssai)
	ListesAltloc, ListeOccMoy = GetCheminsPossibles (listeAtomes)
	print ListesAltloc
	print ListeOccMoy





if __name__ == "__main__":
	main()





