import sys
import Definitions
import ModelObj
import ResiduObj
import GestionAltLoc
import Assignation_Protos
import AssignationPBs
import StructBuilder
import needle
import re
from math import *
import AnglesDiedres
import AssigneSII
import AssigneAccessibilite
import AssigneSII_Stride
import AssigneSII_XTLSSTR
import AssignSII_promotif
import AssignSII_segno
import AssignSII_KAKSI
import Dictionary_rama
import ParseSCOP
from numpy import *
#from rpy import *
#r.library("MASS")	


__doc__= """Module de definir un objet Model structural (gestion des fichiers RMNs ainsi que X avec plusieurs modeles)"""

#-------------------------------------------------------------------------------------------------------------
class AtomeNonDefini (Exception):
	def __init__(self,TypeAtome ):
		print >>sys.stderr, "L'atome %s n'est pas defini"%(TypeAtome )
#-------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------
class ProblemeAlignementSeqetStruct (Exception):
	def __init__(self,Align1, Align2 ):
		print >>sys.stderr, "L'alignement entre la seq SEQRES et la seq ATOM est problematique\n%s\n%s"%(" ".join(Align1)," ".join(Align2))

#-------------------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------------------
class DecalageAnormaleNumerotationStructure (Exception):
	def __init__(self,IDchaine ):
		print >>sys.stderr, "La numerotation de la structure de la chaine %s semble anormale"%(IDchaine)
#-------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------
class DiffSeqLettreETStruct (Exception):
	def __init__(self,IDchaine ):
		print >>sys.stderr, "Les sequences 3lettres et structurales de la chaine %s sont de longueurs differentes apres ajout des breaks"%(IDchaine)

#-------------------------------------------------------------------------------------------------------------



#-------------------------------------------------------------------------------------------------------------
class Model :
	
	def __init__(self):
		"""Initialisation d'un objet modele proteique. Les differents modeles sont classes das une chaine proteique"""
		self.Seq_Struct = []
		self.pdb_struct = []
		self.number_pair = {}
		self.FlagExistanceBreak = 0
		self.ListeIndicesPartiesProt = None
		self.acc_res = {}
		self.res_stat = {}
		self.dssp_stat = {}	
		self.memoConformationsALT_CA = None #de la forme : [ [ [],[] ] ,     [ [],[]   ] ]
										#                    |partie1| break |Partie2  | + au sein des parties:listes des ALTLOC Ca dans l'ordre
										#                                               et classees par occupancy moyenne decroissante
										#Cet attribut n'est pas calcule lors du parsage du PDB. Il est a calcule si utile ensuite
		
		self.memoNombreConformationsALT_CA = [] #On conserve le nombre de conformation alt par partie
		# : si l'arbre des conformations possible est trop grand (> 70 conf alt --> autorisation de 6 fragment alt de 2 conf alt): impossible de tout calculer --> on arrete 
	
		self.list_phi = []
		self.list_psi = []
		self.list_res_std = []	
		self.list_position = []
		self.list_omega = []
		self.poor_BF = 0
		self.rama_dict = {}
		self.count_atoms = 0
	
	def PourcResidusEntiersBBetSC (self):
		"""Methode pour obtenir le pourcentage de residus ayant des atomes manquants SC et ou BB dans le modele. Prend en compte les breaks INTERNES """
		#Pour avoir le numero du dernier residu cristalise
		for i in range(1,len(self.Seq_Struct)):	
			if self.Seq_Struct[0-i].__class__ == ResiduObj.Residu :	
				break
		try:
			NumDernierResCrist = len(self.Seq_Struct)-i
		except UnboundLocalError:
			i = 1
			NumDernierResCrist = len(self.Seq_Struct)-i
	#	print NumDernierResCrist
		#print len(self.Seq_Struct)
		
		#Pour Compter				
		compteRes = 0
		compteResIncomplet = 0
		compteBreakInterne = 0
		count_missingres = 0
		compteRes_missing = 0		
		flag_IntraStruct = 0
		count_res = 0
		for i in range(NumDernierResCrist+1):
			#print i
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu :
				count_res += 1
				if self.Seq_Struct[i].Type in Definitions.AA_standard:
					compteRes += 1
					if not self.Seq_Struct[i].ResiduComplet_all(): #and flag_IntraStruct == 1: #Cassure dans la chaine --> on compte un break
						compteBreakInterne +=1
								
					if not self.Seq_Struct[i].ResiduSCentier(): #si pas de cassure on regarde la chaine
						compteResIncomplet +=1
				#	print self.Seq_Struct[i].Type, self.Seq_Struct[i].PositionDansSeq, i
					#flag_IntraStruct = 1
				
			else: #flag_IntraStruct == 1:
				count_missingres += 1
				#compteBreakInterne +=1
				compteRes_missing += 1	
				#compteRes += 1
		#print compteRes,compteResIncomplet,(float(compteResIncomplet)/float(compteRes) )*100
		return (float(compteBreakInterne)/float(compteRes) )*100, (float(compteBreakInterne)/float(compteRes+compteRes_missing) )*100, (float(compteResIncomplet)/float(compteRes) )*100, compteRes, count_res
	
	#----------------------------------------------------------------------------------------------------------------------
	def getIndicesDebFinPartie (self):
		"""Methode pour obtenir la liste des indices de debut et de fin de chaque partie du modele"""
		ListeDebFin = []
		j=0
		for i in range(len(self.Seq_Struct)):
			#On verifie qu'on a a faire a un residu
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
				if j == 0 :
					deb = i
					#print deb
				if i == len(self.Seq_Struct)-1:
					fin = i
					#print fin
					ListeDebFin.append([deb,fin])
				j += 1
			else :
				if j != 0:				
					fin = i-1
					#print fin
					ListeDebFin.append([deb,fin])
				j=0
				
		self.ListeIndicesPartiesProt=ListeDebFin
				
	#----------------------------------------------------------------------------------------------------------------------
	
	
	
	#----------------------------------------------------------------------------------------------------------------------
	def set_AnglesPhiParRes(self):
		"""Methode pour calculer les angles Phi de chaque residu d'un modele structural"""
		
		#Reinitialisation des Phi
		[res.Del_AnglesPhi () for res in self.Seq_Struct]
		[res.Del_AnglesPhi () for res in self.pdb_struct]
		#Calculs
		j = 0  #suivi des CA dans les listes de conformations alternatives (si besoin)
		PartieProt =0
		for i in range(len(self.Seq_Struct)):
			
			#On verifie qu'on a a faire a un residu
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
				#print self.Seq_Struct[i].Type, self.Seq_Struct[i].PositionDansSeq
				#print "Atome"
				#Debut de la sequence ou Residus absents
				if i == 0  or  self.Seq_Struct[i-1].__class__==ResiduObj.Break or not self.Seq_Struct[i-1].ResiduComplet():
					#print "Phi 999"
					self.Seq_Struct[i].setAnglePhi(999)
					self.pdb_struct[i].setAnglePhi(999)	
				
				else:
					try :
						#si ce modele structural ne possede par de conformation alternatives
						if self.memoConformationsALT_CA  == None :
							CoordAtomCimoins1 = self.Seq_Struct[i-1].Backbone['C'].Coord[0]
							CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
							CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
							CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
							AnglePhi = AnglesDiedres.CalculAngleDiedre (CoordAtomCimoins1,CoordAtomNi,CoordAtomCAi ,CoordAtomCi)
							self.Seq_Struct[i].setAnglePhi(AnglePhi)
							self.pdb_struct[i].setAnglePhi(AnglePhi)
							
						else : #Des conformations alternatives existent
							#Parcours de toutes les conformations dans cette partie de la prot pour verifier que le fragment en cours a bien des conformations diff
							alt = 0
							for k in range(len(self.memoConformationsALT_CA[PartieProt]))[:-1]:
								if self.memoConformationsALT_CA[PartieProt][k][j-1:j+1] != self.memoConformationsALT_CA[PartieProt][k+1][j-1:j+1]  :
									#print self.memoConformationsALT_CA[PartieProt][k][j-1:j+1]
									#print self.memoConformationsALT_CA[PartieProt][k+1][j-1:j+1]
									alt = 1 #deux conformations du fragment se succedant sont differentes --> donc le frag en cours a bien des conformation diff
									break
							if alt == 1:
								#Parcours de toutes les conformations dans cette partie de la prot
								#On considere que les N et les C sont dans les memes altloc que les CA ou n'ont pas d'altloc!!!!
								for conf in self.memoConformationsALT_CA[PartieProt]:
									#print "conf diff:"
									print conf[j]
									if self.Seq_Struct[i-1].Backbone['C'].AltLocation >0:
										CoordAtomCimoins1 = self.Seq_Struct[i-1].Backbone['C'].Coord[conf[j-1]]
									else :
										CoordAtomCimoins1 = self.Seq_Struct[i-1].Backbone['C'].Coord[0]
									if self.Seq_Struct[i].Backbone['N'].AltLocation >0:
										CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[conf[j]]
									else:
										CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
									if self.Seq_Struct[i].Backbone['CA'].AltLocation >0:
										CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
									else :
										CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
									if self.Seq_Struct[i].Backbone['C'].AltLocation >0:
										CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[conf[j]]
									else :
										CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
									#print CoordAtomCimoins1
									#print CoordAtomNi
									#print CoordAtomCAi
									#print CoordAtomCi
									AnglePhi = AnglesDiedres.CalculAngleDiedre (CoordAtomCimoins1,CoordAtomNi,CoordAtomCAi ,CoordAtomCi)
									#print AngleAlpha
									self.Seq_Struct[i].setAnglePhi(AnglePhi)
									self.pdb_struct[i].setAnglePhi(AnglePhi)
							else : #Toutes les conformation de ce fragment sont les memes (la conformation alternative de cette partie de la prot ne touche pas ce fragment)
								conf = self.memoConformationsALT_CA[PartieProt][0]
								if self.Seq_Struct[i-1].Backbone['C'].AltLocation >0:
									CoordAtomCimoins1 = self.Seq_Struct[i-1].Backbone['C'].Coord[conf[j-1]]
								else :
									CoordAtomCimoins1 = self.Seq_Struct[i-1].Backbone['C'].Coord[0]
								if self.Seq_Struct[i].Backbone['N'].AltLocation >0:
									CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[conf[j]]
								else:
									CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
								if self.Seq_Struct[i].Backbone['CA'].AltLocation >0:
									CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
								else :
									CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
								if self.Seq_Struct[i].Backbone['C'].AltLocation >0:
									CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[conf[j]]
								else :
									CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
								#print CoordAtomCimoins1
								#print CoordAtomNi
								#print CoordAtomCAi
								#print CoordAtomCi
								AnglePhi = AnglesDiedres.CalculAngleDiedre (CoordAtomCimoins1,CoordAtomNi,CoordAtomCAi ,CoordAtomCi)
								#print AngleAlpha
								self.Seq_Struct[i].setAnglePhi(AnglePhi)
								self.pdb_struct[i].setAnglePhi(AnglePhi)
					except KeyError:
						#Residu courant (ou le precedent) incomplets 
						self.Seq_Struct[i].setAnglePhi(999)
						self.pdb_struct[i].setAnglePhi(999)
				j+= 1
				
				
			else:
				#Break
				#print "Break"
				if j != 0:				
					PartieProt += 1
				j=0
				self.Seq_Struct[i].setAnglePhi(999)
				self.pdb_struct[i].setAnglePhi(999)
			self.list_phi.append(self.Seq_Struct[i].AnglesPhiPsi['Phi'])
		return self.list_phi
	#----------------------------------------------------------------------------------------------------------------------
	
	#----------------------------------------------------------------------------------------------------------------------
	def set_AnglesPsiParRes(self):
		"""Methode pour calculer les angles Psi de chaque residu d'un modele structural"""
		#Reinitialisation des Psi
		[res.Del_AnglesPsi () for res in self.Seq_Struct]
		[res.Del_AnglesPsi () for res in self.pdb_struct]
		
		#Calculs
		j = 0  #suivi des CA dans les listes de conformations alternatives (si besoin)
		PartieProt =0
		for i in range(len(self.Seq_Struct)):
			
			#On verifie qu'on a a faire a un residu
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
				#print "Atome"
				#Fin de la sequence ou Residus absents
				self.list_res_std.append(self.Seq_Struct[i].Type)
				self.list_position.append(self.Seq_Struct[i].PositionDansSeq)
				if i==len(self.Seq_Struct)-1 or self.Seq_Struct[i+1].__class__==ResiduObj.Break or not self.Seq_Struct[i+1].ResiduComplet():
					#print "Psi 999"
					self.Seq_Struct[i].setAnglePsi(999)
					self.pdb_struct[i].setAnglePsi(999)
				
				else:
					try :
						#si ce modele structural ne possede par de conformation alternatives
						if self.memoConformationsALT_CA  == None :
							
							CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
							CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
							CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
							CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[0]
							AnglePsi = AnglesDiedres.CalculAngleDiedre (CoordAtomNi,CoordAtomCAi ,CoordAtomCi, CoordAtomNiplusUn )
							self.Seq_Struct[i].setAnglePsi(AnglePsi)
							self.pdb_struct[i].setAnglePsi(AnglePsi)
							
						else : #Des conformations alternatives existent
							#Parcours de toutes les conformations dans cette partie de la prot pour verifier que le fragment en cours a bien des conformations diff
							alt = 0
							for k in range(len(self.memoConformationsALT_CA[PartieProt]))[:-1]:
								if self.memoConformationsALT_CA[PartieProt][k][j:j+1+1] != self.memoConformationsALT_CA[PartieProt][k+1][j:j+1+1]  :
									alt = 1 #deux conformations du fragment se succedant sont differentes --> donc le frag en cours a bien des conformation diff
									break
							if alt == 1:	
								#Parcours de toutes les conformations dans cette partie de la prot
								for conf in self.memoConformationsALT_CA[PartieProt]:
									#print "conf diff:"
									if self.Seq_Struct[i].Backbone['N'].AltLocation >0:
										CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[conf[j]]
									else:
										CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
									if self.Seq_Struct[i].Backbone['CA'].AltLocation >0:
										CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
									else :
										CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
									if self.Seq_Struct[i].Backbone['C'].AltLocation >0:
										CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[conf[j]]
									else :
										CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
									if self.Seq_Struct[i+1].Backbone['N'].AltLocation >0:
										CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[conf[j+1]]
									else :
										CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[0]
									#print CoordAtomNi
									#print CoordAtomCAi
									#print CoordAtomCi
									#print CoordAtomNiplusUn
									AnglePsi = AnglesDiedres.CalculAngleDiedre (CoordAtomNi,CoordAtomCAi ,CoordAtomCi,CoordAtomNiplusUn)
									#print AngleAlpha
									self.Seq_Struct[i].setAnglePsi(AnglePsi)
									self.pdb_struct[i].setAnglePsi(AnglePsi)
							
							else : #Toutes les conformation de ce fragment sont les memes (la conformation alternative de cette partie de la prot ne touche pas ce fragment)
								conf = self.memoConformationsALT_CA[PartieProt][0]
								#print "conf diff:"
								if self.Seq_Struct[i].Backbone['N'].AltLocation >0:
									CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[conf[j]]
								else:
									CoordAtomNi = self.Seq_Struct[i].Backbone['N'].Coord[0]
								if self.Seq_Struct[i].Backbone['CA'].AltLocation >0:
									CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
								else :
									CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
								if self.Seq_Struct[i].Backbone['C'].AltLocation >0:
									CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[conf[j]]
								else :
									CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
								if self.Seq_Struct[i+1].Backbone['N'].AltLocation >0:
									CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[conf[j+1]]
								else :
									CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[0]
								#print CoordAtomNi
								#print CoordAtomCAi
								#print CoordAtomCi
								#print CoordAtomNiplusUn
								AnglePsi = AnglesDiedres.CalculAngleDiedre (CoordAtomNi,CoordAtomCAi ,CoordAtomCi,CoordAtomNiplusUn)
								#print AngleAlpha
								self.Seq_Struct[i].setAnglePsi(AnglePsi)
								self.pdb_struct[i].setAnglePsi(AnglePsi)
					except KeyError:
						#Residu courant (ou le precedent) incomplets 
						self.Seq_Struct[i].setAnglePsi(999)
						self.pdb_struct[i].setAnglePsi(999)
				j+= 1
				
				
			else:
				#Break ou residu incomplet
				#print "Break"
				if j != 0:				
					PartieProt += 1
				j=0
				self.Seq_Struct[i].setAnglePsi(999)
				self.pdb_struct[i].setAnglePsi(999)
			self.list_psi.append(self.Seq_Struct[i].AnglesPhiPsi['Psi'])	
		#return self.list_psi
	#----------------------------------------------------------------------------------------------------------------------
	
	def calculate_omega(self):
		
		[res.Del_AngleOmega for res in self.Seq_Struct]
		[res.Del_AngleOmega for res in self.pdb_struct]

                for i in range(len(self.Seq_Struct)):
                        if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
                                if i==len(self.Seq_Struct)-1 or self.Seq_Struct[i+1].__class__==ResiduObj.Break or not self.Seq_Struct[i+1].ResiduComplet():
					AngleOmega = 999
                                        self.Seq_Struct[i].setAngleOmega(AngleOmega)
					self.pdb_struct[i].setAngleOmega(AngleOmega)
                                else:
                                	if self.memoConformationsALT_CA  == None :
                                		CoordAtomCAi = self.Seq_Struct[i].Backbone['CA'].Coord[0]
                                                CoordAtomCi = self.Seq_Struct[i].Backbone['C'].Coord[0]
                                                CoordAtomNiplusUn = self.Seq_Struct[i+1].Backbone['N'].Coord[0]
                                                CoordAtomCAiplusUn = self.Seq_Struct[i+1].Backbone['CA'].Coord[0]
                                                AngleOmega = AnglesDiedres.CalculAngleDiedre (CoordAtomCAi,CoordAtomCi ,CoordAtomNiplusUn, CoordAtomCAiplusUn )
                                                self.Seq_Struct[i].setAngleOmega(AngleOmega)
						self.pdb_struct[i].setAngleOmega(AngleOmega)
			else:
				AngleOmega = 999
				self.Seq_Struct[i].setAngleOmega(AngleOmega)
				self.pdb_struct[i].setAngleOmega(AngleOmega)
			self.list_omega.append(AngleOmega)
		#print self.list_omega
		#return self.list_omega

	def ramachandran_outliers (self):
		list_phi_psi = []
		count_allowed = 0
		count_favored = 0
		count_disallowed = 0
		count = 0
		count_std = 0
		list_res = []
		list_res = self.list_res_std
		dict_rama_general = Dictionary_rama.dict_lovell_general
		dict_rama_gly = Dictionary_rama.dict_lovell_gly
		dict_rama_pro = Dictionary_rama.dict_lovell_pro
		dict_rama_prepro = Dictionary_rama.dict_lovell_prepro
                if len(self.list_phi) == len(self.list_psi):
                        pass
#                       for i in range(len(self.list_phi)):     
#                               list_phi_psi.append((self.list_phi[i],self.list_psi[i]))
                else:
                        print "no of phi diff from no of psi"		
		for k in range(len(self.Seq_Struct)): 
			flag_last = 0
                	if self.Seq_Struct[k].__class__ == ResiduObj.Residu and self.Seq_Struct[k].ResiduComplet():
				list_phi_psi.append((self.Seq_Struct[k].getAnglesPhi(0),self.Seq_Struct[k].getAnglesPsi(0)))
				phi = self.Seq_Struct[k].getAnglesPhi(0)
				psi = self.Seq_Struct[k].getAnglesPsi(0)	
				res = self.Seq_Struct[k].Type
				position = self.Seq_Struct[k].PositionDansSeq
				for (i,j) in dict_rama_general.keys():
					if phi >= i-1 and phi < i+1 and psi >= j-1 and psi < j+1:
						if k == len(self.Seq_Struct)-1:
							flag_last = 1
						if self.Seq_Struct[k+1].PositionDansSeq - position == 1 and self.Seq_Struct[k+1].Type == 'PRO' and flag_last != 1:
                                                        if dict_rama_prepro[i,j] >= 0.02:
                                                                count_favored += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'F'
                                                        elif dict_rama_prepro[i,j] < 0.02 and dict_rama_prepro[i,j] > 0.002:
                                                                count_allowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'A'
                                                        else:
                                                                count_disallowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'D'
						elif res != 'PRO' or res != 'GLY':
							if dict_rama_general[i,j] >= 0.02:
								count_favored += 1
								self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'F'
							elif dict_rama_general[i,j] < 0.02 and dict_rama_general[i,j] > 0.0005:
								count_allowed += 1		
								self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'A'			
					 		else:
								count_disallowed += 1	
								self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'D'
						elif res == 'GLY':
                                                        if dict_rama_gly[i,j] >= 0.02:
                                                                count_favored += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'F'
                                                        elif dict_rama_gly[i,j] < 0.02 and dict_rama_gly[i,j] > 0.002:
                                                                count_allowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'A'        
                                                        else:
                                                                count_disallowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'D'

						elif res == 'PRO':
                                                        if dict_rama_pro[i,j] >= 0.02:
                                                                count_favored += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'F'
                                                        elif dict_rama_pro[i,j] < 0.02 and dict_rama_pro[i,j] > 0.002:
                                                                count_allowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'A'        
                                                        else:
                                                                count_disallowed += 1
                                                                self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'D'
				count += 1
			if not self.rama_dict.has_key(self.pdb_struct[k].PositionDansSeq):	
				self.rama_dict[self.pdb_struct[k].PositionDansSeq] = 'X'
		return [count_disallowed, count_allowed, count_favored, count]	

	def ramachandran_plot (self):
		scatter_phi = []
		scatter_psi = []
		STEP=2
		HALF_STEP=1
		STEP = HALF_STEP*2
		lower_bounds = range(-180, 180, STEP)
		mid_points = range(-180+HALF_STEP, 180+HALF_STEP, STEP)
		upper_bounds = range(-180+STEP, 180+STEP, STEP)

		data = array([[0.0 for x in mid_points] for y in mid_points], 'f')
		#print data

		print "Creating R function",
		r("""
		ramachandran.plot <- function(x.scatter, y.scatter,
		    x.grid = seq(0, 1, len = nrow(z)), y.grid = seq(0, 1, len = ncol(z)), z.grid,
		    xlim = range(x.grid, finite = TRUE), ylim = range(y.grid, finite = TRUE),
		    zlim = range(z.grid, finite = TRUE), levels = pretty(zlim, nlevels),
		    nlevels = 20, color.palette = cm.colors, col = color.palette(length(levels) -
		        1), plot.title="", plot.axes, key.title, key.axes, asp = NA,
		    xaxs = "i", yaxs = "i", las = 1, axes = TRUE, frame.plot = axes,
		    ...)
		{
		    if (missing(z.grid)) {
		        stop("no 'z.grid' matrix specified")
		    }
		    else if (is.list(x.grid)) {
		        y.grid <- x.grid$y
		        x.grid <- x.grid$x
		    }
		    if (any(diff(x.grid) <= 0) || any(diff(y.grid) <= 0))
		        stop("increasing 'x.grid' and 'y.grid' values expected")
		
		    plot.new()
		    plot.window(xlim, ylim, "", xaxs = xaxs, yaxs = yaxs, asp = asp)
		    if (!is.matrix(z.grid) || nrow(z.grid) <= 1 || ncol(z.grid) <= 1)
		        stop("no proper 'z.grid' matrix specified")
		    if (!is.double(z.grid))
		        storage.mode(z.grid) <- "double"
		    .Internal(filledcontour(as.double(x.grid), as.double(y.grid), z.grid, as.double(levels),
		        col = col))

		    if (!(missing(x.scatter)) && !(missing(y.scatter))) {
		        plot.xy(xy.coords(x.scatter,y.scatter,NULL,NULL,NULL,NULL),
	                xlim=xlim, ylim=ylim, xlab="", ylab="", asp=asp,
	                type="p", pch=20, cex=0.1)
		    }
        
		    if (missing(plot.axes)) {
		        if (axes) {
		            title(main=plot.title, xlab=expression(phi), ylab=expression(psi))
		            axis(1, at=c(-180,-90,0,90,180))
		            axis(2, at=c(-180,-90,0,90,180))
		        }
		    }
		    else plot.axes
		    #if (frame.plot)
		    #    box()
		    #if (missing(plot.title))
		    #    title(...)
		    #else plot.title
		    #invisible()
		}
		""")

		#r.par(mfrow=[2,2])
		for (i,j) in Dictionary_rama.dict_lovell_general.keys():
			i1, i2 = mid_points.index(int(i)), mid_points.index(int(j))
			data[i1,i2] = Dictionary_rama.dict_lovell_general[i,j]
		#print data.shape
		#print data
		rama_levels = [0, 0.0005, 0.02, 1]
		rama_colors = ['#FFFFFF','#B3E8FF','#7FD9FF']
		#for (i,ramachandran_type) in enumerate(ramachandran_types) :
 	 	#pdf_filename = "../%s_%s.pdf" % (pdb, ramachandran_type)
	    	#(rama_levels, rama_colors, rama_filename) = rama_settings[ramachandran_type]

	    	#print "Loading data file: " + rama_filename,
	    	#data, lower_bounds, mid_points, upper_bounds = load_data_file(rama_filename)
		
		#print "Creating PDF output file: " + pdf_filename,
		#r.pdf(pdf_filename)
    		#r.plot(scatter_phi, scatter_psi)

    		#print "Generating quadrant %i, %s" % (i+1, ramachandran_type)
    		#r.screen(i+1)

    		#Use small margins to make the plots nice and big,
   	 	#and specify a SQUARE plot area (to go with aspect ratio, asp=1)
		count = 0
		#print self.list_res_std
		for res in self.list_res_std:
			if count < len(self.list_res_std[:-1]):
                		if self.list_position[count+1] - self.list_position[count] == 1 and self.list_res_std[count+1] == 'PRO':
                			pass
                		elif res != 'PRO' or res != 'GLY':
					scatter_phi.append(self.list_phi[count]) 
					scatter_psi.append(self.list_psi[count])
			else:
				if res != 'PRO' or res != 'GLY':
					scatter_phi.append(self.list_phi[count])
					scatter_psi.append(self.list_psi[count])
			count += 1
    		#r.par(mar = [2, 2, 2, 2], pty="s")
		r.png(file = "test.png")	
    		#This function will do a Ramachandran plot in the next quadrant
    		#which we setup using par(mfrow-...)
    		r.ramachandran_plot(x_scatter=scatter_phi,
                        y_scatter=scatter_psi,
                        x_grid=mid_points, y_grid=mid_points, z_grid=data,
                        xlim=[-180,180], ylim=[-180,180], asp=1.0,
                        plot_title="general", drawlabels=False,
                        levels=rama_levels, col=rama_colors)
    		#print ramachandran_type + " Done"

		r.dev_off()
		print "Done"
	
	#----------------------------------------------------------------------------------------------------------------------
	def set_AnglesAlphasParRes (self) :
		"""Methode pour calculer les angles diedres ALPHAs d'un modele structural"""
		
		#Reinitialisation des angles alphas
		[res.Del_AngleAlpha() for res in self.Seq_Struct]
		[res.Del_AngleAlpha() for res in self.pdb_struct]
				
		#Calcul
		j = 0  #suivi des CA dans les listes de conformations alternatives (si besoin)
		PartieProt =0
		for i in range(len(self.Seq_Struct)):
			#On verifie qu'on a a faire a un residu
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu  and self.Seq_Struct[i].ResiduComplet():
				#print "Atome"
				#print i
				#print self.Seq_Struct[i-1].ResiduComplet() and self.Seq_Struct[i+1].ResiduComplet() and self.Seq_Struct[i+2].ResiduComplet() 
				#Debut et fin de la sequence
				if i == 0 or i== len(self.Seq_Struct)-2 or i== len(self.Seq_Struct)-1 :
					#print "999"
					self.Seq_Struct[i].setAngleAlpha(999)
					self.pdb_struct[i].setAngleAlpha(999)
				#Residus absents
				elif self.Seq_Struct[i-1].__class__==ResiduObj.Break or self.Seq_Struct[i+1].__class__==ResiduObj.Break or self.Seq_Struct[i+2].__class__==ResiduObj.Break:
					#print "999"
					self.Seq_Struct[i].setAngleAlpha(999)
					self.pdb_struct[i].setAngleAlpha(999)
				#Residus du fragment : incomplets
				elif not (self.Seq_Struct[i-1].ResiduComplet() and self.Seq_Struct[i+1].ResiduComplet() and self.Seq_Struct[i+2].ResiduComplet() ):
					self.Seq_Struct[i].setAngleAlpha(999)
					self.pdb_struct[i].setAngleAlpha(999)
				else:
					try :
						#si ce modele structural ne possede par de conformation alternatives
						if self.memoConformationsALT_CA  == None :
							CoordAtomA = self.Seq_Struct[i-1].Backbone['CA'].Coord[0]
							CoordAtomB = self.Seq_Struct[i].Backbone['CA'].Coord[0]
							CoordAtomC = self.Seq_Struct[i+1].Backbone['CA'].Coord[0]
							CoordAtomD = self.Seq_Struct[i+2].Backbone['CA'].Coord[0]
							AngleAlpha = AnglesDiedres.CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD)
							self.Seq_Struct[i].setAngleAlpha(AngleAlpha)
							self.pdb_struct[i].setAngleAlpha(AngleAlpha)
						else : #Des conformations alternatives existent
							#Parcours de toutes les conformations dans cette partie de la prot pour verifier que le fragment en cours a bien des conformations diff
							alt = 0
							for k in range(len(self.memoConformationsALT_CA[PartieProt]))[:-1]:
								if self.memoConformationsALT_CA[PartieProt][k][j-1:j+2+1] != self.memoConformationsALT_CA[PartieProt][k+1][j-1:j+2+1]  :
									alt = 1 #deux conformations du fragment se succedant sont differentes --> donc le frag en cours a bien des conformation diff
									break
							if alt == 1:	
								#Parcours de toutes les conformations dans cette partie de la prot
								for conf in self.memoConformationsALT_CA[PartieProt]:
									#print "conf diff:"
									CoordAtomA = self.Seq_Struct[i-1].Backbone['CA'].Coord[conf[j-1]]
									CoordAtomB = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
									CoordAtomC = self.Seq_Struct[i+1].Backbone['CA'].Coord[conf[j+1]]
									CoordAtomD = self.Seq_Struct[i+2].Backbone['CA'].Coord[conf[j+2]]
									#print CoordAtomA
									#print CoordAtomB
									#print CoordAtomC
									#print CoordAtomD
									AngleAlpha = AnglesDiedres.CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD)
									#print AngleAlpha
									self.Seq_Struct[i].setAngleAlpha(AngleAlpha)
									self.pdb_struct[i].setAngleAlpha(AngleAlpha)
							else : #Toutes les conformation de ce fragment sont les memes (la conformation alternative de cette partie de la prot ne touche pas ce fragment)
								conf = self.memoConformationsALT_CA[PartieProt][0]
								CoordAtomA = self.Seq_Struct[i-1].Backbone['CA'].Coord[conf[j-1]]
								CoordAtomB = self.Seq_Struct[i].Backbone['CA'].Coord[conf[j]]
								CoordAtomC = self.Seq_Struct[i+1].Backbone['CA'].Coord[conf[j+1]]
								CoordAtomD = self.Seq_Struct[i+2].Backbone['CA'].Coord[conf[j+2]]
								#print CoordAtomA
								#print CoordAtomB
								#print CoordAtomC
								#print CoordAtomD
								AngleAlpha = AnglesDiedres.CalculAngleDiedre (CoordAtomA, CoordAtomB,CoordAtomC,CoordAtomD)
								#print AngleAlpha
								self.Seq_Struct[i].setAngleAlpha(AngleAlpha)
								self.pdb_struct[i].setAngleAlpha(AngleAlpha)
								
								
					except KeyError:
						#Absence de l'atome Ca
						self.Seq_Struct[i].setAngleAlpha(999)
						self.pdb_struct[i].setAngleAlpha(999)
				j+= 1
			else : #Break ou residu incomplet
				#print "Break"
				if j != 0:				
					PartieProt += 1
				j=0
				self.Seq_Struct[i].setAngleAlpha(999)
				self.pdb_struct[i].setAngleAlpha(999)
	#----------------------------------------------------------------------------------------------------------------------	
		
	#----------------------------------------------------------------------------------------------------------------------
	def set_ProtoParRes (self, RepertoireProtos):
		"""Methode pour calculer l'assignation en Prototypes pour toute le model proteique"""
		#print "set_ProtoParRes"
		
		#Reinitialisation des Protos
		[res.Del_Proto () for res in self.Seq_Struct]
		[res.Del_Proto () for res in self.pdb_struct]
		
		
		#print "set_ProtoParRes apres reinitialisation"
		#print  [res.Proto for res in self.Seq_Struct]
		
				
		#Calculs
		
		#Chargement des coordonnees des 120 protos
		ListeProtos = []
		for i in range(0,120):
			#Lecture du PDB du proto
			ProtoBuilder = StructBuilder.StructBuilder(RepertoireProtos+"/Proto_Pos"+str(i)+".pdb",verbose=0)
			ProtoBuilder.Build_Chains(verbose=0)
			ProtoBuilder.Proteine.add_Chaine ('A')
			Proto = ProtoBuilder.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct
			ListeProtos.append(Proto)
		#print "ListeProtos=",ListeProtos
		
		j = 0  #suivi des CA dans les listes de conformations alternatives (si besoin)
		PartieProt =0
		for i in range(len(self.Seq_Struct)):
			#print self.Seq_Struct[i].Type, self.Seq_Struct[i].PositionDansSeq
			#print [self.Seq_Struct[k].Type for k in range(i-5,i+5+1)]
			#print [self.Seq_Struct[k].PositionDansSeq for k in range(i-5,i+5+1)]
			#print [self.Seq_Struct[k].ResiduComplet() for k in range(i-5,i+5+1)]
			#On verifie qu'on a a faire a un residu et qu'il est complet 
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
				#print "Atome"
#				#Que contient le fragment ?:
#				frag_classes = [self.Seq_Struct[k].__class__  for k in range(i-5,i+5)]
#				print frag_classes
				
				#Debut et fin de la sequence
				if i in range(0,5) or i in range(len(self.Seq_Struct)-5,len(self.Seq_Struct)) :
					#print "999"
					#print "i deb fin = ",i
					self.Seq_Struct[i].setProto (999)
					self.pdb_struct[i].setProto(999)
				#Residus absents
				elif ResiduObj.Break in [self.Seq_Struct[k].__class__  for k in range(i-5,i+5+1)]:
					#print "999"
					#print "i break =",i
					self.Seq_Struct[i].setProto(999)
					self.pdb_struct[i].setProto(999)
				#Residus incomplets dans le fragment
				
				elif False in [self.Seq_Struct[k].ResiduComplet() for k in range(i-5,i+5+1)]:
					self.Seq_Struct[i].setProto(999)
					self.pdb_struct[i].setProto(999)
				else:
					
					try :
						
						#print "i=",i
						#si ce modele structural ne possede par de conformation alternatives
						if self.memoConformationsALT_CA  == None :
							ProtoGagnant = Assignation_Protos.AssignationProto (self.Seq_Struct[i-5:i+5+1],[0]*11, ListeProtos)
							self.Seq_Struct[i].setProto(ProtoGagnant )
							self.pdb_struct[i].setProto(ProtoGagnant )
							
						else : #Des conformations alternatives existent
							#Parcours de toutes les conformations dans cette partie de la prot pour verifier que le fragment en cours a bien des conformations diff
							alt = 0
							#print self.memoConformationsALT_CA[PartieProt]
							#print self.Seq_Struct[i].Type , self.Seq_Struct[i].PositionDansSeq
							for k in range(len(self.memoConformationsALT_CA[PartieProt]))[:-1]:
								#print self.memoConformationsALT_CA[PartieProt][k][j-5:j+5+1],self.memoConformationsALT_CA[PartieProt][k+1][j-5:j+5+1]
								if self.memoConformationsALT_CA[PartieProt][k][j-5:j+5+1] != self.memoConformationsALT_CA[PartieProt][k+1][j-5:j+5+1]  :
									alt = 1 #deux conformations du fragment se succedant sont differentes --> donc le frag en cours a bien des conformation diff
									break
							if alt == 1:	
								#Parcours de toutes les conformations dans cette partie de la prot
								for conf in self.memoConformationsALT_CA[PartieProt]:
									#print 'seq',len(self.Seq_Struct),'conf',len(conf)
									#print "conf diff:"
									#print self.Seq_Struct[i-5:i+5+1]
									#print conf[j-5:j+5+1]
									ProtoGagnant = Assignation_Protos.AssignationProto (self.Seq_Struct[i-5:i+5+1],conf[j-5:j+5+1], ListeProtos)
									self.Seq_Struct[i].setProto(ProtoGagnant )
									self.pdb_struct[i].setProto(ProtoGagnant )	
							else : #Toutes les conformation de ce fragment sont les memes (la conformation alternative de cette partie de la prot ne touche pas ce fragment)
								conf = self.memoConformationsALT_CA[PartieProt][0]
								#print "j",j
								#print conf[j-5:j+5+1]
								ProtoGagnant = Assignation_Protos.AssignationProto (self.Seq_Struct[i-5:i+5+1],conf[j-5:j+5+1], ListeProtos)
								self.Seq_Struct[i].setProto(ProtoGagnant )
								self.pdb_struct[i].setProto(ProtoGagnant )
					except KeyError:
						#Absence de l'atome Ca
						self.Seq_Struct[i].setProto(999)
						self.pdb_struct[i].setProto(999)
				j+= 1
			else : #Break
				#print "Break"
				if j != 0:				
					PartieProt += 1
				j=0
				self.Seq_Struct[i].setProto(999)
				self.pdb_struct[i].setProto(999)
	#----------------------------------------------------------------------------------------------------------------------

	#----------------------------------------------------------------------------------------------------------------------
	def set_PBsParRes (self, NomFichierAnglesPBs):
		"""Methode pour calculer l'assignation en PBs pour tout le model proteique"""
		
		#Reinitialisation des Protos
		[res.Del_PB () for res in self.Seq_Struct]
		[res.Del_PB () for res in self.pdb_struct]
		
		#Calculs
		j = 0  #suivi des CA dans les listes de conformations alternatives (si besoin)
		PartieProt =0
		for i in range(len(self.Seq_Struct)):
			#print self.Seq_Struct[i].Type, self.Seq_Struct[i].PositionDansSeq
			#print [self.Seq_Struct[k].Type for k in range(i-2,i+2)]
			#print [self.Seq_Struct[k].PositionDansSeq for k in range(i-2,i+2+1)]
			#print [self.Seq_Struct[k].ResiduComplet() for k in range(i-2,i+2+1)]
			#On verifie qu'on a a faire a un residu et qu'il est complet
			if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet():
				#Debut et fin de la sequence
				if i in range(0,2) or i in range(len(self.Seq_Struct)-2,len(self.Seq_Struct)) :
					#print "999"
					#print "i deb fin = ",i
					self.Seq_Struct[i].setPB (999)
					self.pdb_struct[i].setPB (999)
				#Residus absents
				elif ResiduObj.Break in [self.Seq_Struct[k].__class__  for k in range(i-2,i+2+1)]:
					#print "999"
					#print "i break =",i
					self.Seq_Struct[i].setPB(999)
					self.pdb_struct[i].setPB (999)
				#Residus incomplets dans le fragment
				elif False in [self.Seq_Struct[k].ResiduComplet() for k in range(i-2,i+2+1)]:
					self.Seq_Struct[i].setPB(999)
					self.pdb_struct[i].setPB (999)
				else:
					
					#print "i=",i
					#si ce modele structural ne possede par de conformation alternatives
					if self.memoConformationsALT_CA  == None :
						PBGagnant = AssignationPBs.Assignation_PBs(self.Seq_Struct[i-2:i+2+1],[0]*5, NomFichierAnglesPBs)
						self.Seq_Struct[i].setPB(PBGagnant )
						self.pdb_struct[i].setPB (PBGagnant)
						
					else : #Des conformations alternatives existent
						#Parcours de toutes les conformations dans cette partie de la prot pour verifier que le fragment en cours a bien des conformations diff
						alt = 0
						#print self.memoConformationsALT_CA[PartieProt]
						for k in range(len(self.memoConformationsALT_CA[PartieProt]))[:-1]:
							if self.memoConformationsALT_CA[PartieProt][k][j-2:j+2+1] != self.memoConformationsALT_CA[PartieProt][k+1][j-2:j+2+1]  :
								alt = 1 #deux conformations du fragment se succedant sont differentes --> donc le frag en cours a bien des conformation diff
								break
						if alt == 1:	
							#Parcours de toutes les conformations dans cette partie de la prot
							for conf in self.memoConformationsALT_CA[PartieProt]:
								#print "conf diff:"
								PBGagnant = AssignationPBs.Assignation_PBs (self.Seq_Struct[i-2:i+2+1],conf[j-2:j+2+1], NomFichierAnglesPBs)
								self.Seq_Struct[i].setPB(PBGagnant )
								self.pdb_struct[i].setPB (PBGagnant)
								if PBGagnant == 999: break #le fragment comprend au moins un residu incomplet
						else : #Toutes les conformation de ce fragment sont les memes (la conformation alternative de cette partie de la prot ne touche pas ce fragment)
							conf = self.memoConformationsALT_CA[PartieProt][0]
							PBGagnant = AssignationPBs.Assignation_PBs(self.Seq_Struct[i-2:i+2+1],[0]*5, NomFichierAnglesPBs)
							self.Seq_Struct[i].setPB(PBGagnant )
							self.pdb_struct[i].setPB (PBGagnant)
			
				j+= 1
			else : #Break
				#print "Break"
				if j != 0:				
					PartieProt += 1
				j=0
				self.Seq_Struct[i].setPB(999)
				self.pdb_struct[i].setPB (999)
	#----------------------------------------------------------------------------------------------------------------------

	def write_pdb_chains(self, RepertoirePDB, pdb_id, model_number, chain_id, lines_seqres):
		char_chain = chain_id
		if chain_id == " ":
			char_chain = "_"
		pdb_file_name = RepertoirePDB+"/"+pdb_id+"_"+`model_number`+"_"+chain_id+".pdb"
		pdb = open(pdb_file_name, "w")
		pdb.writelines(lines_seqres)
		LignesPDB = []
		NumAtom = [0]
		for i in range(len(self.pdb_struct)):
			self.pdb_struct[i].ResiduTOListOfString_PDB (NumAtom,chain_id ,LignesPDB, None, None)
		LignesPDB.append("TER\n\n")
		pdb.writelines(LignesPDB)
		pdb.close()

	def res_summary (self, pdb_id, exp, resln, r_factor, model, chain, scop, scopid_list, folder):
		file_name = folder+"/"+pdb_id+"_"+`model`+"_"+chain+"_"+".sum"
		print scop
		scop_class = ""
		id_list = ""
		if len(scop) != 0:
			for keys in scop:
				scop_class += keys + ":" + `scop[keys][0]`+ ","+`scop[keys][1]`+","
			for ids in scopid_list:
				id_list += ids + ","
			scop_class = scop_class[:-1]
			id_list = id_list[:-1]
		else :
			scop_class = " "
			id_list = " "	
#		print scop_class
#		print id_list
		bfactor_list = self.norm_bfactor()
		norm_b_factor = bfactor_list[0]
		avg_b_factor = bfactor_list[1]
		file = open(file_name, "w")
		file.write("l1:pdb ID, l2: exptl method,l3:resolution,l4:rfactor,l5:scop class,l6:scop ID list\n")
		file.write("#%-4s\n#%-3s\n#%-6.3f\n#%-5.3f\n#%s\n#%s\n"%(pdb_id, exp, resln, r_factor, scop_class, id_list))
		file.write("\n\nres: residue type\npos:position of residue\nPB:Protein Block assignment\nProt:Prototype assignment\nDSSP:DSSP assignment\nNACCESS:NACCESS relative surface accessibility\nNorm_BF: Normalised B-Factor\nAvg_BF: B_factor avg for backbone atoms\nAvg_occ: Occupancy avg for backbone atoms\nPhi: Phi dihedral angle\nPsi:Psi dihedral angle\nOmega:Omega dihedral angle\nRC:Allowed, Favored and Disallowed (based on Lovell et al)\n\n\n\nRes pos   PB  DSSP Stride XTLSSTR PMOTIF  SEG KAKSI NACCESS Norm_BF Avg_BF  Avg_occ Phi      Psi      Omega    RC\n")	
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:# and res.ResiduComplet():
#				print res.Type
#				print res.PositionDansSeq
#				print res.getPB()
#				print res.getDSSP()
#				print res.getaccess()
#				print norm_b_factor[res.PositionDansSeq]
#				print avg_b_factor[res.PositionDansSeq]
#				print res.avg_occup()
#				print res.getAnglesPhi()
#				print res.getAnglesPsi()
#				print res.getAngleomega()
#				print self.rama_dict[res.PositionDansSeq]
				line = "%-3s %-5s %-3s  %-2s  %-4s  %-4s %-12s %2s %3s  %7.4f %7.4f %8.4f %8.4f %8.4f %8.4f %8.4f %2s\n"%(res.Type, res.PositionDansSeq, res.getPB(), res.getDSSP(), res.getStride(), res.getXtl(), res.getPM(), res.getSegno(), res.getKaksi(), res.getaccess(), norm_b_factor[res.PositionDansSeq], avg_b_factor[res.PositionDansSeq], res.avg_occup(), res.getAnglesPhi(), res.getAnglesPsi(), res.getAngleomega(), self.rama_dict[res.PositionDansSeq])
				file.writelines(line)		
		file.close()

	def calpha_only(self):
		count = 0
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:
				if res.ResiduComplet():
					count += 1
		if count < 5:
			return True
		else: return False

	def residue_stat(self):
		dssp_assign = []
		res_ss = {}
		dssp_prev = "#"
                for res in self.pdb_struct:
                        if res.__class__ == ResiduObj.Residu:
				
  				if res.Type in Definitions.AA_standard:
					index = Definitions.AA_standard.index(res.Type)
					try:
						self.res_stat[res.Type] += 1 
					except KeyError:
						self.res_stat[res.Type] = 1
				if res.getaccess() > 7:
					accessibility = 'A'
					try:
						self.acc_res['A'].append(res.Type)
					except KeyError:
						self.acc_res['A'] = [res.Type]
				elif res.getaccess() < 7: 
					accessibility = 'B'
					try:
						self.acc_res['B'].append(res.Type)
					except KeyError:
						self.acc_res['B'] = [res.Type] 
									
				if res.getDSSP() in dssp_assign:
					if res.getDSSP() != dssp_prev:
						try:
							count_ss[res.getDSSP()] += 1
						except KeyError:
							count_ss[res.getDSSP()] = 1
					index_dssp = dssp_assign.index(res.getDSSP())
					key_dssp = res.getDSSP() + count_ss[res.getDSSP()]
					element_dssp = res.Type + accessibility
					try:
						self.dssp_stat[key_dssp].append(element_dssp)
					except KeyError:
						self.dssp_stat[key_dssp] = [element_dssp]
	def pb_assign (self):
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:
				dict_pb[res.PositionDansSeq] = [Definitions.pb_dict[res.getPB()],res.Type]
		return dict_pb
	def chain_details(self, name,RepertoirePDB):
		dict = {}
		dict[name] = []
		j = 0 
		LignesPDB = []
                NumAtom = [0]
		NumRes = [0]
		chain = name[-1]
		pdb_file_name = RepertoirePDB+"/"+name.lower()+".pdb"
                PDB = open(pdb_file_name, "w")
                for res in self.pdb_struct: 
                	if res.__class__ == ResiduObj.Residu: 
                        	count = 0
				acc = '#' 
                                flag_enter = 0 
                                for k in range(j-2,j+3): 
                                	try: 
                                		if not self.pdb_struct[k].getaccess() == -77: 
                                        		if self.pdb_struct[k].getaccess() >= 7: count += 1 
                                                	flag_enter  += 1 
	                                        else: acc = '#' 
        	                        except IndexError: 
                	                        acc = '#' 
                        	if count >= 3: acc = 'A' 
                                elif count < 3 and flag_enter == 5: acc = 'B' 
				else: acc = '#'
				if j in [0,1,len(self.pdb_struct)-1,len(self.pdb_struct)-2]: acc = '#'
				try: x1 = res.Backbone['CA'].Coord[0][0]
				except KeyError: continue
				try: 
					dict[name].append([Definitions.EquiAA_3to1lettres[res.Type], Definitions.pb_dict[res.getPB()], res.getaccess(), res.PositionDansSeq, acc, res.Backbone['CA'].Coord[0][0],res.Backbone['CA'].Coord[0][1],res.Backbone['CA'].Coord[0][2]])
					res.ResiduTOListOfString_PDB (NumAtom,chain,LignesPDB, None, NumRes)
				except KeyError: pass
				j += 1
		LignesPDB.append("TER\n\n")
                PDB.writelines(LignesPDB)
                PDB.close()
		return dict


	def scop_domains(self, scop_file_name, RepertoirePDB, pdb_id, model_number, chain,domain):
		dict = {}
		length = 0
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:
				#print res.PositionDansSeq
				length += 1
				if length == 1: start = res.PositionDansSeq
				end = res.PositionDansSeq
		
		scop_details = ParseSCOP.ParserSCOP(scop_file_name).getSCOP(pdb_id, chain, length, start, end)
		scopid_lim = scop_details[2]
		chain_list = scop_details[3]
		if chain == " ":
                        char_chain = "_"
		else: char_chain = chain
                #pdb_file_name = RepertoirePDB+"/"+pdb_id+"_"+`model_number`+"_"+char_chain+".pdb"
		#print chain_list
                LignesPDB = []
                NumAtom = [0]
		for keys in scopid_lim.keys():
			LignesPDB = []
			NumAtom = [0]
			#pdb_file_name = RepertoirePDB+"/"+keys+".pdb"
			#PDB = open(pdb_file_name, "w")
			i = 0
			dict[keys] = []
			if keys != domain: continue
			pdb_file_name = RepertoirePDB+"/"+keys[:4].upper()+".pdb"
			PDB = open(pdb_file_name, "w")
			while i < len(scopid_lim[keys]):
				j = 0
				for res in self.pdb_struct:
					if res.__class__ == ResiduObj.Residu:
						count = 0
						flag_enter = 0
						for k in range(j-2,j+3):
							try:
								if not self.pdb_struct[k].getaccess() == -77:
									if self.pdb_struct[k].getaccess() >= 7: count += 1
									flag_enter  += 1
								else: acc = '#'
							except IndexError:
								acc = '#'
						if count >= 3: acc = 'A'
						elif count < 3 and flag_enter == 5: acc = 'B'
						try: x1 = res.Backbone['CA'].Coord[0][0]
                                		except KeyError: continue
                                                if re.search("[A-Z|a-z]", scopid_lim[keys][i][-1]) == None and re.search("[A-Z|a-z]", scopid_lim[keys][i+1][-1]) == None:
                                                        if int(res.PositionDansSeq[:-1]) >= int(scopid_lim[keys][i]) and int(res.PositionDansSeq[:-1]) <= int(scopid_lim[keys][i+1]):
                                                                dict[keys].append([Definitions.EquiAA_3to1lettres[res.Type], Definitions.pb_dict[res.getPB()], res.getaccess(), res.PositionDansSeq, acc, res.Backbone['CA'].Coord[0][0],res.Backbone['CA'].Coord[0][1],res.Backbone['CA'].Coord[0][2]])
                                                                res.ResiduTOListOfString_PDB (NumAtom,chain,LignesPDB, None, None)
                                                else:
                                                        if re.search("[A-Z|a-z]", scopid_lim[keys][i][-1]) == None:
                                                                if int(res.PositionDansSeq[:-1]) >= int(scopid_lim[keys][i]) and int(res.PositionDansSeq[:-1]) <= int(scopid_lim[keys][i+1][:-1]):
                                                                        dict[keys].append([Definitions.EquiAA_3to1lettres[res.Type], Definitions.pb_dict[res.getPB()], res.getaccess(), res.PositionDansSeq, acc,res.Backbone['CA'].Coord[0][0],res.Backbone['CA'].Coord[0][1],res.Backbone['CA'].Coord[0][2]])
                                                                        res.ResiduTOListOfString_PDB (NumAtom,chain,LignesPDB, None, None)

                                                        elif re.search("[A-Z|a-z]", scopid_lim[keys][i+1][-1]) == None:
                                                                if int(res.PositionDansSeq[:-1]) >= int(scopid_lim[keys][i][:-1]) and int(res.PositionDansSeq[:-1]) <= int(scopid_lim[keys][i+1]):
                                                                        dict[keys].append([Definitions.EquiAA_3to1lettres[res.Type], Definitions.pb_dict[res.getPB()], res.getaccess(), res.PositionDansSeq, acc,res.Backbone['CA'].Coord[0][0],res.Backbone['CA'].Coord[0][1],res.Backbone['CA'].Coord[0][2]])
                                                                        res.ResiduTOListOfString_PDB (NumAtom,chain,LignesPDB, None, None)
                                                        else:
                                                                if int(res.PositionDansSeq[:-1]) >= int(scopid_lim[keys][i][:-1]) and int(res.PositionDansSeq[:-1]) <= int(scopid_lim[keys][i+1][:-1]):
                                                                        dict[keys].append([Definitions.EquiAA_3to1lettres[res.Type], Definitions.pb_dict[res.getPB()], res.getaccess(), res.PositionDansSeq, acc,res.Backbone['CA'].Coord[0][0],res.Backbone['CA'].Coord[0][1],res.Backbone['CA'].Coord[0][2]])
                                                                        res.ResiduTOListOfString_PDB (NumAtom,chain,LignesPDB, None, None)
					j += 1
				i += 2
			LignesPDB.append("TER\n\n")
                	PDB.writelines(LignesPDB)
                	PDB.close()
		return dict, chain_list

	def main_chain_checks (self):
		count = 0
		bad_bond = 0
		bond_count = 0
		bad_angle = 0
		angle_count = 0
		for res in self.pdb_struct:
			type = res.Type
			pos = res.PositionDansSeq
			flag_end = 0
			if res.__class__ == ResiduObj.Residu and res.ResiduComplet():
				bond_count += 3
				angle_count += 3
				if count == (len(self.pdb_struct)-1):
					flag_end = 1
				else:
					if count < len (self.pdb_struct) and self.pdb_struct[count+1].ResiduComplet() and self.pdb_struct[count+1].__class__ == ResiduObj.Residu:
						bond_CN = self._CalculDistanceAtome(count, count+1, 'C', 'N', 0, 0)
						dot_CNCA = self._CalculAngleAtome(count, count+1, count+1, 'C','N','CA', 0,0,0)
						dot_CACN = self._CalculAngleAtome(count, count, count+1, 'CA','C','N', 0,0,0)
#						dot_OCN = self._CalculAngleAtome(count, count, count+1, 'O','C','N', 0,0,0)
					else: flag_end = 1
				bond_NCA = self._CalculDistanceAtome(count, count, 'N', 'CA', 0, 0)
				bond_CAC = self._CalculDistanceAtome(count, count, 'CA', 'C', 0, 0)
#				bond_CO = self._CalculDistanceAtome(count, count, 'C', 'O', 0, 0)
#				dot_CACO = self._CalculAngleAtome(count, count, count, 'CA','C','O', 0,0,0)
				dot_NCAC = self._CalculAngleAtome(count, count, count, 'N','CA', 'C',0,0,0)
				if flag_end != 1: angle_CNCA = (acos(dot_CNCA/(bond_CN*bond_NCA)))*(180.0/pi)
				if flag_end != 1: angle_CACN = (acos(dot_CACN/(bond_CAC*bond_CN)))*(180.0/pi)
				angle_NCAC = (acos(dot_NCAC/(bond_NCA*bond_CAC)))*(180.0/pi)
				if type == 'PRO':
					if (bond_CN < (1.341 - (2*0.016))) or (bond_CN > (1.341 + (2*0.016))):
						if flag_end != 1:
							bad_bond += 1
					if (bond_NCA < (1.466 - (2*0.015))) or (bond_NCA > (1.466 + (2*0.015))):
						bad_bond += 1
                	                if (bond_CAC < (1.525 - (2*0.021))) or (bond_CAC > (1.525 + (2*0.021))):
                        	                bad_bond += 1
#                                if (bond_CO < (1.231 - 0.020)) or (bond_CO > (1.231 + 0.020)):
#                                        bad_bond += 1
					if (angle_CNCA < (122.6 - (2*5.0))) or (angle_CNCA > (122.6 + (2*5.0))):
        	                                if flag_end != 1:
							bad_angle += 1
					if (angle_CACN < (116.9 - (2*1.5))) or (angle_CACN > (116.9 + (2*1.5))):
                                	        if flag_end != 1:
							bad_angle += 1
					if (angle_NCAC < (111.8 - (2*2.5))) or (angle_NCAC > (111.8 + (2*2.5))): 	
						bad_angle += 1

				elif type == 'GLY':
					if (bond_CN < (1.329 - (2*0.014))) or (bond_CN > (1.329 + (2*0.014))):
						if flag_end != 1:
							bad_bond += 1
					if (bond_NCA < (1.451 - (2*0.016))) or (bond_NCA > (1.451 + (2*0.016))):
						bad_bond += 1
					if (bond_CAC < (1.516 - (2*0.018))) or (bond_CAC > (1.516 + (2*0.018))):
						bad_bond += 1
#					if (bond_CO < (1.231 - 0.020)) or (bond_CO > (1.231 + 0.020)):
#						bad_bond += 1
	                                if (angle_CNCA < (120.6 - (2*1.7))) or (angle_CNCA > (120.6 + (2*1.7))):
						if flag_end != 1:
							bad_angle += 1
                        	        if (angle_CACN < (116.4 - (2*2.1))) or (angle_CACN > (116.4 + (2*2.1))):  
                                	        if flag_end != 1:
							bad_angle += 1
	                                if (angle_NCAC < (112.5 - (2*2.9))) or (angle_NCAC > (112.5 + (2*2.9))):
						bad_angle += 1
	
				else:
                	                if (bond_CN < (1.329 - (2*0.014))) or (bond_CN > (1.329 + (2*0.014))):
						if flag_end != 1:
	                        	                bad_bond += 1
					if (bond_NCA < (1.458 - (2*0.019))) or (bond_NCA > (1.458 + (2*0.019))):
        	                                bad_bond += 1
					if (bond_CAC < (1.525 - (2*0.021))) or (bond_CAC > (1.525 + (2*0.021))):
                        	                bad_bond += 1
#					if (bond_CO < (1.231 - 0.020)) or (bond_CO > (1.231 + 0.020)):
 #                                       bad_bond += 1
	                                if (angle_CNCA < (121.7 - (2*1.8))) or (angle_CNCA > (121.7 + (2*1.8))):
        	                                if flag_end != 1:
                	                                bad_angle += 1
                        	        if (angle_CACN < (116.2 - (2*2.0))) or (angle_CACN > (116.2 + (2*2.0))):  
                                	        if flag_end != 1:
                                        	        bad_angle += 1
	                                if (angle_NCAC < (111.2 - (2*2.8))) or (angle_NCAC > (111.2 + (2*2.8))):
        	                                bad_angle += 1
			count += 1
		return [bad_bond, bad_angle, bond_count, angle_count]			
			
	def assign_scop (self, scop_file_name, pdb_id, chain):
		length = len(self.pdb_struct)
		scop_list = ParseSCOP.ParserSCOP(scop_file_name).getSCOP(pdb_id, chain, length)[0]
		scop_id_list = ParseSCOP.ParserSCOP(scop_file_name).getSCOP(pdb_id, chain, length)[1]
		return [scop_list, scop_id_list]


				
	def norm_bfactor (self):
		count = 0
		bf_ca_sum = 0
		bf_ca_norm = {}
		bf_avg = {}
		bf_ca = []
		sum = 0
		sum_avg = 0
		bf_avg_avg = 0
		bf_avg_sum = 0
		bf_avg_list = []
		bf_avg_norm = {}
		count_noca = 0
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu and len(res.Backbone) != 0:
				res.b_factor()
				bf_ca_sum = bf_ca_sum + res.bf_ca
				bf_ca.append(res.bf_ca)
				count += 1
				if not res.Backbone.has_key('CA'):
					count_noca += 1
				bf_avg[res.PositionDansSeq] = res.bf_avg
				bf_avg_sum = bf_avg_sum + res.bf_avg
				bf_avg_list.append(res.bf_avg)
				
		try: bf_ca_avg = bf_ca_sum/(count-count_noca)
		except ZeroDivisionError: bf_ca_avg = 0	
		try: bf_avg_avg = bf_avg_sum/count
		except ZeroDivisionError: bf_avg_avg = 0
		for i in range(len(bf_ca)):
			sum = sum + pow((float(bf_ca[i]) - float(bf_ca_avg)),2)/(count-count_noca)
			sum_avg = sum_avg + pow((float(bf_avg_list[i]) - float(bf_avg_avg)),2)/count
		sd = sqrt(sum)
		sd_avg = sqrt(sum_avg)
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:
				if len(res.Backbone) != 0:
					if res.Backbone.has_key('CA'):
						bf_ca_norm[res.PositionDansSeq] = float((res.bf_ca - bf_ca_avg)/sd)
						res.norm_bfactor(bf_ca_avg, sd)
					else: 
						bf_ca_norm[res.PositionDansSeq] = 999
						res.norm_bfactor(bf_ca_avg, sd)
					bf_avg_norm[res.PositionDansSeq] = float((res.bf_avg - bf_avg_avg)/sd_avg)
				else:
					bf_ca_norm[res.PositionDansSeq] = 999
					bf_avg_norm[res.PositionDansSeq] = 999
					bf_avg[res.PositionDansSeq] = 999
					
		return [bf_ca_norm, bf_avg, bf_ca_avg, bf_avg_norm]

	def set_accessParRes (self, NomPDB,model_number,IDchaine,RepertoirePDB, Repertoireaccess):
                """Methode pour assigner en accessibility les parties de prot de ce modele proteique"""
                #Reinitialisation des assignations DSSP
                [res.Del_access() for res in self.pdb_struct]

                #Calculs :      
#                                ^M
#        ^M
                #Parcours des differentes parties de la prot et de ses differentes conformations et calcul de la structure secondaire^M
                        #print "numPart" ,numPart^M
                        #si ce modele structural ne possede par de conformation alternatives^M
		if self.memoConformationsALT_CA  == None :
			altloc = 0
                        info_naccess = AssigneAccessibilite.Recup_AssignationNACCESS (NomPDB,model_number, IDchaine, altloc,RepertoirePDB, Repertoireaccess, False)
                                #Parcours des residu pour enregistrer l'assignation
#                        ires = 0
#                        iresDSSP = 0
#                                flagiDSSPchange = 0
                                #print ListePartiesProt[numPart]
			
                        for res in self.pdb_struct:
				if res.__class__ == ResiduObj.Residu:
					flag_access = 0
					if res.PositionDansSeq in info_naccess.keys():
#						if res.PositionDansSeq == pos:
						res.set_access (info_naccess[res.PositionDansSeq])
						flag_access = 1	
					if flag_access == 0 and res.ResiduComplet():
						print "accessibility value missing for %s"%(res.PositionDansSeq)
                                        #print res
                                        #print ListeInfoDSSP[iresDSSP][9]
                                        #print ListeDebFinParties[numPart][0]+ires
#                                        if iresDSSP > (len(ListeInfoDSSP)-1):
#                                                res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], 0,0 )
#                                                ires +=1
#                                        elif ListeInfoDSSP[iresDSSP][9] == ListeDebFinParties[numPart][0]+ires: #si DSSP ne commence pas au premier residu
                                                #print "correspondance"
#                                                res.setDSSP (ListeInfoDSSP[iresDSSP], 0, 0)
#                                                ires +=1
#                                                iresDSSP += 1
#                                                flagiDSSPchange = 1
#                                        else :
#                                                res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], 0,0 )
#                                                ires +=1
#                                                if flagiDSSPchange == 1:
#                                                        flagiDSSPchange = 0
#                                                        iresDSSP += 1^M
#                        else : #des conformations alternatives existent^M
#                                confs = self.memoConformationsALT_CA[numPart]^M
                                #Parcours des conformations possibles^M
#                                numConf = 0^M
#                                for conf in confs:^M
#                                        ListeInfoDSSP = AssigneSII.AssignationDSSP (NomPDB,IDchaine, ListePartiesProt[numPart] , conf, numPart, ListeDebFinParties[numPart][0], ListeDebFinParties[numPart][1],numConf, RepertoirePDBPartieProt, RepertoireSortieDSSP)
                                        #Parcours des residus pour enregistrer l'assignation^M
#                                        ires = 0
#                                        iresDSSP = 0
#                                        flagiDSSPchange = 0
#                                        for res in ListePartiesProt[numPart]:
                                                #print res
                                                #print ListeInfoDSSP[iresDSSP][9]
                                                #print ListeDebFinParties[numPart][0]+ires
#                                                if iresDSSP > (len(ListeInfoDSSP)-1):
#                                                        res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], conf[ires],numConf  )
#                                                        ires +=1
#                                                elif ListeInfoDSSP[iresDSSP][9] == ListeDebFinParties[numPart][0]+ires:
#                                                        res.setDSSP (ListeInfoDSSP[iresDSSP], conf[ires],numConf )
#                                                        ires +=1
#                                                        iresDSSP += 1
#                                                        flagiDSSPchange = 1
#                                                else :
#                                                        res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], conf[ires],numConf )
#                                                        ires +=1
#                                                        if flagiDSSPchange == 1:
#                                                                flagiDSSPchange = 0
#                                                                iresDSSP += 1
#                                                ^M
#                                        numConf +=1^M

#----------------------------------------------------------------------------------------------------------------------------------------
	def set_SIIParRes (self, NomPDB,model, IDchaine,RepertoirePDB, RepertoireSortieDSSP, dir_stride =None):
		"""Methode pour assigner en SII les parties de prot de ce modele proteique"""
		#Reinitialisation des assignations DSSP
		[res.Del_DSSP() for res in self.Seq_Struct]
		[res.Del_DSSP() for res in self.pdb_struct]
		
		#Calculs : 	
				
		#Recup des differentes parties du modele en terme d'objets residus
#		ListePartiesProt = []
#		ListeDebFinParties = []
#		PartieProt = []
#		DebFinPartie = []
#		j=0
#		for i in range(len(self.pdb_struct)):
#			#On verifie qu'on a a faire a un residu et qu'il est complet 
#			if self._pdb_struct[i].__class__ == ResiduObj.Residu and self.pdb_struct[i].ResiduComplet() :
#				PartieProt.append(self.Seq_Struct[i])
#				if j== 0 : DebFinPartie.append(i)
#				j+=1
#				if i == (len(self.Seq_Struct) - 1):#Dernier residu
#					ListePartiesProt.append(PartieProt)
#					DebFinPartie.append(i)
#					ListeDebFinParties.append(DebFinPartie)
					
#			else:
#				#Break
#				#print "Break"
#				if j != 0:
#					ListePartiesProt.append(PartieProt)
#					DebFinPartie.append(i-1)
#					ListeDebFinParties.append(DebFinPartie)
#					PartieProt=[]
#					DebFinPartie=[]
#				j=0
				
		
		#print ListePartiesProt
		#print len(ListePartiesProt)
		#print ListeDebFinParties 
	
		#Parcours des differentes parties de la prot et de ses differentes conformations et calcul de la structure secondaire
#		for numPart in range(len(ListePartiesProt)):
#			print "numPart" ,numPart
			#si ce modele structural ne possede par de conformation alternatives
		if self.memoConformationsALT_CA  == None :
				
#			altloc = [0]*len(ListePartiesProt[numPart])
			ListeInfoDSSP = AssigneSII.AssignationDSSP (NomPDB, model, IDchaine, RepertoirePDB, RepertoireSortieDSSP)
			#Parcours des residu pour enregistrer l'assignation
			ires = 0
			iresDSSP = 0
			flagiDSSPchange = 0
			#print ListePartiesProt[numPart]
			i = 0
			j = 0
			for res in self.pdb_struct:
				if res.__class__ == ResiduObj.Residu:
					if j > 3:
						i = j-3
					flag_dssp = 0
					flag_stride = 0
					while i < len(ListeInfoDSSP):
						if res.PositionDansSeq == ListeInfoDSSP[i][9]:
							res.setDSSP(ListeInfoDSSP[i], 0, 0)
							flag_dssp = 1
							break
						i += 1
					j += 1
					if flag_dssp == 0 and res.ResiduComplet():
						print "warning: no dssp assignment for %s"%(res.PositionDansSeq)
						res.setDSSP([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], 0, 0)

					#print res
				#print ListeInfoDSSP[iresDSSP][9]
				#print ListeDebFinParties[numPart][0]+ires
#				if iresDSSP > (len(ListeInfoDSSP)-1):
#					res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], 0,0 )
#					ires +=1 
#				elif ListeInfoDSSP[iresDSSP][9] == ListeDebFinParties[0][0]+ires: #si DSSP ne commence pas au premier residu
#					#print "correspondance"
#					res.setDSSP (ListeInfoDSSP[iresDSSP], 0, 0)
#					ires +=1 
#					iresDSSP += 1
#					flagiDSSPchange = 1
#				else :
#					res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], 0,0 )
#					ires +=1 
#					if flagiDSSPchange == 1:
#						flagiDSSPchange = 0
#						iresDSSP += 1
		else : #des conformations alternatives existent
			print "error : alternate confs for dssp"
#			confs = self.memoConformationsALT_CA[numPart]
#			#Parcours des conformations possibles
#			numConf = 0
#			for conf in confs:
#				ListeInfoDSSP = AssigneSII.AssignationDSSP (NomPDB,IDchaine, ListePartiesProt[numPart] , conf, numPart, ListeDebFinParties[numPart][0], ListeDebFinParties[numPart][1],numConf, RepertoirePDBPartieProt, RepertoireSortieDSSP)
				#Parcours des residus pour enregistrer l'assignation
#				ires = 0
#				iresDSSP = 0
#				flagiDSSPchange = 0
#				for res in ListePartiesProt[numPart]:
#					#print res
#					#print ListeInfoDSSP[iresDSSP][9]
#					#print ListeDebFinParties[numPart][0]+ires
#					if iresDSSP > (len(ListeInfoDSSP)-1):
#						res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], conf[ires],numConf  )
#						ires +=1 
#					elif ListeInfoDSSP[iresDSSP][9] == ListeDebFinParties[numPart][0]+ires:
#						res.setDSSP (ListeInfoDSSP[iresDSSP], conf[ires],numConf )
#						ires +=1 
#						iresDSSP += 1
#						flagiDSSPchange = 1
#					else :
#						res.setDSSP ([IDchaine,Definitions.EquiAA_3to1lettres[res.Type],' ',360.0,360.0,360.0,0.0,0.0,0.0], conf[ires],numConf )
#						ires +=1 
#						if flagiDSSPchange == 1:
#							flagiDSSPchange = 0
#							iresDSSP += 1
#					
#				numConf +=1
	
	
	
	#----------------------------------------------------------------------------------------------------------------------

	def set_SIIStride(self,NomPDB,model, IDchaine,RepertoirePDB, dir_stride):
                [res.Del_Stride() for res in self.Seq_Struct]
                [res.Del_Stride() for res in self.pdb_struct]

                if self.memoConformationsALT_CA  == None :
#                       altloc = [0]*len(ListePartiesProt[numPart])^M
                        if dir_stride != None:
                                ListInfoStride = AssigneSII_Stride.Recup_AssignationSTRIDE(NomPDB, model, IDchaine, RepertoirePDB, dir_stride)

				for res in self.pdb_struct:
					flag_stride = 0
                                        if len(ListInfoStride) != 0 and res.__class__ == ResiduObj.Residu:
                                                if res.PositionDansSeq in ListInfoStride.keys():
#                                                        if res.PositionDansSeq == keys:
 	                                               res.setStride(ListInfoStride[res.PositionDansSeq][2])
                                                       flag_stride = 1
 #                                                               break
                                                if flag_stride == 0 and res.ResiduComplet():
                                                        print "warning: no stride assignment for %s"%(res.PositionDansSeq)
                                                        res.setStride("#")
		else:
			print "warning!! alt conf considered for SII assignment using Stride"

	#----------------------------------------------------------------------------------------------------------------------
	def set_SIIXtlsstr(self,NomPDB,model, IDchaine,RepertoirePDB, dir_xtlsstr):
		[res.Del_Xtl() for res in self.Seq_Struct]
		[res.Del_Xtl() for res in self.pdb_struct]
		sequence_str = []
		pos_str = []
		pos_prev = "-999"
		for res in self.pdb_struct:
			if res.__class__ == ResiduObj.Residu:
				sequence_str.append(Definitions.EquiAA_3to1lettres[res.Type])
				pos_str.append(res.PositionDansSeq)

		if self.memoConformationsALT_CA  == None :		
			if dir_xtlsstr != None:
				ListInfoXtl = AssigneSII_XTLSSTR.Recup_AssignationXTLSSTR(NomPDB, model, IDchaine, RepertoirePDB, dir_xtlsstr, sequence_str, pos_str)
#			 	print ListInfoXtl	
				for i in range(len(self.pdb_struct)):
					res = self.pdb_struct[(len(self.pdb_struct)-1) - i]
					if res.__class__ == ResiduObj.Residu:
						if len(ListInfoXtl) != 0:
							for keys in ListInfoXtl.keys():
								if res.PositionDansSeq[:-1] == keys[:-1] and res.PositionDansSeq[:-1] != pos_prev:
									res.set_Xtl(ListInfoXtl[keys])
									pos_prev = res.PositionDansSeq[:-1]
								elif res.PositionDansSeq[:-1] == pos_prev:
									res.set_Xtl('#')
						else: 
							res.set_Xtl('#')
							print "xtlsstr assignment failed for %s"%(res.PositionDansSeq)
											
								


	#---------------------------------------------------------------------------------------------------------------#
        def set_SIIPM(self,NomPDB,model, IDchaine,RepertoirePDB, dir_PM):
                [res.Del_PM() for res in self.Seq_Struct]
                [res.Del_PM() for res in self.pdb_struct]
		pos_str = []
                for res in self.pdb_struct: 
                        if res.__class__ == ResiduObj.Residu: 
                                pos_str.append(res.PositionDansSeq) 

                if self.memoConformationsALT_CA  == None :
                        if dir_PM != None:
                                ListInfoPM = AssignSII_promotif.Recup_AssignationPM(NomPDB, model, IDchaine, RepertoirePDB, dir_PM, pos_str)
#                                print ListInfoPM
				str = ''      
                                for res in self.pdb_struct:
                                        if res.__class__ == ResiduObj.Residu:
                                                if len(ListInfoPM) != 0:
							flag_PM = 0
                                                        if res.PositionDansSeq in ListInfoPM.keys():
								if len(ListInfoPM[res.PositionDansSeq]) == 1:
#									print keys
#									print ListInfoPM[keys]
#									if Definitions.EquiAA_3to1lettres[res.Type] == ListInfoPM[keys][-1]:
                                                        		res.set_PM(ListInfoPM[res.PositionDansSeq])
									flag_PM = 1
 #                                                               	elif ListInfoPM[keys][0][:2] == "HP":
  #                                                                      	res.set_PM(ListInfoPM[keys])
#									if ListInfoPM[keys][0] == "H" or ListInfoPM[keys][0] == "E" or ListInfoPM[keys][0] == "G" or ListInfoPM[keys][0][:2] == "HP":
#										res_cont = 1
#									else: res_cont = 0
								else: 
									new_list = []
									str = ''	
									for elem in ListInfoPM[res.PositionDansSeq]:
										str = str + ","+elem[0].strip()
									new_list.append([str[1:]])
									res.set_PM(new_list)	
									flag_PM =1
#								if res_cont == 1:		
#									
#								elif res.PositionDansSeq[:-1] == keys[:-1] and ListInfoPM[keys][0][:2] == "HP":
#									res.set_PM(ListInfoPM[keys])
#								else: res.set_PM(['C'])
							elif res.ResiduComplet(): res.set_PM('C')
                                                else:
                                                        res.set_PM(['#'])
                                                        print "Promotif assignment failed for %s"%(res.PositionDansSeq)

	#------------------------------------------------------------------------------------------------------------------------#
        #----------------------------------------------------------------------------------------------------------------------^M
        def set_SIISEGNO(self,NomPDB,model, IDchaine,RepertoirePDB, dir_segno):
                [res.Del_Segno() for res in self.Seq_Struct]
                [res.Del_Segno() for res in self.pdb_struct]

                if self.memoConformationsALT_CA  == None :
#                       altloc = [0]*len(ListePartiesProt[numPart])^M
                        if dir_segno != None:
                                ListInfoSegno = AssignSII_segno.AssignationSEGNO(NomPDB, model, IDchaine, RepertoirePDB, dir_segno)

                                for res in self.pdb_struct:
                                        flag_segno = 0
                                        if len(ListInfoSegno) != 0 and res.__class__ == ResiduObj.Residu:
                                                if res.PositionDansSeq in ListInfoSegno.keys():
#                                                        if res.PositionDansSeq == keys:
							if res.Type == ListInfoSegno[res.PositionDansSeq][-1]:
	                                                	res.setSegno(ListInfoSegno[res.PositionDansSeq])
                                                       		flag_segno = 1
 #                                                               break
                                                if flag_segno == 0 and res.ResiduComplet():
                                                        print "warning: no segno assignment for %s"%(res.PositionDansSeq)
                                                        res.setSegno("#")
                else:
                        print "warning!! alt conf considered for SII assignment using Segno"

        #----------------------------------------------------------------------------------------------------------------------
	
        def set_SIIKAKSI(self,NomPDB,model, IDchaine,RepertoirePDB, dir_kaksi):
                [res.Del_Kaksi() for res in self.Seq_Struct]
                [res.Del_Kaksi() for res in self.pdb_struct]

                if self.memoConformationsALT_CA  == None :
#                       altloc = [0]*len(ListePartiesProt[numPart])^M
                        if dir_kaksi != None:
                                ListInfoKaksi = AssignSII_KAKSI.AssignationKAKSI(NomPDB, model, IDchaine, RepertoirePDB, dir_kaksi)
#				print ListInfoKaksi
                                for i in range(len(self.pdb_struct)):
					res = self.pdb_struct[i]
                                        flag_kaksi = 0
                                        if len(ListInfoKaksi) != 0 and len(ListInfoKaksi) == len(self.pdb_struct):
						if res.__class__ == ResiduObj.Residu:
#                                                        if res.PositionDansSeq == keys:
							kaksi = ListInfoKaksi[i]
							if ListInfoKaksi[i] == '.':
								kaksi = 'C'
                                                	res.setKaksi(kaksi)
                                                        flag_kaksi = 1
 #                                                               break
					else:
						print "problem with parsing kaksi output"
						res.setKaksi("#")
                else:
                        print "warning!! alt conf considered for SII assignment using Kaksi"
	#---------------------------------------------------------------------------------------------------------------------------#	



	def set_Seq_Struct (self, DicoParseATOM_UneChaine, dict_missing) :
		"""Methode pour enregistrer les informations issues du parser relatives a la structure pour une chaine.
		C'est une liste de dictionnaire du type (un dico par atome):
		[{'SIGATM': [], 'ATOM': [1, ' N  ', ' ', 'MET', 1, 41.472, 3.19, 102.565, 32.920]}, {'SIGATM': [], 'ATOM': [2, ' CA ', ' ', 'MET', 1, 42.898, 2.967, 102.161, 33.8]},...]
		Dans l'odre :Numero de l'atome, Nom de l'atome,	Id de la position alternative (si besoin), Nom du residu 
		Numero du residu dans la sequence, Coord X (pour SIGATM: sd en Angstroms),Coord Y	(pour SIGATM: sd en Angstroms)
		Coord Z	(pour SIGATM: sd en Angstroms),	Bfact (pour SIGATM: sd)
		--> Cree et ajoute les acides amines a la sequence au fur et a mesure"""

		MemoireNumAA_str = "-999 "
		ResiduEnCours = None
		FlagInsertionCode = 0
		count_insertion = 0

		#Iteration sur les atomes
#		print DicoParseATOM_UneChaine
		Iterateur_atomes = iter(DicoParseATOM_UneChaine)
		while 1 :
			try:
				Atome = Iterateur_atomes.next()
				#print Atome #***
				#Recuperation du numero du residu en cours
				NumAA_str = Atome['ATOM'][4]
				flag_het = 0
				num_err = 0
				brk_and_insert = 0
				if MemoireNumAA_str != "-999 " and (int(NumAA_str[:-1]) - int(MemoireNumAA_str[:-1]) > 500):
					flag_het = 1 
				#print NumAA_str#***
				
				#Si le residu n'a jamais ete vu: on le cree et on l'ajoute a la sequence
				#Il devient le residu en cours Et on ajoute l'atome en cours
				#Si le residu existe deja, on ajoute juste l'atome en cours
				if NumAA_str != MemoireNumAA_str and flag_het != 1:
					#print 	ResiduEnCours #***
					
					NumAA = int(NumAA_str[:-1]) #prise en compte du "insertion code"
#					print NumAA
					MemoireNumAA = int(MemoireNumAA_str[:-1])
#					print MemoireNumAA
					if (NumAA - MemoireNumAA) > 1 and MemoireNumAA > 0:
						i = 1
						if len(dict_missing) != 0:
							for res in dict_missing:
								if int(res[:-1]) < NumAA and int(res[:-1]) >= MemoireNumAA:
									i += 1
							if (NumAA - MemoireNumAA) != i and i != 1:
								num_err = (NumAA - MemoireNumAA) - i	
							#print NumAA_str
							#print MemoireNumAA_str
							#print num_err
					
                                        if NumAA_str[-1] == " ":
						brk_and_insert = 0
                                        	if len(dict_missing) != 0:
                                                	#print dict_missing
                                                        for res in dict_missing:
                                                        	if MemoireNumAA_str[-1] != " " and res[-1] != " " and int(res[:-1]) < NumAA and int(res[:-1]) >= MemoireNumAA:
									if ord(res[-1]) > ord(MemoireNumAA_str[-1]):
                                                                                brk_and_insert += 1
                                                                elif MemoireNumAA_str[-1] == " " and res[-1] != " " and int(res[:-1]) < NumAA and int(res[:-1]) >= MemoireNumAA:     
                                                                	brk_and_insert += 1
                                                         #               print brk_and_insert
                                                FlagInsertionCode += brk_and_insert
						if brk_and_insert > 0:
							NumAA = NumAA + FlagInsertionCode
      							#print NumAA

					#prise en compte du "insertion code"
					if NumAA_str[-1] != " ":
						if MemoireNumAA_str[-1] != " ":
							FlagInsertionCode += 1
							#print FlagInsertionCode
							brk_and_insert = (ord(NumAA_str[-1]) - ord(MemoireNumAA_str[-1])) - 1
							FlagInsertionCode += brk_and_insert
							NumAA = NumAA + FlagInsertionCode
							#print NumAA
						else:
							FlagInsertionCode += 1
							brk_and_insert = ord(NumAA_str[-1]) - ord('A')
							FlagInsertionCode += brk_and_insert
							NumAA = NumAA + FlagInsertionCode
						#print NumAA	
					elif FlagInsertionCode >= 1 and brk_and_insert == 0:
						#print FlagInsertionCode
						NumAA = NumAA + FlagInsertionCode
						#print NumAA
					if num_err > 0:
						NumAA = NumAA - num_err
						#print NumAA
					elif num_err < 0:
						NumAA = NumAA + num_err + 2
						#print NumAA										
					
					#print NumAA
					MemoireNumAA_str = NumAA_str	
					#print "'"+str(NumAA)+"'"
					NewResidu = ResiduObj.Residu( Atome['ATOM'][3], NumAA ) 
					Residue_pdbno = ResiduObj.Residu( Atome['ATOM'][3], NumAA_str)
					self.number_pair[NumAA_str] = NumAA
					
					
					self.Seq_Struct.append(NewResidu)
					self.pdb_struct.append(Residue_pdbno)
					ResiduEnCours = NewResidu
					residue_in_course = Residue_pdbno
				#	print self.Seq_Struct #**
#					MemoireNumAA_str = NumAA_str

				#Definition du type de l'atome en cours
				NomAtomeDansPDB = Atome['ATOM'][1].split()[0] #le split sert a eliminer les espaces autour du nom
					#print NomAtomeDansPDB #** 
				
	 
				try :
					TypeAtome = Definitions.DefAtomes[NomAtomeDansPDB]
					#print TypeAtome #***
				except KeyError:	
					#on regarde si l'atome est un hydrogene, si oui: on ne le prend pas en compte
					#print NomAtomeDansPDB
					rechercheH = re.search("H",NomAtomeDansPDB )
					if (rechercheH != None ):
						#print "%s"%(NomAtomeDansPDB)
						continue
					else:
						TypeAtome = NomAtomeDansPDB
					#	print NomAtomeDansPDB
#						raise AtomeNonDefini(NomAtomeDansPDB)
				
				#Ajout de l'atome en cours et enregistrement des informations le concernant
				#Nb: si l'atome existe deja, il ne sera pas ajoute mais un flag indique qu'il existe une position alternative
				if len(Atome['SIGATM']) != 0 : #le champ SIGATM existe
					ResiduEnCours.add_Atome (TypeAtome,Atome['ATOM'][5:8],Atome['ATOM'][8],Atome['ATOM'][9],Atome['SIGATM'][5:8],Atome['SIGATM'][8],Atome['SIGATM'][9])
					residue_in_course.add_Atome (TypeAtome,Atome['ATOM'][5:8],Atome['ATOM'][8],Atome['ATOM'][9],Atome['SIGATM'][5:8],Atome['SIGATM'][8],Atome['SIGATM'][9])
					
				else:
					ResiduEnCours.add_Atome (TypeAtome,Atome['ATOM'][5:8],Atome['ATOM'][8],Atome['ATOM'][9])
					
					residue_in_course.add_Atome (TypeAtome,Atome['ATOM'][5:8],Atome['ATOM'][8],Atome['ATOM'][9])
				#print Atome['ATOM'][5:8],Atome['ATOM'][8]	
							
				
			
			except StopIteration:
				break	
		#print 	ResiduEnCours #***
	#----------------------------------------------------------------------------------------------------------------------	
	
	
	
	def count_breaks (self):
		seqBrk = 0
		for i in range(len(self.Seq_Struct)):
                                        #Qd j'ai un break
                                        if self.Seq_Struct[i].__class__ == ResiduObj.Break :
                                                seqBrk = 1
                                                if i == len(self.Seq_Struct)-1:
                                                        break_count += 1
                                                        res_in_break += len(seqBrk)
                                                        dict_break_count[break_count] = [res_in_break]

                                        elif seqBrk != 0:
                                                break_count += 1
                                                res_in_break += len(seqBrk)
                                                seqBrk = 0
                                                dict_break_count[break_count] = [res_in_break]
		return dict_break_count	


	#----------------------------------------------------------------------------------------------------------------------
	
        def _CalculAngleAtome (self ,PosResi, PosResj, PosResk, NomAtomi, NomAtomj, NomAtomk, jeuCoordi, jeuCoordj, jeuCoordk, EnsAtomei = "Backbone",EnsAtomej = "Backbone", EnsAtomek = "Backbone"):
		distanceAAencoursAAprecedent = 0
                for icoord in range(0,3):
                        #print self.Seq_Struct[PosResi].Backbone[NomAtomi].Coord[jeuCoordi][icoord] , self.Seq_Struct[PosResj].Backbone[NomAtomj].Coord[jeuCoordj][icoord]^M
                        distanceAAencoursAAprecedent += ((self.Seq_Struct[PosResi].__dict__[EnsAtomei][NomAtomi].Coord[jeuCoordi][icoord] - self.Seq_Struct[PosResj].__dict__[EnsAtomej][NomAtomj].Coord[jeuCoordj][icoord] )) * ((self.Seq_Struct[PosResk].__dict__[EnsAtomek][NomAtomk].Coord[jeuCoordk][icoord] - self.Seq_Struct[PosResj].__dict__[EnsAtomej][NomAtomj].Coord[jeuCoordj][icoord] ))
                #print distanceAAencoursAAprecedent^M
                return distanceAAencoursAAprecedent



	def _CalculDistanceAtome (self ,PosResi, PosResj, NomAtomi, NomAtomj, jeuCoordi, jeuCoordj, EnsAtomei = "Backbone",EnsAtomej = "Backbone"):
		distanceAAencoursAAprecedent = 0
		for icoord in range(0,3):
			#print self.Seq_Struct[PosResi].Backbone[NomAtomi].Coord[jeuCoordi][icoord] , self.Seq_Struct[PosResj].Backbone[NomAtomj].Coord[jeuCoordj][icoord]
			distanceAAencoursAAprecedent += pow(self.Seq_Struct[PosResi].__dict__[EnsAtomei][NomAtomi].Coord[jeuCoordi][icoord] - self.Seq_Struct[PosResj].__dict__[EnsAtomej][NomAtomj].Coord[jeuCoordj][icoord],2 )
		distanceAAencoursAAprecedent = sqrt(distanceAAencoursAAprecedent)
		#print distanceAAencoursAAprecedent
		return distanceAAencoursAAprecedent
	#----------------------------------------------------------------------------------------------------------------------

	#----------------------------------------------------------------------------------------------------------------------
	def calculate_short_contact_wo_hb (self, dict_atom_chain):
        	count_SC = 0
		count_res = 0
		count_hyd = 0
		res_num_prev = 0	
		Iterateur_atomes = iter(dict_atom_chain)
                while 1 :
                	try:
                        	Atome = Iterateur_atomes.next()
				self.count_atoms += 1
 				Number_atom = Atome['ATOM'][0]
				res_number = Atome['ATOM'][4]
				b_factor = Atome['ATOM'][8]
				if b_factor > 40:
					self.poor_BF += 1	
				SeqStruct_PositionPDB = [res.PositionDansSeq for res in self.Seq_Struct]
				if res_number != res_num_prev:
					count_res += 1
 				NomAtomeDansPDB = Atome['ATOM'][1].split()[0]
                                try :
                                	TypeAtome = Definitions.DefAtomes[NomAtomeDansPDB]
                                        #print TypeAtome #***^M
                                except KeyError:        
                                        #on regarde si l'atome est un hydrogene, si oui: on ne le prend pas en compte
                                        #print NomAtomeDansPDB^M
                                        rechercheH = re.search("H",NomAtomeDansPDB )
                                        if (rechercheH != None ):
                                                #print "%s"%(NomAtomeDansPDB)^M
                                        	continue
                                        else:
                                                TypeAtome = NomAtomeDansPDB
 				atom_type = TypeAtome[0]
 				coord = [Atome['ATOM'][5],Atome['ATOM'][6],Atome['ATOM'][7]]
				res_no_prev = 0
				i =  0
 				for atom in dict_atom_chain:
					res_no = atom['ATOM'][4]
					if res_no != res_no_prev:
						i += 1
					if i > (count_res + 1) or i < (count_res - 1):
	 					NomAtomePDB = atom['ATOM'][1].split()[0]
 						type_atom = NomAtomePDB[0]
						if type_atom != "H":
							distance = 0		
 							atom_coord = [atom['ATOM'][5],atom['ATOM'][6],atom['ATOM'][7]]
							for j in range(0,3):
 								distance += pow(atom_coord[j]-coord[j],2)
	 						distance = sqrt(distance)
							try:
 								sum_vdW = Definitions.radius_vanderWaal[atom_type] + Definitions.radius_vanderWaal[type_atom]
							except KeyError:
								pass
 							if distance < (sum_vdW - 0.5) and distance != 0:
								if (type_atom == "O" or type_atom == "N") and (atom_type == "O" or atom_type == "N"):
									if distance < (sum_vdW - 1.2):
#										count_hyd += 1		
										count_SC += 1
								else:
									count_SC += 1
					res_no_prev = res_no
				res_num_prev = res_number
                        except StopIteration:
                                break
		return count_SC/2
	
	def _ReTTTAlignementSeq_PeCdesPositionsPDB_try1 (self,Align_seq , Align_struct, listePositionsPDB):
		"""Fonction privee. Gestion des Breaks
		Sert au retraitement de l'alignement realise par needleman en tenant compte de la numerotation donnee dans la partie structure du PDB"""


		#Calcul de la position des gap dans la structure : on utilise la distance C - N
		liste_distancesCN=[]
#		print Align_struct
#		print Align_seq
		for pos in range(len(self.Seq_Struct))[1:]:
			#print self.Seq_Struct[pos-1]
			#print self.Seq_Struct[pos]
#			if self.Seq_Struct[pos].ResiduComplet_NC() and self.Seq_Struct[pos-1].ResiduComplet_NC():
			try:
				liste_distancesCN.append(self._CalculDistanceAtome (pos, pos-1, 'N', 'C', 0, 0))
			except KeyError:
					#print pos, self.Seq_Struct[pos].Type
					#print self.Seq_Struct[pos]
				liste_distancesCN.append(999) #Flag lorsque la distance n'est pas calculable car l'un des residus n'est pas complet
#			else:
#				liste_distancesCN.append(999)
#		print liste_distancesCN		


		#La sequence brute selon la structure. ajouts des gaps (nombre de gaps non determines)
		SeqStruct_3lettres = [res.Type for res in self.Seq_Struct]
#		print SeqStruct_3lettres
		SeqStruct_PositionPDB = [res.PositionDansSeq for res in self.Seq_Struct]
#		print SeqStruct_PositionPDB
		ListeFrag_SeqStruct = []
		pos = 1
#		print liste_distancesCN
		for dist in liste_distancesCN:
			if pos == 1:
				ListeFrag_SeqStruct.append(SeqStruct_3lettres[0])
				#print ListeFrag_SeqStruct
			if dist > Definitions.BondlengthCN+(Definitions.BondlengthCN/2):
				
				if dist == 999 and (SeqStruct_PositionPDB[pos]-SeqStruct_PositionPDB[pos-1]) == 1: 
				#Pour retenir qu'a cet endroit existe une incertitude puisque la distance n'a pu etre calculee
				#On tient compte de la numerotation dans ce cas
					
					#ListeFrag_SeqStruct.append(['*'])
					ListeFrag_SeqStruct.append(SeqStruct_3lettres[pos])	
				else :
					try:
						if self._CalculDistanceAtome (pos, pos-1, 'N', 'C', 0, 0) > (Definitions.BondlengthCN+(Definitions.BondlengthCN/2)):
							ListeFrag_SeqStruct.append('*' )
						#print self._CalculDistanceAtome (SeqStruct_PositionPDB[pos], SeqStruct_PositionPDB[pos-1], 'N', 'C', 0, 0)
						#print SeqStruct_PositionPDB[pos]
						#print SeqStruct_PositionPDB[pos-1]
					except KeyError:
						if dist == 999:
							pass
						else: print "warning: pblm with brk computation based on peptide bond length"
					ListeFrag_SeqStruct.append(SeqStruct_3lettres[pos])
#					print ListeFrag_SeqStruct
					
			else :
				ListeFrag_SeqStruct.append(SeqStruct_3lettres[pos])
			pos += 1
		
		#print "Ajout Gap selon backbone"
		#print ListeFrag_SeqStruct #***
		#print len(ListeFrag_SeqStruct) #**

		#Parcours de la sequence en structure brute et ajout du nombre de gaps necessaires selon la numerotation
		#print	SeqStruct_PositionPDB 
		ListeFrag_SeqStructCopie = ListeFrag_SeqStruct[:]
		i=0
		k=0
		for j in range(len(ListeFrag_SeqStruct)):
			#print ListeFrag_SeqStruct[j], ListeFrag_SeqStructCopie[i]
			if ListeFrag_SeqStruct[j]=='*':
				
				ValposAAPrecedent = SeqStruct_PositionPDB[k-1]
#				print ValposAAPrecedent
#				print ValposAAPrecedent
				ValposAASuivant = SeqStruct_PositionPDB[k]
#				print ValposAASuivant
				#print i
				#ListeFrag_SeqStructCopie.pop(i)
				#print ListeFrag_SeqStructCopie[:i]
				#print ListeFrag_SeqStructCopie[i+1:]
			#	print SeqStruct_PositionPDB[k]
			#	print SeqStruct_PositionPDB[k-1]
			#	print ((SeqStruct_PositionPDB[k]-SeqStruct_PositionPDB[k-1])-1)
				ListeFrag_SeqStructCopie = ListeFrag_SeqStructCopie[:i]+(['_']*(SeqStruct_PositionPDB[k]-SeqStruct_PositionPDB[k-1]-1))+ListeFrag_SeqStructCopie[i+1:]
				i += (SeqStruct_PositionPDB[k]-SeqStruct_PositionPDB[k-1]-1)
			else :
				#print i
				i+=1
				k+=1
				
		#print "New"
		#print ListeFrag_SeqStructCopie 
		ListeFrag_SeqStruct = ListeFrag_SeqStructCopie 

		#Verif du nombre de gaps
		NbGapStructAlignee = Align_struct.count('_')
#		print "NbGapStructAlignee =%d"%(NbGapStructAlignee)
		NbGapStructBrute = ListeFrag_SeqStructCopie.count('_')
		#print "NbGapStructBrute =%d"%(NbGapStructBrute)
		
		if NbGapStructAlignee == NbGapStructBrute :
		
			Align_struct = ListeFrag_SeqStructCopie 
		
		else :#!!!! Il manque des gaps aux extremites (ou la numerotation n'a pas permis d'ajouter les gaps corrects)
			
			Align_seq,Align_SeqStructCopie = needle.Needle (Align_seq,ListeFrag_SeqStructCopie ,gapSymbol=".")
			#print "Alignement 2"
			#print Align_seq
			#print Align_SeqStructCopie

			#Verif 1 : s'il y en a, on supprime les gaps inseres dans Align_seq --> n'ont pas lieu d'etre
			while 1:
				try:
					Align_seq.remove('.')
				except ValueError:	
					break

			#Verif de la localisation des nvx gaps. S'ils sont a l'interieur de la sequence: on les replace a l'exterieur. 
			#Ceux en premiere partie de sequence vont au debut, ceux en deuxieme partie vont a la fin
			TailleSeq = len(Align_SeqStructCopie)
			Align_SeqStructCopie2 = Align_SeqStructCopie[:]
			
			for i in range(TailleSeq):
				if Align_SeqStructCopie[i]==".":
					#--> Premiere partie de sequence
					if i < (TailleSeq/2):
						Align_SeqStructCopie2.remove(".")
						Align_SeqStructCopie2.insert(0,"_")
					#--> Fin de la sequence
					else :
						
						Align_SeqStructCopie2.remove(".")
						Align_SeqStructCopie2.append("_")
		
			#print "Apres deplacement des nvx gaps"
			#print Align_seq
			#print Align_SeqStructCopie2

			#Verif du nombre de gaps
			NbGapStructBrute = Align_SeqStructCopie2.count("_")
#			print "NbGapStructBrute2 =%d"%(NbGapStructBrute)
			#NbNewGapStructBrute = Align_SeqStructCopie2.count('.')
			if NbGapStructAlignee == NbGapStructBrute : #+NbNewGapStructBrute  :
				
				Align_struct = Align_SeqStructCopie2
			else : 
				print NbGapStructAlignee
				print NbGapStructBrute
			#	print "L'alignement entre la seq SEQRES et la seq ATOM est problematique\n%s\n%s"%(" ".join(Align_seq)," ".join(Align_SeqStructCopie2))
				raise ProblemeAlignementSeqetStruct (Align_seq , Align_SeqStructCopie2)
		#print "Dernier"
		#print Align_struct 
		return Align_struct	
	#----------------------------------------------------------------------------------------------------------------------
				


	#----------------------------------------------------------------------------------------------------------------------
	def _ReTTTAlignementSeq_PeCdesPositionsPDB_try2 (self,Align_seq , Align_struct, listePositionsPDB):
		"""Fonction privee. Gestion des Breaks
		Sert au retraitement de l'alignement realise par needleman en tenant compte de la numerotation donnee dans la partie structure du PDB"""
		#print len(listePositionsPDB), listePositionsPDB
		#print Align_struct
		#Recup du numero de la position centrale de la premiere diag alignee plus longue que 10
		j = -1
		i = 0
		ListePos = []
		for i in range(len( Align_struct)):
			
			if Align_struct[i]!= "_" :
				j += 1
				ListePos.append({"posPDB":listePositionsPDB[j],"posSeq":i+1}) #i+1 pour commencer a 1
				#print listePositionsPDB[j]-i+1
			elif len(ListePos) > 10:
				break
			else :
				ListePos = []
	#	print ListePos
		#--> Correspondance :
		PositionCentrale = len(ListePos)/2
		#print ListePos[PositionCentrale ]["posPDB"] , ListePos[PositionCentrale ]["posSeq"]
		NumPosition1PDB = ListePos[PositionCentrale ]["posPDB"] - ListePos[PositionCentrale ]["posSeq"]+1
		#print "NumPosition1PDB = %d"%(NumPosition1PDB)

		#Parcours pour voir si les numeros des positions correspondent
		#S'il ne correspondent pas : est-ce qu'il est possible d'echanger 
		# Echange a gauche a la volee, Echange a droite : on retient la position
		k=0
		j=-1
		
		MemoPosProbDroite = []
		for i in range(NumPosition1PDB, len(Align_struct)+NumPosition1PDB) :
			if Align_struct[k]!= "_" :
				j+= 1
				#print "i=%d   num=%d"%(i,listePositionsPDB[j])
				
				
				if i != listePositionsPDB[j] :
					#print "Prob i=%d   num=%d"%(i,listePositionsPDB[j])
					#print self.Seq_Struct[j].Type

					#si la fin du residu precedent et le debut du residu courant sont trop geometriquement eloignes pour que ce ne soit pas un gap
					if i > listePositionsPDB[j]:
						
						IndexPosPDB = range(NumPosition1PDB, len(Align_struct)+NumPosition1PDB).index(listePositionsPDB[j])
						#print IndexPosPDB
						#Est-ce qu'il n'y a que des gaps jusqu a la bonne position ?
						#print Align_struct[IndexPosPDB:k] #**
						#print ["_"]*len(Align_struct[IndexPosPDB:k])
						if Align_struct[IndexPosPDB:k] == ["_"]*len(Align_struct[IndexPosPDB:k]):
							#Est-ce qu a la bonne position, on retrouve le bon aa ?
							#print "Align_seq[IndexPosPDB]= %s   Align_struct[k]=%s"%(Align_seq[IndexPosPDB],Align_struct[k])
							if Align_seq[IndexPosPDB] == Align_struct[k]:
								#si tout est bon on echange:
								tmp = Align_struct[IndexPosPDB]
								Align_struct[IndexPosPDB] = Align_struct[k]
								Align_struct[k] = tmp
							else :	
								raise ProblemeAlignementSeqetStruct (Align_seq , Align_struct)
						else :
							raise ProblemeAlignementSeqetStruct (Align_seq , Align_struct)
					#Decalage vers la droite necessaire
					if i <  listePositionsPDB[j]:
						#print i, listePositionsPDB[j]
						#print "decalage a droite necessaire"
						MemoPosProbDroite.append(k) #on retient la position dans les sequences de struct et seq
						
			k += 1
		
		#Echange a droite (!!! Non teste encore. Needle semble favorise le decalage a droite)
		#print "Echange a droite"
		#print MemoPosProbDroite 
		FlagAAenStructVU = len(Align_struct)+NumPosition1PDB #Differentiation faux gap/ vrai gap
		if MemoPosProbDroite != []:
			j = len(listePositionsPDB)
			k = len(Align_struct)-1
			#print len(Align_struct)
			#print range(NumPosition1PDB, len(Align_struct)+NumPosition1PDB).reverse()
			#print NumPosition1PDB, len(Align_struct)+NumPosition1PDB
			PositionSeqCorrespondance = range(NumPosition1PDB, len(Align_struct)+NumPosition1PDB)
			PositionSeqCorrespondance.reverse()
			for i in PositionSeqCorrespondance :
				#print i
				if Align_struct[k] != "_":
					j -= 1
					#Differentiation faux gap/ vrai gap
					FlagAAenStructVU -=1
					#print "FlagAAenStructVU = %d"%(FlagAAenStructVU)

					if k in MemoPosProbDroite:
						#print "decal droite i=%d   num=%d"%(i,listePositionsPDB[j])
						#print self.Seq_Struct[j].Type
						#Differentiation faux gap/ vrai gap
						if FlagAAenStructVU >= NumPosition1PDB:
							try :
								distanceAAencoursAAprecedent = self._CalculDistanceAtome (j, j-1, 'N', 'C', 0,0)
								BorneSupAutorisee = Definitions.BondlengthCN+(Definitions.BondlengthCN/2)
							except KeyError:	
								print "Attention calcul d'une distance Ca-Ca pour positionnement des Gaps !"
								distanceAAencoursAAprecedent = self._CalculDistanceAtome (j, j-1, 'CA', 'CA', 0,0)
								BorneSupAutorisee = Definitions.BondlengthCaCa
						
							if  distanceAAencoursAAprecedent > BorneSupAutorisee:
								#print "GAP !"
								IndexPosPDB = range(NumPosition1PDB, len(Align_struct)+NumPosition1PDB).index(listePositionsPDB[j])
								#print listePositionsPDB[j]
								#print IndexPosPDB
								#print k
								#Est-ce qu'il n'y a que des gaps jusqu a la bonne position ?
								#print Align_struct[k+1:IndexPosPDB+1] #**
								#print ["_"]*len(Align_struct[k+1:IndexPosPDB+1])
								if Align_struct[k+1:IndexPosPDB+1] == ["_"]*len(Align_struct[k+1:IndexPosPDB+1]):
									#Est-ce qu a la bonne position, on retrouve le bon aa ?
									#print "Align_seq[IndexPosPDB]= %s   Align_struct[k]=%s"%(Align_seq[IndexPosPDB],Align_struct[k])
									if Align_seq[IndexPosPDB] == Align_struct[k]:
										#si tout est bon on echange:
										tmp = Align_struct[IndexPosPDB]
										Align_struct[IndexPosPDB] = Align_struct[k]
										Align_struct[k] = tmp
									else :	
										raise ProblemeAlignementSeqetStruct (Align_seq , Align_struct)
								else :
									raise ProblemeAlignementSeqetStruct (Align_seq , Align_struct)
				
				k -= 1
			
		#print 	Align_struct
		return Align_struct
	#----------------------------------------------------------------------------------------------------------------------

	#----------------------------------------------------------------------------------------------------------------------
	def PriseEnCompteDesBreaks (self, IDchaine, Seq_3lettres,Seq_1lettre,verbose=1):
		"""Gestion des Breaks.
		Methode pour passer comparer la sequence en aa et la sequence en structure enregistrees si les 2 existent 
		(prend en compte la sequence en aa et les positions indiquees pour les aa dans la structure)
		Sinon, on passe en revue la sequence en structure pour voir si les numeros d'acides amines se suivent.
		Si des discontinuites sont observees, un Break (objet) est ajoute dans la sequence"""

		#Copie de la sequence en structure au cas ou il faudrait la modifier. Les modif seront faites sur la copie et la copie remplacera l'originale a la fin
		Seq_Struct_Copie = self.Seq_Struct[:]
		pdb_struct_copy = self.pdb_struct[:]

		#print len(Seq_3lettres),len(self.Seq_Struct)
		#print self.Seq_Struct
		
		if len(Seq_3lettres) != 0:	#si la sequence est disponible
			#print "J'ai la sequence"
			#Recuperation de la sequence en structure (liste d'aa)
			Seq_Struct_enAA3lettres = [res.Type for res in self.Seq_Struct]
			position_res = [res.PositionDansSeq for res in self.Seq_Struct]
#			print position_res

			#Alignement au mieux des 2 sequences
			Align_seq , Align_struct = needle.Needle (Seq_3lettres,Seq_Struct_enAA3lettres )
#			print Align_seq #**
#			print Align_struct #***
			
			#Si un gap a ete place dans la seq 3lettres --> pb
#			if "_" in Align_seq:
#				raise ProblemeAlignementSeqetStruct (Align_seq , Align_struct)
			
			#si l'alignement a introduit des gaps
			if Align_seq != Align_struct :
				#Retraitement de l alignement en fonction du la numerotation du PDB
				try:
					#print "Premier essai de retraitement"
					Align_struct_try = self._ReTTTAlignementSeq_PeCdesPositionsPDB_try1 (Align_seq , Align_struct, [res.PositionDansSeq for res in self.Seq_Struct])
					
					
				except ProblemeAlignementSeqetStruct:
					print "La methode de rettt 1 a echouee.Essai de la 2e methode"
					#print msg
					#print "Essai de la 2e methode"
					#print Align_struct 
					try: Align_struct_try = self._ReTTTAlignementSeq_PeCdesPositionsPDB_try2 (Align_seq , Align_struct, [res.PositionDansSeq for res in self.Seq_Struct])
					except: 
						print "problem with break computation based on peptide bond length and pdb numbering"
						Align_struct_try = Align_struct
					#print Align_struct_try 
				
				Align_struct = Align_struct_try
				#print "Au final"
				#print Align_seq #**
				#print Align_struct #***
				
				#Debut de l'alignement
				SuiviNum_seq = 1
				SuiviNum_struct_copie = 1	
				SuiviNum_struct = 0	
				decal = 0
				FlagPremier_aastruct = 0
			
				#Parcours de l'alignement
				for i in range(len(Align_seq)):
					#print 	SuiviNum_seq, 	SuiviNum_struct, SuiviNum_struct_copie
					#print 	Align_seq[i], Seq_Struct_Copie[SuiviNum_struct_copie-1].Type
					#Si un gap a ete place dans l'alignement, on ajoute un objet break a la seq_struct
					#Si repetition d'aa, procedure particuliere 
					if Align_struct[i] == "_" :
						##Sans repetition d'aa autour
						#if Align_seq[i]!=Align_seq[i-1] or Align_seq[i]!=Align_seq[i+1] :
						NewBreak = ResiduObj.Break( Align_seq[i], SuiviNum_struct_copie )
						Seq_Struct_Copie.insert(SuiviNum_struct_copie-1, NewBreak )
						pdb_struct_copy.insert(SuiviNum_struct_copie-1, NewBreak)
						self.FlagExistanceBreak=1
						#Suivi de l'avancee dans la copie de seq structure a modifiee si besoin
						SuiviNum_struct_copie += 1
											
	
					#si pas de gap
					elif Align_struct[i] != "_"	:
						#Si deux aa differents ont ete alignes -->pb
						#print i, Definitions.EquiAA_3to1lettres[Align_seq[i]] , Definitions.EquiAA_3to1lettres[Align_struct[i]]
						try:
							if Definitions.EquiAA_3to1lettres[Align_seq[i]] != Definitions.EquiAA_3to1lettres[Align_struct[i]]: 
								
								print Align_struct[i]
						except KeyError:
							print "res missing in seqres record"	
#							raise ProblemeAlignementSeqetStruct (Align_seq, Align_struct)		
						
						#Suivi de l'avancee dans la seq structure non modifiee
						SuiviNum_struct += 1
						#Suivi de l'avancee dans la copie de seq structure a modifiee si besoin
						SuiviNum_struct_copie += 1
	
					#Suivi de l'avancee dans la sequence 3 lettres
					SuiviNum_seq += 1
	
					#La copie eventuellement modifiee remplace l'originale a la fin
				self.Seq_Struct = Seq_Struct_Copie 
				self.pdb_struct = pdb_struct_copy
				Seq_Struct_enAA3lettres = [res.Type for res in self.Seq_Struct]
				Seq_Struct_enAA1lettre = [Definitions.EquiAA_3to1lettres[res.Type] for res in self.Seq_Struct]
				Seq_Struct_enAA1lettre_string = "".join(Seq_Struct_enAA1lettre)
				#print self.Seq_Struct #***
				#print Seq_Struct_enAA3lettres #***
				#print Seq_Struct_enAA1lettre_string #***
				#print len(self.Seq_Struct)#***
				#print Seq_3lettres	#***
				#print Seq_1lettre	#***
				#print len(Seq_3lettres)#***
				#Comparaison de la longueur des sequences de la chaine a posteriori
				#et de la composition des sequences
				if (len(self.Seq_Struct)!=len(Seq_3lettres)) or (len(Seq_Struct_enAA1lettre_string) != len(Seq_1lettre)):
#					raise DiffSeqLettreETStruct(IDchaine)
					print Seq_Struct_enAA1lettre_string
			
		else : #La sequence n'est pas disponible (on ne peut que supposer que la sequence commence bien au premier aa)
			#seuls les breaks internes seront vus !!
			#Iterateur pour la structure
			#print "Sans SEQRES"
			#print "je n'ai pas la sequence"
			IterSeqStruc = iter(self.Seq_Struct)	
			AA_Struct_pre = IterSeqStruc.next()
			AA_Struct = IterSeqStruc.next()
			comptaaLettres =AA_Struct.PositionDansSeq
			MemoNumDeb = AA_Struct_pre.PositionDansSeq
			Indice_AA_Struct = 1
			
			while 1:
				try:	
					#calcul de distance entre le residu en cours et le residu precedent
					#print self.Seq_Struct[Indice_AA_Struct].Type,self.Seq_Struct[Indice_AA_Struct].PositionDansSeq,self.Seq_Struct[Indice_AA_Struct-1].Type,self.Seq_Struct[Indice_AA_Struct-1].PositionDansSeq
					try :
						distanceAAencoursAAprecedent = self._CalculDistanceAtome (Indice_AA_Struct, Indice_AA_Struct-1, 'N', 'C', 0,0)
						BorneSupAutorisee = Definitions.BondlengthCN+(Definitions.BondlengthCN/2)
					except KeyError:	
						if verbose == 1 :
							print "Attention calcul d'une distance Ca-Ca pour positionnement des Gaps ! (pecBrk sans seq)"
						distanceAAencoursAAprecedent = self._CalculDistanceAtome (Indice_AA_Struct, Indice_AA_Struct-1, 'CA', 'CA', 0,0)
						BorneSupAutorisee = Definitions.BondlengthCaCa
						
						
					#print comptaaLettres-MemoNumDeb, comptaaLettres , AA_Struct.Type, AA_Struct.PositionDansSeq,distanceAAencoursAAprecedent  #***
					#Si le numero du residu n'est pas compatible avec sa position, ajout d'un break
					#et Si la distance est trop importante pour que les residus se suivent	
					if comptaaLettres < AA_Struct.PositionDansSeq and  distanceAAencoursAAprecedent > BorneSupAutorisee:
						#print comptaaLettres, AA_Struct.PositionDansSeq
						#Ajout d'un objet break dans la sequence en structure
						NewBreak = ResiduObj.Break( "BRK", comptaaLettres )
						Seq_Struct_Copie.insert(comptaaLettres-MemoNumDeb, NewBreak )
						pdb_struct_copy.insert(comptaaLettres-MemoNumDeb, NewBreak )
						self.FlagExistanceBreak=1
						comptaaLettres +=1
				
					elif comptaaLettres == AA_Struct.PositionDansSeq or distanceAAencoursAAprecedent <= BorneSupAutorisee:
						AA_Struct_pre = AA_Struct
						AA_Struct = IterSeqStruc.next()
						comptaaLettres +=1	
						Indice_AA_Struct +=1
						
					else : #il y a un decalage anormal,la numerotation du PDB 
						raise DecalageAnormaleNumerotationStructure(IDchaine)
				except StopIteration:
					break	
			
			#La copie eventuellement modifiee remplace l'originale a la fin
			self.Seq_Struct = Seq_Struct_Copie
			self.pdb_struct = pdb_struct_copy
			#print  [res.Type for res in self.Seq_Struct]
			#print 	[res.PositionDansSeq for res in self.Seq_Struct]
			#print self.Seq_Struct #***
			#print len(self.Seq_Struct)#***
	#----------------------------------------------------------------------------------------------------------------------


	#----------------------------------------------------------------------------------------------------------------------
	def _getListesConformations (self, ListeAtomesCA ,ListeConformations):
		"""Methode privee utilisee par setMemoConformationsAlternatives_Ca (self) pour :
		- La Generation de l'arbre des conformations possibles et recup de ces conformations
		- + : Les conformations sont ordonnees en fonction de leur Occupancy moyenne"""
		
		#print "Avant chemins"
		#Generation de l'arbre des conformations possibles et recup de ces conformations
		ListesAltloc, ListeOccMoy = GestionAltLoc.GetCheminsPossibles (ListeAtomesCA)
		print ListesAltloc
		print ListeOccMoy
		ListeAtomesCA = []
		#print "Apres chemins"
						
						
		#Les conformations sont ordonnees en fonction de leur Occupancy moyenne:
		NewListesAltloc=[]
		while ListesAltloc!=[]:
			OccMax = max(ListeOccMoy)
			print OccMax
			IndexOccMax = ListeOccMoy.index(OccMax)
			print IndexOccMax
			liste = ListesAltloc.pop(IndexOccMax)
			occ = ListeOccMoy.pop(IndexOccMax)
			print occ
			NewListesAltloc.append(liste)
			#print liste
		ListeConformations.append(NewListesAltloc)
		
		return ListeAtomesCA
		
	
	#----------------------------------------------------------------------------------------------------------------------

	def alt_conf (self):
		confalt_sc = 0
		confalt_bb = 0
		multocc_count = 0
		for residu in self.Seq_Struct:
                	if residu.__class__ == ResiduObj.Residu:
                        	if residu.AltLocation_sc != 0:
                                	confalt_sc += 1
                                        multocc_count += residu.AltLocation_sc
                                if residu.__class__ == ResiduObj.Residu and residu.ResiduComplet() :
                                        #print residu.AltLocationOfCa
                                        if residu.AltLocationOfCa != 0:
                                                confalt_bb += 1
                                                multocc_count += residu.AltLocationOfCa
		return [confalt_bb, confalt_sc, multocc_count]


	#----------------------------------------------------------------------------------------------------------------------
	def setMemoConformationsAlternatives_Ca (self):
		"""Methode permettant de calculer les conformations alternatives possibles d'une chaine proteique. 
		Memoire pour les Ca"""
		
		#Reinitialisation du nombre de conformations enregistrees
		self.memoNombreConformationsALT_CA = []
		
		#Verification de l'exitence de conformations alternatives et comptage du nombre de conformation
		ConfAlt = 1
		IterateurSequence = iter(self.Seq_Struct)
		NbConfCA = 1
		FlagPartie = 0
		confalt_sc = 0
		confalt_bb = 0
		
		while 1:
			try:
				
				residu = IterateurSequence.next()
				#print residu.__class__
				if residu.__class__ == ResiduObj.Residu:
					if residu.AltLocation_sc != 0:
						confalt_sc += 1
						multocc_count += residu.AltLocation_sc		
				if residu.__class__ == ResiduObj.Residu and residu.ResiduComplet() :
					#print residu.AltLocationOfCa
					if residu.AltLocationOfCa != 0:
						confalt_bb += 1
						multocc_count += residu.AltLocationOfCa
					FlagPartie = 1
					if (residu.AltLocationOfCa+1) != NbConfCA : #and residu.AltLocationOfCa+1 != 1:
						ConfAlt = ConfAlt*(residu.AltLocationOfCa+1)
						NbConfCA=residu.AltLocationOfCa+1 
				elif  FlagPartie != 0: #Break
					self.memoNombreConformationsALT_CA.append(ConfAlt)
					ConfAlt = 1 
					NbConfCA = 1
					FlagPartie = 0
						
			except StopIteration:
				break
#		if residu.__class__ == ResiduObj.Residu: #Si le dernier residu n'etait pas un break, on note le nombre de conformations ici
#			self.memoNombreConformationsALT_CA.append(ConfAlt)
		
#		print confalt_sc, confalt_bb	
#		print "Nb Conformation  = ",self.memoNombreConformationsALT_CA
#		number_alt  = self.memoNombreConformationsALT_CA
#		testNbConf = [nbConf < 70 for nbConf in self.memoNombreConformationsALT_CA]
#		testConfAlt =  [nbConf >1 for nbConf in self.memoNombreConformationsALT_CA]
		
		#S'il existe des conformations alternatives et qu'elles ne sont pas en trop grand nombre
#		if not (False in testNbConf) and True in testConfAlt:
#			print "Conf Alt !"
#			IterateurSequence = iter(self.Seq_Struct)
#			ListeAtomesCA = []
#			ListeConformations = []
#			
#			while 1:
#				try:
#					residu = IterateurSequence.next()
#					#print residu
#					
#					if residu.__class__ == ResiduObj.Residu and residu.ResiduComplet():
#						ListeAtomesCA.append(residu.Backbone['CA'])
#						#print ListeAtomesCA
#					elif len(ListeAtomesCA) != 0:
#						#Generation de l'arbre des conformations possibles et recup de ces conformations
#						ListesAltloc, ListeOccMoy = GestionAltLoc.GetCheminsPossibles (ListeAtomesCA)
#						print ListesAltloc
#						print ListeOccMoy
#						ListeAtomesCA = []
#						
#						
#						#Les conformations sont ordonnees en fonction de leur Occupancy moyenne:
#						NewListesAltloc=[]
#						while ListesAltloc!=[]:
#							OccMax = max(ListeOccMoy)
#							IndexOccMax = ListeOccMoy.index(OccMax)
#							liste = ListesAltloc.pop(IndexOccMax)
#							occ = ListeOccMoy.pop(IndexOccMax)
#							NewListesAltloc.append(liste)
#						ListeConformations.append(NewListesAltloc)
#						
#						#print len(ListeAtomesCA)
						#print ListeAtomesCA[-1].Coord
#						ListeAtomesCA =self._getListesConformations (ListeAtomesCA ,ListeConformations)
#						#print residu.Type, residu.PositionDansSeq
						#print ListeAtomesCA
						#print len(ListeAtomesCA)
				
#				except StopIteration:
#					if len(ListeAtomesCA) != 0:
						#print len(ListeAtomesCA)
#						ListeAtomesCA =self._getListesConformations (ListeAtomesCA ,ListeConformations)
						#print ListeAtomesCA
						#print residu.Type, residu.PositionDansSeq
						#print len(ListeAtomesCA)
#					break
			#print ListeConformations
			#print "hi"
			#print len(ListeConformations)
#			self.memoConformationsALT_CA = ListeConformations
#			return number_alt
		return confalt_sc, confalt_bb, multocc_count
	
	
	#----------------------------------------------------------------------------------------------------------------------
	def ModelTOListOfString_PDB (self,IDchaine, stringList=[] ):
		"""Impression du modele structural d'une chaine prot au format PDB"""
		NumAtom = [0]
		[res.ResiduTOListOfString_PDB (NumAtom,IDchaine ,stringList ) for res in self.Seq_Struct][0]
		#print stringList
		stringList.append("TER\n")
		#print stringList
	#----------------------------------------------------------------------------------------------------------------------


	#----------------------------------------------------------------------------------------------------------------------
	def ModelTOListOfString_Banque (self,stringList=[],CodePDB=None, IdChaine=None ,NumModel=None,PartieProt = None,PositionsPartiesProt=None, TypeFich="BanqueGenerale" ):
		"""Impression du modele structural d'une chaine prot au format Banque. PositionsPartiesProt donne les indices de debut et de fin de la partie demandee de la chaine dans le cas ou l'on veut une impression par chaine et par partie (pas du type banque)"""
		
		if TypeFich=="BanqueGenerale":
			
			if self.memoConformationsALT_CA == None:
				#print "Ecriture type 1"
				NumRes = [-1]
				[self.Seq_Struct[i].ResiduTOListOfString_Banque (NumRes,0, 0, stringList,CodePDB=CodePDB, IdChaine=IdChaine, TypeFich=TypeFich ) for i  in range(len(self.Seq_Struct))][0]
			else: 
				#Existe des conformations alternatives enregistrees
				#On ecrit la conformation la plus probable de chaque partie de la prot
				#print "Ecriture type 2"
				j=0
				PartieProt = 0
				for i  in range(len(self.Seq_Struct)):
					NumRes = [i]
					if self.Seq_Struct[i].__class__ == ResiduObj.Residu and self.Seq_Struct[i].ResiduComplet() : 
						j+= 1
						#print "PartieProt=",PartieProt
						#print 	self.memoConformationsALT_CA[PartieProt][0][j-1]
					else :
						if j != 0 and PartieProt !=len(self.memoConformationsALT_CA)-1 :				
							PartieProt += 1
						j=0
					#print "PartieProt=",PartieProt
					#print 	self.memoConformationsALT_CA[PartieProt][0][j-1]
					self.Seq_Struct[i].ResiduTOListOfString_Banque ([NumRes[0]-1],0, self.memoConformationsALT_CA[PartieProt][0][j-1],stringList, CodePDB=CodePDB, IdChaine=IdChaine, TypeFich=TypeFich ) 
		elif TypeFich=="ParChaine":
			
			#on ecrit toutes les conformations de la partie demandee
			if self.memoConformationsALT_CA == None:
				#print "Ecriture type 3"
				for j in range(PositionsPartiesProt[1]-PositionsPartiesProt[0]+1):
					NumRes = [PositionsPartiesProt[0]+j]
					self.Seq_Struct[PositionsPartiesProt[0]+j].ResiduTOListOfString_Banque ([NumRes[0]-1],0, 0, stringList,CodePDB=CodePDB, IdChaine=IdChaine,NumModel=NumModel, PartieProt=PartieProt ,TypeFich=TypeFich) 
						
			else : 
				#print "Ecriture type 4"
				for NumConf in range(len(self.memoConformationsALT_CA[PartieProt])):
					for j in range(PositionsPartiesProt[1]-PositionsPartiesProt[0]+1):
						#print j, PositionsPartiesProt[0]
						NumRes = [PositionsPartiesProt[0]+j]
						self.Seq_Struct[PositionsPartiesProt[0]+j].ResiduTOListOfString_Banque ([NumRes[0]-1],NumConf, self.memoConformationsALT_CA[PartieProt][NumConf][j],stringList, CodePDB=CodePDB, IdChaine=IdChaine, NumModel=NumModel,PartieProt=PartieProt,TypeFich=TypeFich ) 
						
		
		#print stringList
	#----------------------------------------------------------------------------------------------------------------------
