import sys
import re
import Definitions

__doc__="""Module de lecture du fichier PDB directement issu de la PDB"""


#-------------------------------------------------------------------------------------------------------------
class PbOuvertureFichier (Exception):	
	"""Probleme d'ouverture du fichier"""
	def __init__(self, msg, NomFichier):
		self.msg = msg
		print >>sys.stderr, "Probleme ouverture Fichier %s : %s"%(NomFichier, msg )
#-------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------
class NombreDeChainesDifferent (Exception):
	"""Apres la premiere etape de lecture du PDB et de recuperation des lignes SEQRES et ATOM:
	Ces 2 categories ne semblent pas avoir le meme nombre de chaines"""
	def __init__ (self, NbChainSEQRES, NbChainATOM):	
		print >>sys.stderr, "Attention, SEQRES donne %d chaines et ATOM donne %d chaines\n"%(NbChainSEQRES,NbChainATOM)
#-------------------------------------------------------------------------------------------------------------		

#-------------------------------------------------------------------------------------------------------------
class PDB_Lecteur :
	"""Class pour lire (filtrer) les fichiers PDBs
	Renvoie uniquement des iterateurs sur les lignes en rapport avec la sequence et les coordonnees de la structure"""
	
	_NomFichierPDB = None
	_FichierPDB = None
	LignesPDB = None


	def __init__ (self, NomFichierPDB):
		self._NomFichierPDB = NomFichierPDB

	def _getLignes (self):
		"""Methode pour lire les lignes du PDB. Le fichier est ouvert, lu puis referme"""
		try:
			self._FichierPDB = open(self._NomFichierPDB, 'r')
		except IOError, err:
			print "unable to open PDB file"
#			raise PbOuvertureFichier(err, self._NomFichierPDB)
		
		self.LignesPDB = self._FichierPDB.readlines()
		self._FichierPDB.close()


	def getSeqAndStruct (self) :
		"""Methode renvoyant des dictionnaires pour les lignes pertinentes pour la sequence et la structure de la prot
		Si plusieurs chaines : les dico renvoyes contiennent une cle par liste de lignes (une pour chaque chaine)"""
		
		Dict_SeqRes = {}
		#Dict_Coord = {}
		Dict_Model = [] 
		ligne_resolution = None
		ligne_ExperimentType = None
		testHETATM = 0
		remark_missing = 0
		flagMODEL = 0
		lines_modres = {}
		dict_missing = {}
		list_r_factor = []
		flag_ter = 0
		chain_ter = 'ZZ'
		chain_end = []
		NumModel = -99
		#Lecture des lignes 
#		try:	
		self._getLignes()
#		except PbOuvertureFichier:
#			sys.exit()

		#Boucle sur les lignes 
		for ligne in self.LignesPDB :
			if (re.search("MISSING RESIDUES", ligne) != None):
				remark_missing = ligne[7:10]
		#		print remark_missing
			if ligne[:10]=="REMARK   3" and re.search("R VALUE", ligne) != None:
				list_r_factor.append(ligne)
			#Recuperation des sequences
			if (ligne[:6]=="SEQRES") : 
				#Extraction du nom de la chaine en cours
				ChainID = ligne[11] #Position de l ID de la chaine
				#print ChainID #**
				try:
					Dict_SeqRes[ChainID].append(ligne)
				except KeyError:		
					Dict_SeqRes[ChainID] = [ligne]

			elif (ligne[:6] == "MODRES"):
				chain = ligne[16]
				try:
					lines_modres[chain].append(ligne)
				except KeyError:
					lines_modres[chain] = [ligne]
			elif (ligne[:6]== "MODEL ") :
				flagMODEL = 1
				try:
                                        NumModel = int(ligne[10:14]) #Numero du modele
                                except Exception:
                                        try:
                                                NumModel = int(ligne.split()[1])
                                        except IndexError:
                                                NumModel = 1
				Dict_Coord = {}
				Dict_Model.append( Dict_Coord )
			elif (ligne[:6] == "TER   "):
#				flag_ter = 1
				if not ligne[21] == ' ':
					chain_ter = ligne[21]		
				else: chain_ter = Chainid
				chain_end.append(chain_ter)
			
			elif (ligne[:6]=="HETATM") :
				#test permettant de ne pas tenir compte des SIGATM se rapportant aux HETATM
				testHETATM = 1
				if ligne[17:20] in Definitions.AA_3Lettres:
					if flagMODEL==0 :
                                        	Dict_Coord = {}
	                                        Dict_Model.append( Dict_Coord )
        	                                flagMODEL = 1
#						flag_ter = 0

                	                Chainid = ligne[21] #Position de l ID de la chaine
					if not Chainid in chain_end:
                        	        	try:
                                	        	Dict_Coord[Chainid].append(ligne)
	                              		except KeyError:
        	                                	Dict_Coord[Chainid] = [ligne]
					elif NumModel != -99:
                                                try:
                                                        Dict_Coord[Chainid].append(ligne)
                                                except KeyError:
                                                        Dict_Coord[Chainid] = [ligne]
					

			
			elif(ligne[:6]=="ATOM  " or ligne[:6]=="SIGATM"):
				#Extraction du nom de la chaine en cours

				if ligne[:6]=="SIGATM" and testHETATM == 1:#test permettant de ne pas tenir compte des SIGATM se rapportant aux HETATM
					testHETATM = 0 #Reinitialisation du test
					continue #on passe a la ligne suivante car celle ci ne se rapporte pas a une ligne ATOM
				
				#si le pdb ne contient qu'un model (pas de ligne model)
				if flagMODEL==0 :
					Dict_Coord = {}
					Dict_Model.append( Dict_Coord )
					flagMODEL = 1
#					fleg_ter = 0
	
				Chainid = ligne[21] #Position de l ID de la chaine
				if not Chainid in chain_end:
					try:
						Dict_Coord[Chainid].append(ligne)
					except KeyError:		
						Dict_Coord[Chainid] = [ligne]

                                elif NumModel != -99:
                                	try:
                                        	Dict_Coord[Chainid].append(ligne)
                                        except KeyError:
                                                Dict_Coord[Chainid] = [ligne]

			elif (re.search("REMARK   2 *RESOLUTION.+[.|0-9]+.+ANGSTROMS", ligne) != None):	
				#Extraction de la resolution
				ligne_resolution = ligne
			
			elif (re.search("EXPERIMENT TYPE", ligne) != None):	
				#Extraction de la methode experimentale (Xray ou EM pour l'instant)
				ligne_ExperimentType = ligne
			
			elif (ligne[:6]=="REMARK" and ligne[7:10]==remark_missing):
				if ligne[15:18] in Definitions.AA_3Lettres:
					chain = ligne[19]
					try:
						dict_missing[chain].append(ligne)
					except KeyError:
						dict_missing[chain] = [ligne]

		#print Dict_SeqRes.keys() #***
		#print Dict_Coord.keys() #***
		#print len(Dict_Model)  #***

		#Verification de l'adequation du nombre de chaine entre SEQRES et ATOM (si le champ SEQRES existe)
		# Attention, si ce n'est pas le cas: une exception est lancee !
		#print [len(Dict_SeqRes) != len(Dict_Coord) for Dict_Coord in Dict_Model] #**
		if (len(Dict_SeqRes) != 0) and (False in [len(Dict_SeqRes) == len(Dict_Coord) for Dict_Coord in Dict_Model] ):
			#print [len(Dict_SeqRes) != len(Dict_Coord) for Dict_Coord in Dict_Model] #**
		#	raise NombreDeChainesDifferent(len(Dict_SeqRes),len(Dict_Coord))
			#print "number of chains different between seqres and atom records"
			pass	
		#On retourne les dictionnaires de lignes
		#return Dict_SeqRes,Dict_Coord, ligne_resolution, ligne_ExperimentType
		#print Dict_Model
		return Dict_SeqRes, Dict_Model, ligne_resolution, ligne_ExperimentType, lines_modres, dict_missing, list_r_factor
#-------------------------------------------------------------------------------------------------------------


		
	
# TEST ----------------------		
def main ():
	print "PDB1"
	L1 = PDB_Lecteur("10MH.pdb")
	L1.getSeqAndStruct()
	#L1._getLignes()
	#print L1.LignesPDB
#	Dict_SeqRes,Dict_Model, ligne_resolution, ligne_ExperimentType = L1.getSeqAndStruct()
#	print Dict_SeqRes 
	#print len(Dict_SeqRes['A']), len(Dict_SeqRes['B'])
	#print Dict_Model
	#print len(Dict_Model[0]['A']),len(Dict_Model[0]['B'])
#	print ligne_resolution
#	print ligne_ExperimentType

#	print "PDB2"
#	L2 = PDB_Lecteur("PDBs/pdb1ee8.ent")
	#L2._getLignes()
	#print L2.LignesPDB
#	Dict_SeqRes,Dict_Model, ligne_resolution, ligne_ExperimentType =L2.getSeqAndStruct()
#	print Dict_SeqRes 
#	print len(Dict_SeqRes['A']), len(Dict_SeqRes['B'])
#	print Dict_Model
#	print len(Dict_Model[0]['A']),len(Dict_Model[0]['B'])
#	print ligne_resolution
#	print ligne_ExperimentType


if __name__ == "__main__":
	main()
