import sys
import re
import Definitions
import AtomeObj
import AssigneSII

__doc__= """Module de definir un objet Residu"""

delta = 0.1



#-------------------------------------------------------------------------------------------------------------
class Residu :
	"""Class permettant de definir la structure d'un acide amine"""

	def __init__ (self, TypeResidu, PositionSeq):
		"""Initialisation d'un objet Residu"""
		self.Type = TypeResidu #3lettres
		self.PositionDansSeq = PositionSeq
		self.Backbone = {} #cle:Nom atome , value: Objet Atome
		self.SideChaine = {}
		self.AltLocationOfCa = 0 #nombre de positions alternatives du Ca : si 0 --> non, si 1 ou plus: oui
		self.AltLocation_sc = 0
		self.AnglesPhiPsi = {'Phi':[],'Psi':[]}
		self.AngleAlpha = []
		self.PB = []
		self.Proto = []
		self.DSSP = []
		self.angle_omega = []
		self.AttributTEMP = [] #Attribut facilitant l'assignation d'autre description. Temporaire car on ne l'enregistre pas dans le residu. L'information sera conservee ailleurs
		#print self.PositionDansSeq
		self.access = []
		self.bf_ca_norm = 0	
		self.bf_ca = 0
		self.bf_avg = 0
		self.stride_ss = []
		self.xtl_ss = []
		self.pm_ss = []
		self.seg_ss = []
		self.kak_ss = []

	
	def change_PositionDansSeq (self,PositionDansSeq ) :
		"""Methode pour modifier la position du residu dans la sequence par rapport a celle notee dans le PDB"""
		self.PositionDansSeq = PositionDansSeq

	def ResiduComplet_all (self):
                """Methode pout tester le non interuption du backbone au niveau du residu"""
                if self.Backbone.has_key("CA") and self.Backbone.has_key("N") and self.Backbone.has_key("C") and self.Backbone.has_key("O"):
                        return True
                else:
                        return False
	
	def ResiduComplet (self):
		"""Methode pout tester le non interuption du backbone au niveau du residu"""
		if self.Backbone.has_key("CA") and self.Backbone.has_key("N") and self.Backbone.has_key("C"):
			return True
		else:
			return False

	def ResiduComplet_NC (self):
		if self.Backbone.has_key("N") and self.Backbone.has_key("C"):
			return True
		else:
			return False	

	def ResiduSCentier (self):
		"""Methode pout tester si le residu a tous les atomes qu'il devrait avoir au niveau de la side chaine"""
#		#On regarde d'abord si le bb est complet
#		if not self.ResiduComplet():
#			return False
#		else : #si oui, on regarde si la chaine est complete
		
		try:
			ListeAtomesTheorique = Definitions.AtomsPerResidusSC[self.Type]
		#print ListeAtomesTheorique
			for atom in ListeAtomesTheorique:
				if not self.SideChaine.has_key(atom):
#					count_miss += 1
					return False
			return True
		except KeyError:
			pass
	
	
	def setAngleAlpha (self, ValAngle):
		"""Methode d'initialisation ou ajout de l'angle Alpha associe au Residu"""
		self.AngleAlpha.append(ValAngle)
	
	def setAnglePhi (self, ValPhi):
		"""Methode d'initialisation ou ajout de valeur d'angle Phi associe au residu"""
		self.AnglesPhiPsi['Phi'].append(ValPhi)
		
	def setAnglePsi (self, ValPsi):
		"""Methode d'initialisation ou ajout de valeur d'angle Psi associe au residu"""
		self.AnglesPhiPsi['Psi'].append(ValPsi)	

	def setAngleOmega (self, ValOmega):
		self.angle_omega.append(ValOmega)
	
	def setProto (self, typeProto)	:
		"""Methode d'initialisation ou ajout d'une assignation de prototype associee au residu"""
		self.Proto.append(typeProto)
		
	def setPB (self,typePB):
		"""Methode d'initialisation ou ajout d'une assignation du PB associee au residu"""
		self.PB.append(typePB)
		
	def setAttributTEMP (self,attr):
		"""Methode d'initialisation ou ajout d'un attribut temporaire"""
		self.AttributTEMP.append(attr)
	
	
	
	
	def set_access (self, info_access):
		self.access.append(info_access[5])
	
	def setDSSP (self, InfoDSSP, altloc, indiceConformation):
		"""Methode permettant d'assigner une sII au residu a partir des informations fournies par DSSP. 
		On en profite pour verifier que les calculs d'angles sont corrects:
		InfoDSSP donne [Nom de la chaine,Nom du residu,SII,angle Alpha,angle Phi,angle Psi,Coord X CA,Coord Y CA,Coord Z CA]"""
		#print InfoDSSP
		
		indiceConformation = 0
		if InfoDSSP[1] == "!" and self.Backbone.has_key('O'): #DSSP n'a pas calcule de SII --> causes possibles:break, residu incomplet
			if self.Backbone.has_key("CA") and self.ResiduComplet() :
				#Le residu n'est pas incomplet et donc Probleme
				raise AssigneSII.DSSPbreakDansPartieProt()
			else : #le residu est incomplet
				self.DSSP.append(None)
		else :
			#print "res", self.Type,' ',self.PositionDansSeq 
			#print Definitions.EquiAA_3to1lettres[self.Type], InfoDSSP[1]
			#On verifie si le residu est du bon type
			if self.Type in Definitions.AA_standard and self.ResiduComplet() and Definitions.EquiAA_3to1lettres[self.Type] == InfoDSSP[1]:
				#On verifie les coordonnees du CA enregistrees
				Xenr = self.Backbone["CA"].Coord[altloc][0]
				#print 'Xenr = ',Xenr
				Yenr = self.Backbone["CA"].Coord[altloc][1]
				#print 'Yenr = ',Yenr
				Zenr = self.Backbone["CA"].Coord[altloc][2]
				#print 'Zenr = ',Zenr
				Xdssp = InfoDSSP[6]
				#print 'Xdssp = ',Xdssp
				Ydssp = InfoDSSP[7]
				#print 'Ydssp = ',Ydssp
				Zdssp = InfoDSSP[8]
				#print 'Zdssp = ',Zdssp
				if (Xenr > Xdssp+delta  or Xenr < Xdssp-delta ) and Xdssp!=0.0: #Coord X differente dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				if (Yenr > Ydssp+delta  or Yenr < Ydssp-delta) and Ydssp!=0.0: #Coord Y differente dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				if (Zenr > Zdssp+delta  or Zenr < Zdssp-delta) and Zdssp!=0.0: #Coord Z differente dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				#NB : pas de verif qd dssp a considere un break (X,Y,Z == 0.0), car ca peut juste etre un 'O' qui manque (or pas d'interuption du backbone)
				
				#On verifie les angles alpha et Phi/Psi
				AngleAlphaenr = self.getAngleAlpha (indiceConformation)
				#print "AngleAlphaenr=",AngleAlphaenr
				AnglePhienr = self.getAnglesPhi (indiceConformation)
				#print "AnglePhienr =",AnglePhienr 
				AnglePsienr = self.getAnglesPsi (indiceConformation)
				#print 'AnglePsienr=',AnglePsienr
				
#				memoCalculDSSP = 1 #test pour verifier que DSSP a fait le calcul (des qu'un residu du fragment est incomplet (N,C ou CA) : DSSP ne fait plus le calcul pour l'angle alpha)
				AngleAlphadssp = InfoDSSP[3]
#				if AngleAlphadssp==360.0 : AngleAlphadssp = 999
#				if AngleAlphadssp==360.0 : memoCalculDSSP=0
				#print "AngleAlphadssp=",AngleAlphadssp
				AnglePhidssp = InfoDSSP[4]
#				if AnglePhidssp==360.0 : AnglePhidssp=999
#				if AnglePhidssp==360.0 : memoCalculDSSP=0
				#print "AnglePhidssp=",AnglePhidssp
				AnglePsidssp = InfoDSSP[5]
#				if AnglePsidssp==360.0 : AnglePsidssp=999
#				if AnglePsidssp==360.0 : memoCalculDSSP=0
				#print "AnglePsidssp=",AnglePsidssp
				
#				if  memoCalculDSSP == 1:#test pour verifier que DSSP a fait le calcul (des qu'un residu du fragment est incomplet (N,C ou CA) : DSSP ne fait plus le calcul pour l'angle alpha)
				if (AngleAlphaenr > AngleAlphadssp+delta  or AngleAlphaenr < AngleAlphadssp-delta) and AngleAlphadssp !=360.0 :#angle alpha different dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				if (AnglePhienr > AnglePhidssp+delta  or AnglePhienr < AnglePhidssp-delta) and AnglePhidssp !=360.0 : #angle phi different dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				if (AnglePsienr > AnglePsidssp+delta  or AnglePsienr < AnglePsidssp-delta) and AnglePsidssp !=360.0 : #angle psi different dans DSSP et enregistrement
					raise AssigneSII.InfoDSSPetEnregistrementDiff()
				#NB : pas de verif qd dssp a considere un break (AngleAlpha,AnglePhi,AnglePsi == 360.0), car ca peut juste etre un 'O' qui manque (or pas d'interuption du backbone)
				
				#Si tout est bon, on enregistre l'assignation SII
				#print "enregistrement"
				dssp = InfoDSSP[2]
				if dssp == ' ': dssp = 'C'
				self.DSSP.append(dssp)
				
			
				
		
		
		


	def setStride(self, ss):
		if ss != None:
			self.stride_ss.append(ss)
		else: self.stride_ss.append("#")
		
	def set_Xtl(self, SS):
		if SS != None:
			self.xtl_ss.append(SS)
		else: self.xtl_ss.append('#')

	def set_PM (self, pm):
		if pm != None:
			self.pm_ss.append(pm[0][0])
		else: self.pm_ss.append("#")
	def setSegno(self, seg):
		if seg != None:
			self.seg_ss.append(seg[0])
		else: self.seg_ss.append("#")

	def setKaksi (self, kak):
		if kak != None:
			self.kak_ss.append(kak)
		else: self.kak_ss.append('#')

	
	
	def getAngleAlpha (self, indiceConformation):
		"""Methode pour obtenir l'angle alpha du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible. La difficulte est que, pour gagner de la place, lorsque plusieurs conformations donnent le meme angle pour le residu
		d'interet: une seule valeur est sauvegardee.. donc les indices ne correspondent plus toujours au nombre de conformations du modele"""
		if len(self.AngleAlpha) == 1:
			return self.AngleAlpha[0]
		else :
			return self.AngleAlpha[indiceConformation]
	
		
	
	def getAnglesPhi (self, indiceConformation=0):
		"""Methode pour obtenir l'angle Phi du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible. La difficulte est que, pour gagner de la place, lorsque plusieurs conformations donnent le meme angle pour le residu
		d'interet: une seule valeur est sauvegardee.. donc les indices ne correspondent plus toujours au nombre de conformations du modele"""
		if len(self.AnglesPhiPsi['Phi']) == 1:
			return self.AnglesPhiPsi['Phi'][0]
		else :
			return self.AnglesPhiPsi['Phi'][indiceConformation]
	
	
	
	def getAngleomega (self):
		return self.angle_omega[0]

	def getAnglesPsi (self, indiceConformation=0):
		"""Methode pour obtenir l'angle Psi du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible. La difficulte est que, pour gagner de la place, lorsque plusieurs conformations donnent le meme angle pour le residu
		d'interet: une seule valeur est sauvegardee.. donc les indices ne correspondent plus toujours au nombre de conformations du modele"""
		if len(self.AnglesPhiPsi['Psi']) == 1:
			return self.AnglesPhiPsi['Psi'][0]
		else :
			return self.AnglesPhiPsi['Psi'][indiceConformation]
	
	
	def getProto (self, indiceConformation=0):
		"""Methode pour obtenir le prototype du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible. La difficulte est que, pour gagner de la place, lorsque plusieurs conformations donnent le meme angle pour le residu
		d'interet: une seule valeur est sauvegardee.. donc les indices ne correspondent plus toujours au nombre de conformations du modele"""
		if len(self.Proto) == 1:
			return self.Proto[0]
		else :
			return self.Proto[indiceConformation]
			
	def getPB (self, indiceConformation=0):
		"""Methode pour obtenir le PB du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible. La difficulte est que, pour gagner de la place, lorsque plusieurs conformations donnent le meme angle pour le residu
		d'interet: une seule valeur est sauvegardee.. donc les indices ne correspondent plus toujours au nombre de conformations du modele"""
		if len(self.PB) == 1:
			return self.PB[0]
		else :
			self.PB = [999]
			return self.PB[indiceConformation]	
			
	def getAttributTEMP (self, indiceConformation):
		"""Methode pour obtenir l'attribut temporaire desire"""
		if len(self.AttributTEMP) == 1:
			return self.AttributTEMP[0]
		else :
			return self.AttributTEMP[indiceConformation]
			
			
	def getDSSP (self, indiceConformation=0):
		"""Methode pour obtenir la SII du residu d'interet dans le cas ou plusieurs conformations sont possibles pour le modele prot
		sont disponible."""
		if self.DSSP == []:
			return "#"
		return self.DSSP[indiceConformation]

	def getaccess (self):
		try:
			return self.access[0]
		except IndexError:
			return -77

	def getStride (self):
		try:
			return self.stride_ss[0]
		except IndexError:
			return '#'
	def getXtl (self):
		try:
			return self.xtl_ss[0]
		except IndexError: return '#'	

	def getPM (self):
		try:
			return self.pm_ss[0]
		except IndexError: return '#'

	def getSegno(self):
		try: return self.seg_ss[0]
		except IndexError: return '#'

	def getKaksi(self):
		try: return self.kak_ss[0]
                except IndexError: return '#'

	def Del_AngleOmega (self):
		self.angle_omega = []
	def Del_AnglesPhi (self):
		"""Methode pour reinitialiser l'enregistrement des phi"""
		self.AnglesPhiPsi["Phi"] = []
		
	def Del_AnglesPsi (self):
		"""Methode pour reinitialiser l'enregistrement des psi"""
		self.AnglesPhiPsi['Psi'] = []
		
	def Del_AngleAlpha (self):
		"""Methode pour reinitialiser l'enregistrement des Angles Alpha"""
		self.AngleAlpha = []
	
	def Del_PB (self):
		"""Methode pour reinitialiser l'enregistrement des PBs"""
		self.PB = []
	
	def Del_Proto (self):
		"""Methode pour reinitialiser l'enregistrement des Protos"""
		self.Proto = []
		
	def Del_AttributTEMP(self):
		"""Methode pour reinitialiser l'enregistrement de l'attribut temporaire"""
		self.AttributTEMP= []
		
	def Del_DSSP (self):
		"""Methode pour reinitialiser l'enregistrement des DSSP """
		self.DSSP = []
	def Del_Stride(self):

		self.stride_ss = []
	
	def Del_access (self):
		self.access = []
	
	def Del_Xtl (self):
		self.xtl_ss = []

	def Del_PM (self):
		self.pm_ss = []

	def Del_Segno(self):
		self.seg_ss = []

	def Del_Kaksi(self):
		self.kak_ss = []

	
	def add_Atome (self, TypeAtome,Liste_CoordXYZ,Bfacteur,Occupancy, Liste_sdCoordXYZ=None,sdBfacteur=None, sdOccupancy = None):
		"""Ajout d'un atome dans le Backbone ou la side chain"""
#		print TypeAtome
#		print "Bfact = %f, Occ=%f"%(Bfacteur,Occupancy)
#		rechercheH = re.search("H",TypeAtome ) #on regarde si l'atome est un hydrogene, si oui: on ne le prend pas en compte
#		if (rechercheH == None or  TypeAtome == "OH" or TypeAtome == "NH1" or TypeAtome == "NH2"):

		flag_occup = 0
		if  Occupancy > 0.20 : # on considere qu'on a l'atome si la probabilite qu'il occupe la position donnee est > a 0.2
			if (TypeAtome in Definitions.Backbone_Atoms) : #Dans le backbone 
				#1er cas, l'atome n'existe pas
				if not self.Backbone.has_key(TypeAtome)  :
					self.Backbone[TypeAtome] = AtomeObj.Atome(TypeAtome)
				#2 eme cas, cette atome existe deja mais possede une 2e position possible 
				else :
					self.Backbone[TypeAtome].AltLocation += 1 #Je signale qu'une 2e position est a prendre en compte pour l'enregistrement des coordonnees
					if TypeAtome == "CA":
						self.AltLocationOfCa += 1
					if self.Backbone[TypeAtome].Occupancy[0] < Occupancy:
						print "check the occupancy %s %f"%(self.PositionDansSeq, Occupancy)
						flag_occup = 1	
#					if occupancy_temp < Occupancy:
#						print "check the occupancy %f"%(Occupancy)
				AtomeTraite = self.Backbone[TypeAtome]
				#print self.Backbone #***
				#print AtomeTraite.AltLocation #**
		
			else :	#ou la chaine lat
				#1er cas, l'atome n'existe pas
				if not self.SideChaine.has_key(TypeAtome) :
					self.SideChaine[TypeAtome] = AtomeObj.Atome(TypeAtome)
				#2 eme cas, cette atome existe deja mais posede une 2e position possible
				else:
					self.SideChaine[TypeAtome].AltLocation += 1 #Je signale qu'une 2e position est a prendre en compte pour l'enregistrement des coordonnees
					self.AltLocation_sc += 1
#                                        if occupancy_temp < Occupancy:
#                                               print "check the occupancy %f"%(Occupancy)
                                        if self.SideChaine[TypeAtome].Occupancy[0] < Occupancy:
                                                #print "check the occupancy %s %f"%(self.PositionDansSeq, Occupancy)
                                                flag_occup = 1

				AtomeTraite = self.SideChaine[TypeAtome]
				#print self.SideChaine #***
				#print AtomeTraite.AltLocation #**

			#Occupancy_prev = Occupancy
			if flag_occup == 1:
				alt_loc_copy = AtomeTraite.AltLocation
				AtomeTraite.AltLocation = 0
				#print "caution!! occupancy order altered"
			#Enregistrement des informations dans l' atome ajoute
			AtomeTraite.set_Coord(Liste_CoordXYZ)
			AtomeTraite.set_Bfact ( Bfacteur)
			AtomeTraite.set_Occupancy ( Occupancy)
			#print AtomeTraite.Coord ,AtomeTraite.Bfact #***
			if Liste_sdCoordXYZ != None: #le champ SIGATM existe
				AtomeTraite.set_Sigatm_Coord (Liste_sdCoordXYZ)
				AtomeTraite.set_Sigatm_Bfact ( sdBfacteur)
				AtomeTraite.set_Sigatm_Occupancy ( sdOccupancy)



	def b_factor (self):
		bf_sum = 0
		count = 0
		for atom in Definitions.Backbone_Atoms:
			if self.Backbone.has_key(atom):
				bf_sum = bf_sum + self.Backbone[atom].Bfact[0]
				count += 1
				if atom == 'CA':
					self.bf_ca = self.Backbone[atom].Bfact[0]
			if not self.Backbone.has_key('CA'):
				self.bf_ca = 0
		self.bf_avg = float(bf_sum/count)

	def norm_bfactor (self, avg, sd):
		if self.Backbone.has_key('CA'):
			self.bf_ca_norm = float((self.bf_ca - avg)/sd)
		else: self.bf_ca_norm = 999		
	def avg_occup (self):
		count = 0
		sum = 0
                for atom in Definitions.Backbone_Atoms:
                        if self.Backbone.has_key(atom):
                                sum = sum + self.Backbone[atom].Occupancy[0]
                                count += 1
		try: avg_occupancy = float(sum/count)
		except ZeroDivisionError: avg_occupancy = 9
		return avg_occupancy

	
	def __str__(self): #A modifier si besoin
		"""Format d'impression: MET 1	CA 0 42.898 2.967 102.161 33.870	
					MET 1	C 0 43.370 3.955 101.074 33.140	
					MET 1	O 0 42.659 4.227 100.079 33.070 ...	"""
		
		stringRes = '%s %s\t'%(self.Type,self.PositionDansSeq)
		stringListeAtom = []
				
		#Recup des lignes relatives aux atomes du residu sous forme de liste
		if len(self.Backbone)!=0: #Si on a enregistre des atomes pour le backbone
			[self.Backbone[atom].AtomTOListOfString(stringListeAtom) for atom in self.Backbone][0]
			
			#print [self.Backbone[atom].AtomTOListOfString(stringListeAtom) for atom in self.Backbone][0] #**
		if len(self.SideChaine)!=0: ##Si on a enregistre des atomes pour la chaine lat
			[self.SideChaine[atom].AtomTOListOfString(stringListeAtom) for atom in self.SideChaine][0]
			#print [self.SideChaine[atom].AtomTOListOfString(stringListeAtom) for atom in self.SideChaine][0]
		#print 	stringListeAtom #**
	
		#Formation des lignes concernant le residu
		stringFinale=stringRes
		stringFinale+=stringRes.join(stringListeAtom)

		return stringFinale+"\n"



	def ResiduTOListOfString_PDB (self,NumAtom,InfoChaine, stringList=[], altloc=None,NumRes = None ): #A modifier si besoin
		"""Format d'impression PDB"""
		
		if NumRes != None: NumRes[0] = NumRes[0] + 1
		stringRes = '%s %s\t'%(self.Type,self.PositionDansSeq)
#		stringListeAtom = []
		# FOR PROFIT
		residue_1letter = Definitions.EquiAA_3to1lettres[self.Type]
		for keys in Definitions.standard_1letter.keys():
			if Definitions.standard_1letter[keys] == residue_1letter:
				residue_type = keys
		if residue_type == None: residue_type = self.Type
		
		if NumRes == None :
			InfoResidu = (residue_type,self.PositionDansSeq)		
		else :
			InfoResidu = (residue_type,NumRes[0] )	
		
		#Recup des lignes relatives aux atomes du residu sous forme de liste
		if len(self.Backbone)!=0: #Si on a enregistre des atomes pour le backbone
			#NumAtom +=1
			Backbone_ordonne = []
			for typeAtome in Definitions.Backbone_Atoms:
				if self.Backbone.has_key(typeAtome):
					Backbone_ordonne.append(typeAtome)
			[self.Backbone[atom].AtomTOListOfString_PDB (NumAtom, InfoResidu,InfoChaine,stringList, altloc) for atom in Backbone_ordonne][0]
			
			
			#print [self.Backbone[atom].AtomTOListOfString(stringListeAtom) for atom in self.Backbone][0] #**
		if len(self.SideChaine)!=0: ##Si on a enregistre des atomes pour la chaine lat
			#NumAtom +=1
			SideChaine_ordonne = []
			for typeAtome in Definitions.SideChaine_Atoms:
				if self.SideChaine.has_key(typeAtome):
					SideChaine_ordonne.append(typeAtome)
			try:
				[self.SideChaine[atom].AtomTOListOfString_PDB (NumAtom, InfoResidu,InfoChaine, stringList, altloc) for atom in SideChaine_ordonne][0]
			except IndexError:
				pass
			for atoms in self.SideChaine.keys():
				if not atoms in SideChaine_ordonne:
					#print atoms
					[self.SideChaine[atoms].AtomTOListOfString_PDB (NumAtom, InfoResidu,InfoChaine, stringList, altloc)][0]
			#print [self.SideChaine[atom].AtomTOListOfString(stringListeAtom) for atom in self.SideChaine][0]
		#print 	stringListeAtom #**
	



	def ResiduTOListOfString_Banque (self, NumRes,indiceConformation, altloc ,stringList=[],CodePDB=None, IdChaine=None,NumModel=None,PartieProt=None, TypeFich="BanqueGenerale" ): 
	#InfoResidu[0] : Nom du residu, InfoResidu[1]:position dans la seq, InfoChaine : IdChaine
		"""Format d'impression Banque pour les residus"""
		
		NumRes[0] = NumRes[0]+1
		#print NumRes[0]
		string=""
		
		type1lettre = Definitions.EquiAA_3to1lettres[self.Type]
		if IdChaine != None and IdChaine==" ": IdChaine="_"
		

		if self.ResiduComplet () == True :
			if self.Backbone.has_key ("O"):
				Etat = "."
				SII = self.getDSSP (indiceConformation)
			else :
				Etat = "~"
				SII = "#"
			PB = self.getPB (indiceConformation)
		#	Proto = self.getProto (indiceConformation)
			X= self.Backbone['CA'].Coord[altloc][0]
			Y= self.Backbone['CA'].Coord[altloc][1]
			Z= self.Backbone['CA'].Coord[altloc][2]
			Phi =self.getAnglesPhi (indiceConformation)
			Psi = self.getAnglesPsi (indiceConformation)
			AAlpha = self.getAngleAlpha (indiceConformation)
			Bfacteur = self.Backbone['CA'].Bfact[altloc]
			Occ = self.Backbone['CA'].Occupancy[altloc]
		else :
			Etat= '<'
			SII = "#"
			PB = 999
			Proto = 999
			X = 999
			Y = 999
			Z = 999
			Phi = 999
			Psi = 999
			AAlpha = 999
			Bfacteur = 999
			Occ = 999
		
		if TypeFich=="BanqueGenerale" :
			#print X,Y,Z,self.Bfact  #***
			string="%4d  %s  %s  %s  %3d    %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %6.2f  %6.2f  %s%s\n"%(NumRes[0],type1lettre,Etat,SII,PB,X,Y,Z,Phi,Psi,AAlpha,Bfacteur,Occ,CodePDB, IdChaine)
			
		elif  TypeFich=="ParChaine":
			#print NumModel,PartieProt,indiceConformation
			string="%4d  %s  %s  %s  %3d  %3d  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %6.2f  %6.2f  Mod%dPart%dConf%d\n"%(NumRes[0],type1lettre,Etat,SII,PB,X,Y,Z,Phi,Psi,AAlpha,Bfacteur,Occ,NumModel,PartieProt,indiceConformation)
		stringList.append(string)


#-------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------
class Break (Residu):
	"""Class permettant de prendre en compte par polymorphisme un acide amine absent ou modifie dans une sequence en structure d'aa
	Herite de la class Residu"""

	def __init__ (self, TypeResidu, PositionSeq):
		"""Initialisation d'un objet Break"""
		Residu.__init__ (self,TypeResidu, PositionSeq)
		#On enregistre si la raison du break est la presence d'un aa modifie (1) ou une absence de coord cristallo (0)
		if self.Type in Definitions.ResiModif_3lettres : 
			self.MODRES = 1
		else :
			self.MODRES = 0
		#print self.MODRES, self.PositionDansSeq



	def __str__(self):
		"""l'affichage de cette objet ne donne rien"""
		pass

	
	def ResiduTOListOfString_Banque (self,NumRes,indiceConformation, altloc ,stringList=[],CodePDB=None, IdChaine=None, TypeFich="BanqueGenerale" ): 
	#InfoResidu[0] : Nom du residu, InfoResidu[1]:position dans la seq, InfoChaine : IdChaine
		"""Format d'impression Banque pour les residus"""
		
		NumRes[0] = NumRes[0]+1
		string=""
		type1lettre = Definitions.EquiAA_3to1lettres[self.Type]
		Etat= '!'
		SII = "#"
		PB = 999
		Proto = 999
		X = 999
		Y = 999
		Z = 999
		Phi = 999
		Psi = 999
		AAlpha = 999
		Bfacteur = 999
		Occ = 999
		if IdChaine != None and IdChaine==" ": IdChaine="_"
		
		if TypeFich=="BanqueGenerale" :
			#print X,Y,Z,self.Bfact  #***
			string="%4d  %s  %s  %s  %3d    %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %6.2f  %6.2f  %s%s\n"%(NumRes[0],type1lettre,Etat,SII,PB,X,Y,Z,Phi,Psi,AAlpha,Bfacteur,Occ,CodePDB, IdChaine)
			
		elif  TypeFich=="ParChaine":
			string="%4d  %s  %s  %s  %3d  %3d  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %6.2f  %6.2f\n"%(NumRes[0],type1lettre,Etat,SII,PB,Proto,X,Y,Z,Phi,Psi,AAlpha,Bfacteur,Occ)
		stringList.append(string)






#-------------------------------------------------------------------------------------------------------------


