/* A PROGRAM FOR LOCAL SIMILARITIES WITH AFFINE WEIGHTS:

    copyright (c) 1990-1997 Xiaoqiu Huang and Webb Miller
    The distribution of the program is granted provided no charge is made
    and the copyright notice is included.
    E-mail: huang@cs.mtu.edu

IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    Please cite the paper
    "A Time-Efficient, Linear-Space Local Similarity Algorithm"
	(Advances in Applied Mathematics, 12: 337-357, 1991)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Webb Miller
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

     An early version of this program is presented in the paper
     "A Space-Efficient Algorithm for Local Similarities"
	(Computer Applications in the Biosciences, 6(4): 373-381, 1990)

	      Xiaoqiu Huang
	      Department of Computer Science
	      Michigan Technological University
	      Houghton, MI 49931

	      Ross C. Hardison and Webb Miller
	      Department of Molecular and Cell Biology
	      Department of Computer Science
	      The Pennsylvania State University
	      University Park, PA 16802

    The SIM program finds k best non-intersecting alignments between
    two sequences or within one sequence. Using dynamic programming
    techniques, SIM is guaranteed to find optimal alignments. The
    alignments are reported in order of similarity score, with the
    highest scoring alignment first. The k best alignments share no
    aligned pairs. SIM requires space proportional to the sum of the
    input sequence lengths and the output alignment lengths. Thus
    SIM can handle sequences of tens of thousands, even a hundred of
    thousands, of base pairs on a workstation. For example,
    on 73,360-bp and 44,594-bp sequences, SIM took 15 hours to find
    100 best local alignments on a Sun4 workstation.

    Users supply scoring parameters. In the simplest form, users just
    provide 3 integers: ms, q and r, where ms is the score of a mismatch
    and the score of an i-symbol indel is -(q + r * i). Each match
    automatically receives score 10. This simple scoring scheme may be
    used for DNA sequences. NOTE: all scores are integers.

    In general, users can define an alphabet of characters to appear
    in the sequences and a matrix that gives the substitution score
    for each pair of symbols in the alphabet. The 127 ASCII characters
    are eligible. The alphabet and matrix are given in a file, where
    the first line lists the characters in the alphabet and the lower
    triangle of the matrix comes next. An example file looks as follows:

    ARNDC	       
     13
    -15  19
    -10 -22  11
    -20 -10 -20  18
    -10 -20 -10 -20  12

    Here the -22 at position (3,2) is the score of replacing N by R.
    This general scoring scheme is useful for protein sequences where the
    set of protein characters and Dayhoff matrix are specified in the file.

    The SIM program is written in C and runs under Unix systems on
    Sun workstations and under DOS systems on PCs.
    We think that the program is portable to many machines.

    Sequences to be analyzed are stored in separate files.
    Sequences must be in FASTA format. The first line begins with the symbol '>'
    followed by the name of the sequence.  The sequence is on the remaining lines.
    Protein sequences must be in upper case.
    DNA sequences could be in upper or lower case.
    A sample sequence file is shown below.

>DNA sequence
GAATTCTAATCTCCCTCTCAACCCTACAGTCACCCATTTGGTATATTAAA
GATGTGTTGTCTACTGTCTAGTATCCCTCAAGTAGTGTCAGGAATTAGTC
ATTTAAATAGTCTGCAAGCCAGGAGTGGTGGCTCATGTCTGTAATTCCAG
CACTGGAGAGGTAGAAGTG

    The SIM program generates alignments to the standard output.
    Redirection is required to put alignments in a specified file. 
    A sample output file is shown below, where alignment 1 aligns
    the 3240-3313 region of A with the 26300-26374 region of B.

Match   Mismatch   Gap-Open Penalty   Gap-Extension Penalty
 10      -15            30                 5

Upper Sequence: A
        Length: 36741
Lower Sequence: B
        Length: 73360

*********************************************************
      Number 1 Local Alignment
      Similarity Score : 155
      Match Percentage : 70%
      Number of Matches : 54
      Number of Mismatches : 18
      Total Length of Gaps : 5
      Begins at (3240, 26300) and Ends at (3313, 26374)

    0     .    :    .    :    .    :    .    :    .    :
 3240 GAGGGGCATTTGAGGGTGTTTCCAATGTTCCTGTTATTCGGAATAGCGCT
      || || ||||||-|| || ||||||-||   || ||||  |||||| || 
26300 GATGGCCATTTG GGTTGGTTCCAA GTCTTTGGTATTGTGAATAGTGCC

   50     .    :    .    :    .
 3290 GGTGTGAACATTC   TGCACAGGTCT
      |   | ||||| |---|||||| ||||
26348 GCAATAAACATACGTGTGCACATGTCT

*********************************************************
      Number 2 Local Alignment
      Similarity Score : 145
      Match Percentage : 86%
      Number of Matches : 19
      Number of Mismatches : 3
      Total Length of Gaps : 0
      Begins at (4566, 47235) and Ends at (4587, 47256)

    0     .    :    .    :
 4566 GGGGAGGGAGGGGAGCAGAGCA
      |||||||  |||||| ||||||
47235 GGGGAGGTTGGGGAGAAGAGCA


    To find k best non-intersecting alignments of segments from two
    sequences in files A and B, respectively, use a command of form

	   sim  k  A  B  ms  q  r > result

    where sim is the name of the object code, k is a positive integer,
    ms is a negative integer specifying mismatch weight, q and r are
    non-negative integers specifying gap-open and gap-extend penalties,
    respectively. Output alignments are saved in the file "result".

    For using a scoring matrix defined in file S, use a command of form

	   sim  k  A  B  S  q  r > result

    Note that ms is replaced by the file S.

    To find repeats in one sequence in file A, use a command of form

	   sim  k  A  ms  q  r > result
    or
	   sim  k  A  S  q  r > result

    Acknowledgments
    The functions diff() and display() are from Gene Myers. We made
    the following modifications: similarity weights (integer), instead of
    distance weights (float), are used, the aligned pairs already output
    are not used in the subsequent computation, and the positions of
    sequence characters in output alignments are shown by increment of 50.
    T. Mark Reboul found truncation errors in the previous version
    of SIM and also made a few helpful suggestions.
*/

/*MAJOR MODIFICATIONS
1) Anchor selection (structural) and chaining - Done
2) Linking the anchors - Weighting the bounds based on anchors
3) PB specific gap initiation penalties
4) Accessibility (PB) specific gap extension penalties



 */

#include   <stdio.h>
#include   <string.h>
#include   <stdlib.h>
#include   <math.h>
#include   <ctype.h>
#define  MATCHSC    10		/* match score */

static int match, mismh;		/* max and min substitution weights */
static int v[128][128],Vaa[128][128],Vac[128][128];			/* substitution scores */
static float gin[50];			/*residue specific gap initiation penalty*/
static float gext[50];			/*residue specific gap extension*/
static char *dhead1, *dhead2;           /* names of sequences */
static int isdna;          		/* 1, DNA; 0, protein */
static int gi, ge, size, size1, size2;
static float ini_wt[50];			/* PB specific gap initiation wt*/
static char alph[50], alph1[128], alph2[50];	/*residue characters in order*/

typedef struct COORDS
                {
                float x,y,z;
                } coord, *xyz;

static xyz **coordinates;
/*
int gap_specific(A,M) char *A; int M;
{ 
	int pen_op,pen,pen_ext,i;
	if (M == 0) return 0;
        pen_op = gin[A[1]];
        for(i=2;i<M;i++)
        {
                pen_ext += gext[A[i]];
        }
        pen = pen_op+pen_ext;
        return pen;
}
*/

//MAIN FUNCTION
main(argc, argv) int argc; char *argv[];
{ 
  int  M, N, K;				/* Sequence lengths and k     */
  char  *A, *B,*Au,*Bu,*A1,*B1;			/* Storing two sequences      */
  int  symbol;				/* The next character	      */
  int  nseq;				/* Number of sequences        */
  int  ms, q, r;			/* User-supplied weights      */
  FILE *Bp, *Ap, *Sp, *ginit, *cd, *Ac,*Aa,*ckopen();
  char *ckalloc();			/* space-allocating function  */
  register int i, j;
  char  *s;			/* alphabet */
  float  ini;				/* size of alphabet */
  float x,y,z;				/*for storing coordinates*/	
  xyz st_xyz;

if ( argc != 9 && argc != 11 && argc != 12)
{ fprintf(stderr,"Usage: %s k Seq1 [Seq2] mismatch gap_open gap_extend\n\n", argv[0]);
  fprintf(stderr,"k           number of local alignments, a positive integer \n");
  fprintf(stderr,"Seq1        file of one sequence in FASTA format\n");
  fprintf(stderr,"Seq2        file of one sequence in FASTA format\n");
  fprintf(stderr,"mismatch    a negative integer for DNA or substitution matrix for PBs\n");
  fprintf(stderr,"gap_openL   gap open penalty for anchors, a non-negative integer \n");
  fprintf(stderr,"gap_extendL gap extension penalty for anchors, a positive integer \n");
  fprintf(stderr,"gap_openG   gap open penalty for linkers, a positive integer \n");
  fprintf(stderr,"gap_extendG gap extension penalty for linkers, a positive integer \n");
  fprintf(stderr,"coord   file with pdb coordinates\n");
  fprintf(stderr,"gap_PB  gap open penalty specific for PBs, a positive integer \n");
  /*exit(1);*/
}
        /* read k: the number of local alignments to find */
	(void) sscanf(argv[1],"%d", &K);

	/* determine the sequence lengths */
	Ap = ckopen(argv[2], "r");
	if ( (symbol = getc(Ap) ) != '>' )
	  fatal("Sequence one must be in FASTA format, starting with '>'");
	for (size = 2; ( symbol = getc(Ap)) != EOF ; size++ )
	  if ( symbol == '\n' )
	    break;
// Calculate length of seq 1
	for (M = 0; ( symbol = getc(Ap)) != EOF ; )
	   if ( symbol != '\n' )
	      ++M;
	(void) fclose(Ap);

	/* allocate space for A and dhead1 */
	A = ( char * ) ckalloc( (M + 1) * sizeof(char));
	A1 = ( char * ) ckalloc( (M + 1) * sizeof(char));
	Au = ( char * ) ckalloc( (M + 1) * sizeof(char));
	dhead1 = ( char * ) ckalloc( size * sizeof(char));

	/* read the first sequence into A */
	Ap = ckopen(argv[2], "r");
	symbol = getc(Ap);
// reading sequence head
	for (i = 0; ( symbol = getc(Ap)) != EOF && symbol != '\n' ; )
	   dhead1[i++] = symbol;
	dhead1[i] = '\0';
	// storing the PB sequence
	for (M = 0; ( symbol = getc(Ap)) != EOF ; )
	{
	   if ( symbol != '\n' )
	   {
		A[++M] = symbol; // consider acc PBs also
		//Au[M] = toupper(symbol); // for linker alignment
	   }	
	   else break;
	}
	// for reading the aa sequence
	for (i = 0; ( symbol = getc(Ap)) != EOF ; )
           if ( symbol != '\n' )
                A1[++i] = symbol;	
	(void) fclose(Ap);

	nseq = 1;

	/* if there is another sequence, read it into B */
	if ( argc == 10 || argc >= 11)
	  { nseq = 2;
	    Bp = ckopen(argv[3], "r");
	    if ( (symbol = getc(Bp) ) != '>' )
	     fatal("Sequence two must be in FASTA format, starting with '>'");
	    for (size = 2; ( symbol = getc(Bp)) != EOF ; size++ )
	      if ( symbol == '\n' )
	        break;
	    for (N = 0; ( symbol = getc(Bp)) != EOF ; )
	       if ( symbol != '\n' )
	          ++N;
	    (void) fclose(Bp);
	    B = ( char * ) ckalloc( (N + 1) * sizeof(char));
	    B1 = ( char * ) ckalloc( (N + 1) * sizeof(char));
	    Bu = ( char * ) ckalloc( (N + 1) * sizeof(char));	
	    dhead2 = ( char * ) ckalloc( size * sizeof(char));
	    Bp = ckopen(argv[3], "r");
	    symbol = getc(Bp);
	    for (i = 0; ( symbol = getc(Bp)) != EOF && symbol != '\n' ; )
	      dhead2[i++] = symbol;
	    dhead2[i] = '\0';
		// PB sequence
	    for (N = 0; ( symbol = getc(Bp)) != EOF ; )
	{	
	       if ( symbol != '\n' )
	        {	
	            B[++N] = symbol; // consider acc PBs
		    //Bu[N] = toupper(symbol); // no accessibility	 
		}
	       else break;
	}
		// aa sequence
	    for (i = 0; ( symbol = getc(Bp)) != EOF ; )
               if ( symbol != '\n' )
		{
                    B1[++i] = symbol;
		}
	    (void) fclose(Bp);
	  }
	printf("read 3 arguments\n");
	(void) sscanf(argv[6],"%d", &q);
	if ( q < 0 )
	   fatal("The gap-open penalty is a nonnegative integer");

	(void) sscanf(argv[7],"%d", &r);
	if ( r <= 0 )
	   fatal("The gap-extend penalty is a positive integer");


        (void) sscanf(argv[8],"%d", &gi);
        if ( gi < 0 )
           fatal("The gap-open penalty for global aln is a nonnegative integer");

        (void) sscanf(argv[9],"%d", &ge);
        if ( ge <= 0 )
           fatal("The gap-extend penalty for global aln is a positive integer");

	printf("read 8 arguments\n");
	isdna = 0;
	/* check if the argument represents a negative integer */
	s = argv[4];
	if ( *s == '-' ) s++;
	for ( ; *s >= '0' && *s <= '9' ; s++ );
	if ( *s == '\0' )
	  { (void) sscanf(argv[4],"%d", &ms);
	    if ( ms >= 0 )
	       fatal("The mismatch weight is a negative integer");
	    match = MATCHSC;
	    mismh = ms;
	    /* set match and mismatch weights */
	    for ( i = 0; i < 128 ; i++ )
	      for ( j = 0; j < 128 ; j++ )
	         if (i == j )
	            v[i][j] = match;
	         else
	            v[i][j] = mismh;
	    v['N']['N'] = mismh;
	    v['n']['n'] = mismh;
	    v['A']['a'] = v['a']['A'] = match;
	    v['C']['c'] = v['c']['C'] = match;
	    v['G']['g'] = v['g']['G'] = match;
	    v['T']['t'] = v['t']['T'] = match;
	    isdna = 1;
	  }
	else
	  { /* read a file containing alphabet and substitution weights */
	    Sp = ckopen(argv[4], "r");
	    (void) fscanf(Sp, "%s", alph);
	    size = strlen(alph);
	    match = mismh = 0;
	    for ( i = 0; i < size ; i++ )
	      for ( j = 0; j <= i ; j++ )
		{ (void) fscanf(Sp, "%d", &ms);
		  v[alph[i]][alph[j]] = v[alph[j]][alph[i]] = ms;
		  if ( ms > match ) match = ms;
		  if ( ms < mismh ) mismh = ms;
		}
		fclose(Sp);
	  }
	printf("read matrix for PB substitution\n");


	// Read the matrix with amino acid substitution weights
	Aa = ckopen(argv[5],"r");
        (void) fscanf(Aa, "%s", alph1);
		printf("%s\n",alph1);

            size1 = strlen(alph1);
	    printf("no of aa: %d\n", size1);
        for ( i = 0; i < size1 ; i++ )
              for ( j = 0; j <= i ; j++ )
                { (void) fscanf(Aa, "%d", &ms);
                  Vaa[alph1[i]][alph1[j]] = Vaa[alph1[j]][alph1[i]] = ms;
                }
                fclose(Aa);
	printf("read matrix for AA substitution\n");

/*	
	// Reading matrix with accessibility based substitutions
	Ac = ckopen(argv[6],'r');
	(void) fscanf(Ac, "%s", alph1);
            size = strlen(alph1);
	for ( i = 0; i < size1 ; i++ )
              for ( j = 0; j <= i ; j++ )
                { (void) fscanf(Ac, "%d", &ms);
                  vAc[alph[i]][alph[j]] = vAc[alph[j]][alph[i]] = ms;
		}	
		fclose(Ac);
*/
	/*read a file containing the PDB chain coordinates and store in a structure array with elements x, y and z*/
	 if (argc >= 10) 
	 {
	 	 cd = ckopen(argv[10], "r"); 
         	 coordinates = (xyz **) ckalloc(2 * sizeof(xyz *));
	         coordinates[0] = (xyz *) ckalloc((M+1) * sizeof(xyz)); // allocating space for coordinate[0]
        	 coordinates[1] = (xyz *) ckalloc((N+1) * sizeof(xyz)); // allocating space for coordinate[1]
         for (i = 0;i <= M; i++)
	 {
         	coordinates[0][i] = (xyz) ckalloc(sizeof(coord));
	 }
         for (i = 0; i <= N; i++)
	 {
               	coordinates[1][i] = (xyz) ckalloc(sizeof(coord));
	 }
         printf("memory allocated for %s\n", argv[10]);                                                                 
// File format : x1 y1 z1 x2 y2 z2 . . .
// 		 x1 y1 z1 x2 y2 z2 . . .
	 coordinates[0][0]->x = 0.0; // first element: sequence length
	 coordinates[0][0]->y = 0.0;
	 coordinates[0][0]->x = 0.0;
	printf("reading coordinates1\n");
	 for (i = 1;i <= M; i++) // M lines
	 {
		st_xyz = coordinates[0][i]; // first sequence
		(void) fscanf(cd, "%f", &x);
		st_xyz->x = x;
		(void) fscanf(cd, "%f", &y);
                st_xyz->y = y;
		(void) fscanf(cd, "%f", &z);
                st_xyz->z = z;
	 }
	 
	 coordinates[1][0]->x = 0.0;
	 coordinates[1][0]->y = 0.0;
	 coordinates[1][0]->z = 0.0;
         for (i = 1;i <= N; i++)
         {
                st_xyz = coordinates[1][i];
                (void) fscanf(cd, "%f", &x);
                st_xyz->x = x;
                (void) fscanf(cd, "%f", &y);
                st_xyz->y = y;
                (void) fscanf(cd, "%f", &z);
                st_xyz->z = z;
         }
	}
	fclose(cd);
	printf("coordinates read\n");
// If the PB specific gap penalties are given 
	printf("%d\n", size);
        if (argc >= 12)
        {
                ginit = ckopen(argv[argc-1],"r");
                for ( i = 0; i < size ; i++ )
                {
                        fscanf(ginit, "%f", &ini);
			//if (i >= size/2) ini = 2*ini;
                        gin[alph[i]] = q + ((q*ini));
                        ini_wt[alph[i]] = ini;
			printf("weight read: %f\n", gin[alph[i]]);
                }
                for ( i = 0; i < size ; i++ )
                {
                        fscanf(ginit, "%f", &ini);
                        gext[alph[i]] = r*ini;
                }
        fclose(ginit);
        }
	printf("gap weights read\n");
	/*
	if (argc == 13)
        {
                ginit = ckopen(argv[argc-1],"r");
                for ( i = 0; i < size1 ; i++ )
                {
                        fscanf(ginit, "%f", &ini);
                        gin[alph[i]] = q - (q*ini);
                        ini_wt[alph[i]] = ini;
                        printf("weight read: %f\n", gin[alph[i]]);
                }
                for ( i = 0; i < size ; i++ )
                {
                        fscanf(ginit, "%f", &ini);
                        gext[alph[i]] = r*ini;
                }
        fclose(ginit);
        }*/
	
  	if ( nseq == 2 )
	  SIM(A,B,A1,B1,M,N,K,q,r,nseq);
	else
	  SIM(A,A,A1,A1,M,M,K,q,r,nseq);
}

static int q, r;			/* gap penalties */
static int qr;				/* qr = q + r */

typedef struct ONE { int COL ;  struct ONE  *NEXT ;} pair, *pairptr;

typedef struct ANCHORS 	{ 
			int *ARi; // array with i coordinates
			int *ARj; // array with j coordinates
			int sc;	// score for the alignment
			int length; // length of alignment
			int starti; // row where alignment begins 
			int startj; // column where alignment begins
			int endi;
			int endj;
			} anchor, *anc;

typedef struct BNDS 
		{
		int i1;
		int j1;
		int i2;
		int j2; 
		} bd, *bnd;


pairptr *row, z; 			/* for saving used aligned pairs */
static short tt;

typedef struct NODE
	{ float  SCORE;
	  int  STARI;
	  int  STARJ;
	  int  ENDI;
	  int  ENDJ;
	  int  TOP;
	  int  BOT;
	  int  LEFT;
	  int  RIGHT; }  vertex, *vertexptr;
anc *ANCHS; 				/*array for storing anchor coordinates*/
static int NumAnchAdd,NumBadAnch; 		/*number of anchors added*/
bnd *bounds;


vertexptr  *LIST;			/* an array for saving k best scores */
vertexptr  low = 0;			/* lowest score node in LIST */
vertexptr  most = 0;			/* latestly accessed node in LIST */
static int numnode;			/* the number of nodes in LIST */

static int *CC, *DD;			/* saving matrix scores */
static int *RR, *SS, *EE, *FF;	 	/* saving start-points */
static int *HH, *WW;		 	/* saving matrix scores */
static int *II, *JJ, *XX, *YY;	 	/* saving start-points */
static int  m1, mm, n1, nn;		/* boundaries of recomputed area */
static int  rl, cl;			/* left and top boundaries */
static int  min;			/* minimum score in LIST */
static short flag;			/* indicate if recomputation necessary*/
static int num_bnd, num_anch;
static int MIDI, MIDJ,TYPE;
static FILE *o1,*o2,*o3;
static char *seq1_out, *seq2_out;
static int s1,s2;
static int *BadAnch;
static int op1, op2, COUNT;
static char *IM, *JN, *IMe, *JNe;
static float anchor_score, global_score, total_score;

/* DIAG() assigns value to x if (ii,jj) is never used before */
#define DIAG(ii, jj, x, value)				\
{ for ( tt = 1, z = row[(ii)]; z != 0; z = z->NEXT )	\
    if ( z->COL == (jj) )				\
      { tt = 0; break; }				\
  if ( tt )						\
    x = ( value );					\
}

/* replace (ss1, xx1, yy1) by (ss2, xx2, yy2) if the latter is large */
#define ORDER(ss1, xx1, yy1, ss2, xx2, yy2)		\
{ if ( ss1 < ss2 )					\
    { ss1 = ss2; xx1 = xx2; yy1 = yy2; }		\
  else							\
    if ( ss1 == ss2 )					\
      { if ( xx1 < xx2 )				\
	  { xx1 = xx2; yy1 = yy2; }			\
	else						\
	  if ( xx1 == xx2 && yy1 < yy2 )		\
	    yy1 = yy2;					\
      }							\
}

/* The following definitions are for function diff() */

int diff();
static int  zero = 0;				/* int type zero        */

#define gap(k)  ((k) <= 0 ? 0 : q+r*(k-1))	/* k-symbol indel score ; altered*/

static int *sapp;				/* Current script append ptr */
static int  last;				/* Last script op appended */

static int I, J;				/* current positions of A ,B */
static int no_mat; 				/* number of matches */ 
static int no_mis; 				/* number of mismatches */ 
static int al_len; 				/* length of alignment */
						/* Append "Delete k" op */
#define DEL(k)				\
{ I += k;				\
  al_len += k;				\
  if (last < 0)				\
    last = sapp[-1] -= (k);		\
  else					\
    last = *sapp++ = -(k);		\
}
						/* Append "Insert k" op */
#define INS(k)				\
{ J += k;				\
  al_len += k;				\
  if (last < 0)				\
    { sapp[-1] = (k); *sapp++ = last; }	\
  else					\
    last = *sapp++ = (k);		\
}

						/* Append "Replace" op */
#define REP 				\
{ last = *sapp++ = 0; 			\
  al_len += 1;				\
}

/* SIM(A,B,M,N,K,Q,R,nseq) reports K best non-intersecting alignments of
   the segments of A and B in order of similarity scores, where
   v[a][b] is the score of aligning a and b, and -(Q+R*i) is the score
   of an i-symbol indel.  						*/

SIM(A,B,A1,B1,M,N,K,Q,R,nseq) char A[],B[],A1[],B1[]; int M,N,K,nseq; int Q,R;
{ int endi, endj, stari, starj;	/* endpoint and startpoint */ 
  int  flag_end,flag_optim, flag_break;	   			/* the max score in LIST */
  int score;
  int count=0,ct,p,e,ct_bnd,u,v,w,d;				/* maximum size of list */	
  register  int  i, j,flag_opt;			/* row and column indices */
  char *ckalloc();			/* space-allocating function */
  float diff_len,rl1,cl1;	
  int  *S;				/* saving operations for diff */
  vertexptr cur,maxi; 			/* temporary pointer */
  vertexptr findmax();	 		/* return the largest score node */
  bnd bd, temp_bnd;		
	
	/* allocate space for all vectors */
	j = (N + 1) * sizeof(int);
	CC = ( int * ) ckalloc(j);
	DD = ( int * ) ckalloc(j);
	RR = ( int * ) ckalloc(j);
	SS = ( int * ) ckalloc(j);
	EE = ( int * ) ckalloc(j);
	FF = ( int * ) ckalloc(j);
	i = (M + 1) * sizeof(int);
	HH = ( int * ) ckalloc(i);
	WW = ( int * ) ckalloc(i);
	II = ( int * ) ckalloc(i);
	JJ = ( int * ) ckalloc(i);
	XX = ( int * ) ckalloc(i);
	YY = ( int * ) ckalloc(i);
	S = ( int * ) ckalloc(i + j);
	row = ( pairptr * ) ckalloc( (M + 1) * sizeof(pairptr));

	/* set up list for each row */
	for ( i = 1; i <= M; i++ )
	  if ( nseq == 2 )
	     row[i] = 0;
	  else
	    { row[i] = z = ( pairptr ) ckalloc( (int) sizeof(pair));
              z->COL = i;			
              z->NEXT = 0;
	    }

	q = Q;
	r = R;
	qr = q; // gap penalty = q + r(k-1)
	bounds = ( bnd * ) ckalloc(30 * sizeof(bnd));
	for (i = 0;i < 30; i++)
	bounds[i] = ( bnd ) ckalloc( (int) sizeof(bd));
	/*output memory allocation*/
	seq1_out = (char *) ckalloc((M+N)*sizeof(char));
	seq2_out = (char *) ckalloc((M+N)*sizeof(char)); 

 (void) printf("Match   Mismatch   Gap-Open Penalty   Gap-Extension Penalty\n");
 (void) printf(" %d      %d            %d                 %d\n\n",
			match, mismh, q, r);
	if ( nseq == 2 )
	  { (void) printf("Upper Sequence: %s\n", dhead1);
	    (void) printf("        Length: %d\n", M);
	    (void) printf("Lower Sequence: %s\n", dhead2);
	    (void) printf("        Length: %d\n", N);
	  }
	else
	  { (void) printf("Single Sequence: %s\n", dhead1);
	    (void) printf("         Length: %d\n", M);
	  }
	anchor_score = 0;
	global_score = 0;
	total_score = 0;
	flag_end = 0;
	flag_optim = 0;
	o1 = fopen("anchors.aln","w");
        o2 = fopen("pb_alignment.aln","w");
	o3 = fopen("anchor_alignments.aln","w"); // file with alignment of anchor positions
	while(flag_end == 0)
	{
	printf("entering alignment calculations\n");
	// Allocating memory for storing nodes of scores
	LIST = ( vertexptr * ) ckalloc( K * sizeof(vertexptr));
        for ( i = 0; i < K ; i++ )
           LIST[i] = ( vertexptr ) ckalloc( (int) sizeof(vertex));

	num_anch = 0;
	num_bnd = 0;
	numnode = min = 0;
	NumAnchAdd = 0;
	MIDI = MIDJ = 0;
	flag_opt = 0;
	s1 = s2 = 0;
	printf("starting the first big pass\n");
	big_pass(A,B,A1,B1,M,N,K,nseq); // finding sub-optimal alignment regions

	count = 0;


        /* Report the K best alignments one by one. After each alignment is
           output, recompute part of the matrix. First determine the size
	   of the area to be recomputed, then do the recomputation         */

	while ( count >= 0 )

	  { if ( numnode == 0 ) {flag_end = 1; break;}
	      //fatalf("There are no %d alignments between two sequences", K);

	    count++; // counting the number of set of anchors computed
	    
	    ct = 0;
            //if (count == 1) cur = findmax();	/* Return a pointer to a node with max score*/

            if (count == 1) maxi = findmax(); /*Node with maximum score*/
	    else break; // in the ideal case, one set of anchors is sufficient

	    if (maxi->SCORE < 8000) // No anchor above maxi threshold 
	    {
		flag_optim++;
		for (i = 0;i < size; i++)
		{
			gin[alph[i]] = gin[alph[i]] - 200;
			gext[alph[i]] = gext[alph[i]] - 200;
		}
		//q = q - 200;  // Reducing the penalties
		//r = r - 200;  //
		printf("\nNo anchors above the maxi threshold: reducing the gap penalties\n");
		if (flag_optim == 3) flag_end = 1; // three cycles to check if another optimal (with lower penalties) maxi can be found
		break; // Passing to the main loop for recalculation of anchors with lower penalties
	    }
	    else flag_end = 1; // to end iteration when an optimal maxi is found
/*
 		maxi_check(maxi)
 
 * */

	    max_others(maxi,M,N); // compute the set of anchors associated with maxi
	    cur = maxi;

	    anchor_memory(); // Store the anchor alignments in array of structures ANCHS

	/* to generate the alignment of anchors associated with the best anchor*/	
	    for (i = 0;i < num_anch;i++) // num_anch = number of anchors in the alignment set
	    {
	    ct++;		// count the number of anchors in the set
            score = cur->SCORE; /*initiated with the best anchor*/
	    printf("anchor score is %d", score);	
	    anchor_score += score;	
      	    stari = ++cur->STARI; // starting points for the alignment
            starj = ++cur->STARJ;
            endi = cur->ENDI;    // ending coordinates
            endj = cur->ENDJ;
            m1 = cur->TOP;
            mm = cur->BOT;
            n1 = cur->LEFT;
            nn = cur->RIGHT;
/*		stari = 1;
		starj = 1;
		endi = M;
		endj = N;*/
            rl = endi - stari + 1; /*row length*/
            cl = endj - starj + 1; /*column length*/
            I = stari - 1;
            J = starj - 1;
            sapp = S; // for storing the alignment operations
            last = 0;
            al_len = 0;
            no_mat = 0;
	    no_mis = 0;
	    printf("starting the anchor alignment routine diff\n");	
            (void) diff(&A[stari]-1, &B[starj]-1,&A1[stari]-1, &B1[starj]-1,rl,cl,gin[A[stari]],gin[A[endi]]); /*generate the alignment by finding optimal midpoints - Hirschberg*/
	    printf("end of the anchor alignment routine diff\n");	
            /* Output the best alignment - in a file*/
	
 (void) fprintf(o1,"\n*********************************************************\n");
            (void) fprintf(o1,"      Number %d Local Alignment\n", ct);
            (void) fprintf(o1,"      Similarity Score : %d\n",score);
          (void) fprintf(o1,"        Match Percentage : %d%%\n", (100*no_mat)/al_len);
            (void) fprintf(o1,"      Number of Matches : %d\n", no_mat);
            (void) fprintf(o1,"      Number of Mismatches : %d\n", no_mis);
      	    (void) fprintf(o1,"         Total Length of Gaps : %d\n", al_len-no_mat-no_mis);
            (void) fprintf(o1,"      Begins at (%d, %d) and Ends at (%d, %d)\n",
	                stari,starj, endi,endj);
            display(&A[stari]-1,&B[starj]-1,rl,cl,S,stari,starj,o1); /*Display the anchor alignments*/
	    (void) fflush(stdout);

// Store the anchor alignments
	    ANCHS[NumAnchAdd]->sc = score;
            ANCHS[NumAnchAdd]->length = al_len;
            ANCHS[NumAnchAdd]->starti = stari;
            ANCHS[NumAnchAdd]->startj = starj;
            ANCHS[NumAnchAdd]->endi = endi;
            ANCHS[NumAnchAdd]->endj = endj;
	    store_anchor(&A[stari]-1,&B[starj]-1,rl,cl,S,stari,starj); /*storing the anchor matches in structures with arrays of residue numbers*/


	cur = LIST[numnode-(i+1)];/*iterate over anchors associated with maxi, stored in the end of LIST*/ 


	}/*end of anchor iteration*/
	fclose(o1);
	printf("<<<<<<<finished iteration over anchors>>>>>\n");
	printf("anchor score is %f\n", anchor_score);
	
/*		if ( count )
	      { flag = 0;
                locate(A,B,nseq);
                if ( flag )
		   small_pass(A,B,count,nseq);
              }*/     /*TO PICK ANOTHER OPTIMAL HIGH SCORING ALIGNMENT IF PRESENT*/

	//printf("<<<<<<<finished finding other optimal maxi>>>>>\n");

	    /*IF ANOTHER OPTIMAL MAXI IS FOUND, STORE THAT FOR ANOTHER POSSIBLE ALIGNMENT*/

		/*TO CHECK IF ANOTHER OPTIMAL MAXI IS PRESENT*/
/*	   cur = findmax(); *//*next maximim scoring anchor*/
//	   if (cur->SCORE >= maxi->SCORE - 2000) /*if its score is close to the MAX score*/	
//	   {
//		if (cur->ENDI < maxi->STARI && cur->ENDJ < maxi->STARJ) /**/
//			break;
//		else if (cur->STARI > maxi->ENDI && cur->STARJ > maxi->ENDJ)/**/
//			break;
		//else if ()
		/*two sets of anchors would be generated if there is another optimal alignment*/
		//printf("<<<<there exists another optimal maxi>>>>>\n");
//		count = 0;
//		flag_opt ++;

//	   }
//	   else break;	

	}
} // End of the main while loop for maxi optimization
	/*
	if (num_bnd != 0)
	{
	for ( i = 0; i < size1 ; i++ )
              for ( j = 0; j <= i ; j++ )
                { 
                  Vaa[alph1[i]][alph1[j]] = Vaa[alph1[j]][alph1[i]] = 0;
                }
	}
	*/
// Reinitialize the stored alignments
	for ( i = 1; i <= M; i++ )
          if ( nseq == 2 )
             row[i] = 0;
	IM = A;
	JN = B;
	IMe = A+M;
	JNe = B+N;

	q = gi; /*gap initiation penalty for global alignment*/
	r = ge; /*gap extension penalty for global alignment*/
	// Storing the new penalties
	//q = q - flag_optim*100;
	//r = r - flag_optim*100; 
	for(i=0;i<size;i++)
	{
		gin[alph[i]] = q;// + (q*ini_wt[alph[i]]);
		gext[alph[i]] = r;
	}
	qr = q;

	/*sorting the boundary coordinates based on row - since the boundaries are stored in a sorted manner from 0th row, this step is not necessary*/
	printf("\nnumber of anchors and bounds is %d %d\n\n", num_anch, num_bnd);
	ct_bnd = 0;

	printf("selecting anchors based on inter-anchor distance\n");
	if (num_bnd != 0) anchor_selection();

	for(i = 0; i < num_bnd; i++)
	{
		// reinitializing initiation and extension penalties
		for(j=0;j<size;j++)
	        {
			gin[alph[j]] = q;// + (q*ini_wt[alph[j]]);
			gext[alph[j]] = r;
		}
		qr = q;
		ct_bnd++;
		// finding the initial bound
		if (i == 0)
		{
			for(j = 1; j <= (num_bnd - ct_bnd); j++)
			{ 
				if (bounds[i]->i1 > bounds[i+j]->i1)
				{
					temp_bnd = bounds[i+j];
					bounds[i+j] = bounds[i];
					bounds[i] = temp_bnd;
				}
			}
		}
		bd = bounds[i];
                stari = bd->i1;
                starj = bd->j1;
                endi = bd->i2;
                endj = bd->j2;

		// finding the next bound
		if (i < (num_bnd - 2))
		{
		for(j = 1; j <= (num_bnd - (i+2)); j++)
                {
                        if (bounds[i+1]->i1 > bounds[i+j+1]->i1)
                        {
                                temp_bnd = bounds[i+j+1];
                                bounds[i+j+1] = bounds[i+1];
                                bounds[i+1] = temp_bnd;
                        }
                }
		}
		printf("number of bad anchors is %d\n", NumBadAnch);
		// Check if the successive anchor is a bad anchor
		flag_break = 0;
		while (flag_break == 0)
                {
		for (p=0;p<num_anch;p++)
                {
                        if (ANCHS[p]->starti > endi && ANCHS[p]->endi <= bounds[i+1]->i1)
			{
				for(j=0;j<NumBadAnch;j++)
				{
					if (BadAnch[j] == p)
					{
						bd->i2 = bounds[i+1]->i2;
						bd->j2 = bounds[i+1]->j2;
						// remove the next bound
						temp_bnd = bounds[num_bnd-1];
						bounds[num_bnd-1] = bounds[i+1];
						bounds[i+1] = temp_bnd;   
						num_bnd--;  // reduce the number of bounds
						flag_break = 1;
						break;
					}
				}

			}
			if (flag_break == 1) break;
		}
		if (flag_break == 1) // recalculating the i+1th bound
		{
		for(j = 1; j <= (num_bnd - (i+2)); j++)
                {
                        if (bounds[i+1]->i1 > bounds[i+j+1]->i1)
                        {
                                temp_bnd = bounds[i+j+1];
                                bounds[i+j+1] = bounds[i+1];
                                bounds[i+1] = temp_bnd;
                        }
                }
		flag_break = 0;
		}
		else break;
		stari = bd->i1;
                starj = bd->j1;
                endi = bd->i2;
                endj = bd->j2;
		}
		printf("\n\n<<<<<<<<<<<<<<<%d %d %d %d>>>>>>>>>>\n\n", stari, starj, endi, endj);

		printf("finding the anchor between the current and previous bound\n");
		// To generate output alignments  for the anchor regions
		// To generate the output alignments to be read by the MSA
		u = s1;
		for (p = 0;p < num_anch;p++)
		{
			if (ANCHS[p]->starti < stari && ANCHS[p]->starti >= bounds[i-1]->i2)
			{

				if (ANCHS[p]->ARi[0] >= ANCHS[p]->starti && ANCHS[p]->ARj[0] >= ANCHS[p]->startj) //to check if the alignment begins with a mismatch
				{
					seq1_out[s1++] = A[ANCHS[p]->ARi[0]];
					seq2_out[s2++] = B[ANCHS[p]->ARj[0]];
				}
				else if (ANCHS[p]->ARi[0] < ANCHS[p]->starti)
				{
					seq1_out[s1++] = '-';
       	        	                seq2_out[s2++] = B[ANCHS[p]->ARj[0]];
				}
				else if (ANCHS[p]->ARj[0] < ANCHS[p]->startj)
				{
					seq1_out[s1++] = A[ANCHS[p]->ARi[0]];
       		                        seq2_out[s2++] = '-';
				}
				for(e = 1; e < ANCHS[p]->length;e++)
				{
					if (ANCHS[p]->ARi[e] != ANCHS[p]->ARi[e-1])
					{
						if (ANCHS[p]->ARj[e] != ANCHS[p]->ARj[e-1])
						{
							seq1_out[s1++] = A[ANCHS[p]->ARi[e]];
							seq2_out[s2++] = B[ANCHS[p]->ARj[e]];
						}
						else 
						{
							seq1_out[s1++] = A[ANCHS[p]->ARi[e]];
                       		                        seq2_out[s2++] = '-';
						}
					}
					else  
					{
						seq1_out[s1++] = '-';
                        		        seq2_out[s2++] = B[ANCHS[p]->ARj[e]];
					}
				}
				v = s1;
				w = ANCHS[p]->starti;
				// write alignment of anchor positions
				fprintf(o3,">\n");
				for(d = u;d < v;d++)
				{
					if (seq1_out[d] != '-')
					{
						fprintf(o3,"%d ",w);
						w++;
					}
					else fprintf(o3,"-1 ");
				}
				fprintf(o3, "\n");
				w = ANCHS[p]->startj;
				for(d = u;d < v;d++)
                                {
                                        if (seq2_out[d] != '-')
					{
                                                fprintf(o3,"%d ",w);
                                                w++;
					}
					else fprintf(o3,"-1 ");
                                }
				fprintf(o3, "\n");
				break;

			}

		}

		I = stari - 1;
		J = starj - 1;
		rl = endi - stari + 1;
       		cl = endj - starj + 1;
		// difference in length
		rl1 = rl;
		cl1 = cl;
		if (rl1>cl1) diff_len = (rl1 - cl1)/rl1;
		else diff_len = (cl1 -rl1)/cl1;
		//if (diff_len > 0.7) diff_len = 0.7;
		
		for (j=0;j<size;j++)
		{ 
			gin[alph[j]] = gin[alph[j]]*exp(diff_len);
			//gext[alph[j]] = gext[alph[j]]*exp(1-diff_len);
		}
		
		/*
		else if (diff_len <= 0.5) // comparable
		{

		}
		else // similar
		{
		for (i=0;i<size;i++) gext[alph[i]] = gext[alph[i]] - 200;	
		}*/
		sapp = S;
		last = 0;
		al_len = 0;
		no_mat = 0;
		no_mis = 0;
		operations(stari,endi);
		printf("anchor operations are %d %d\n", op1,op2);
		printf("performing linker alignment\n");
		// compute the alignments in the linker regions 
		COUNT = 0; 
		(void) diff_L(&A[stari]-1,&B[starj]-1,&A1[stari]-1,&B1[starj]-1,rl,cl,gin[A[stari]],gin[A[endi]],COUNT);

		/* Since the alignment of linkers need not be saved in an output file, we remove the print output */
	 
		//(void) printf("\n*********************************************************\n");
     	        //(void) printf("      Number %d Local Alignment\n", K - count);
       	        //(void) printf("      Similarity Score : %d\n",score);
	        //(void) printf("      Match Percentage : %d%%\n", (100*no_mat)/al_len);
      		//(void) printf("      Number of Matches : %d\n", no_mat);
            	//(void) printf("      Number of Mismatches : %d\n", no_mis);
    		//(void) printf("      Total Length of Gaps : %d\n", al_len-no_mat-no_mis);
            	//(void) printf("      Begins at (%d, %d) and Ends at (%d, %d)\n",
                //        stari,starj, endi,endj);
            	/*display(&A[stari]-1,&B[starj]-1,rl,cl,S,stari,starj); *//*will be replaced with a similar function to store the output alignment*/
           	//(void) fflush(stdout);

		// Store the alignment in the linker regions, in arrays
		printf("storing linker alignment\n");
            	store_global(&A[1]-1,&B[1]-1,rl,cl,S,stari - 1,starj - 1,&A1[1]-1,&B1[1]-1);

	}

// IF THERE ARE NO OPTIMAL ANCHORS
//
	if (num_bnd == 0)
	{
		printf("there are no anchors !!!!!!!!! computing global alignment$$%d$$\n", num_bnd);
		q = 600;
		r = 600;
		qr = q;
		for(j=0;j<size;j++)
		{
			gin[alph[j]] = q;
			gext[alph[j]] = r;
		}
       		I = 0;
	        J = 0;
		stari = 1;
		starj = 1;
		endi = M;
		endj = N;
       		rl = endi - stari + 1;
	        cl = endj - starj + 1;
	        sapp = S;
	        last = 0;
   	        al_len = 0;
       		no_mat = 0;
        	no_mis = 0;
		COUNT = 0;
        	(void) diff_L(&A[stari]-1,&B[starj]-1,&A1[stari]-1,&B1[starj]-1,rl,cl,gin[A[1]],gin[A[M]],COUNT); // global alignment


		store_global(&A[1]-1,&B[1]-1,rl,cl,S,stari - 1,starj - 1,&A1[1]-1,&B1[1]-1);
	}

	printf("<<<<<<finished aligning global>>>>>\n"); 

	seq1_out[s1] = '\0';
	seq2_out[s2] = '\0';
	total_score = anchor_score + global_score;

	printf("<<<<<<finished aligning global, saving output %d %d>>>>>\n", s1,s2);

	//Storing the output alignment in a file 
	for(i=0;i<=s1;i++)
	{
		if (seq1_out[i] == '\0') 
		{
			printf("entered");
			fprintf(o2,"\n");
			break;
		}
		fprintf(o2,"%c",seq1_out[i]);
	}
	for(i=0;i<=s2;i++)
        {
                if (seq2_out[i] == '\0') 
                {
			printf("finishedddd");
                        fprintf(o2,"\n");
                        break;
                }
                fprintf(o2,"%c",seq2_out[i]);
        }
	fprintf(o2,"raw score: ");
	fprintf(o2,"%f: ",total_score);
	fprintf(o2,"%f: ",(total_score/s1));
	fprintf(o2,"%d\n",s1);
	fclose(o2);

}

/* A big pass to compute K best classes */

big_pass(A,B,A1,B1,M,N,K,nseq) char A[],B[],A1[],B1[]; int M,N,K,nseq;
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  int  *va, c_max;				/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */

	
	/* Compute the matrix and save the top K best scores in LIST
	   CC : the scores of the current row
	   RR and EE : the starting point that leads to score CC
	   DD : the scores of the current row, ending with deletion
	   SS and FF : the starting point that leads to score DD        */
 	/* Initialize the 0 th row */
	for ( j = 1; j <= N ; j++ )
	  {  CC[j] = 0;
	     RR[j] = 0;
	     EE[j] = j;
	     DD[j] = - (q + (q/2)); // To account for the additional PB specific weight
	     SS[j] = 0;
	     FF[j] = j;
	  }
	for ( i = 1; i <= M; i++) 
	  {  c = 0;				/* Initialize column 0 */
	     f = - (q+(q/2));  // to account for the PB specific weight
	     ci = fi = i;
	     va = v[A[i]];
	     if ( nseq == 2 )
	       { p = 0;
	         pi = i - 1;
	         cj = fj = pj = 0;
	       }
	     else
	       { p = CC[i];
		 pi = RR[i];
		 pj = EE[i];
	         cj = fj = i;
	       }
	     for ( j = (nseq == 2 ? 1 : (i+1)) ; j <= N ; j++ )  
	       {  f = f - gext[B[j]]; //
		  c = c - gin[B[j]]; //
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - gin[A[i]]; // 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - gext[A[i]]; //
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(i, j, c, (p+va[B[j]]+Vaa[A1[i]][B1[j]]))		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  c_max = 3000;	
		  if ( c > min && c > c_max)	/* add the score into list ; ADDING A THRESHOLD FOR INCLUDING IN THE LIST, THIS CAN BE A PROTEIN DEPENDANT INPUT*/
		    min = addnode(c, ci, cj, i, j, 100, min); /*K=100 TO COVER MOST OF THE OPTIMAL PATHS*/
		  /*SORT LIST BASED ON SCORE*/
		  /*CHECK IF ANY TWO HAVE SIMILAR SCORES*/
		  /**/		
		  /*??FINDMAX OF LIST AND NOTE THE BOUNDARIES*/
		  /*LOOK FOR THE OPTIMAL ANCHORS*/		
	        }
	  }
}

/* Determine the left and top boundaries of the recomputed area */

locate(A,B,nseq) char A[],B[]; int nseq;
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  short  cflag, rflag;			/* for recomputation */
  int  *va;				/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */
  int  limit;				/* the bound on j */

	/* Reverse pass
	   rows
	   CC : the scores on the current row
	   RR and EE : the endpoints that lead to CC
	   DD : the deletion scores 
	   SS and FF : the endpoints that lead to DD

	   columns
	   HH : the scores on the current columns
	   II and JJ : the endpoints that lead to HH
	   WW : the deletion scores
	   XX and YY : the endpoints that lead to WW
	*/
	for ( j = nn; j >= n1 ; j-- )
          {  CC[j] = 0;
	     EE[j] = j;
	     DD[j] = - (q);
	     FF[j] = j;
	     if ( nseq == 2 || j > mm )
                RR[j] = SS[j] = mm + 1;
	     else
                RR[j] = SS[j] = j;
	  }

        for ( i = mm; i >= m1; i-- )
	  {  c = p = 0;
	     f = - (q);
	     ci = fi = i;
	     pi = i + 1;
	     cj = fj = pj = nn + 1;
	     va = v[A[i]];
	     if ( nseq == 2 || n1 > i )
		limit = n1;
	     else
		limit = i + 1;
	     for ( j = nn; j >= limit ; j-- )  
	       {  f = f - r;
		  c = c - qr;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - qr; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - r;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(i, j, c, p+va[B[j]])		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > min )
		    flag = 1;
	        }
	     if ( nseq == 2 || i < n1 )
	       { HH[i] = CC[n1];
	         II[i] = RR[n1];
	         JJ[i] = EE[n1];
	         WW[i] = f;
	         XX[i] = fi;
	         YY[i] = fj;
	       }
	  }
      
  for ( rl = m1, cl = n1; ; )
    { for ( rflag = cflag = 1; ( rflag && m1 > 1 ) || ( cflag && n1 > 1 ) ;  )
        { if ( rflag && m1 > 1 )	/* Compute one row */
            { rflag = 0;
	      m1--;
      	      c = p = 0;
	      f = - (q);
	      ci = fi = m1;
	      pi = m1 + 1;
	      cj = fj = pj = nn + 1;
	      va = v[A[m1]];
	      for ( j = nn; j >= n1 ; j-- )  
	        { f = f - r;
		  c = c - qr;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - qr; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - r;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(m1, j, c, p+va[B[j]])		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = m1; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > min )
		     flag = 1;
		  if ( ! rflag && ( ci > rl && cj > cl || di > rl && dj > cl
	 		                            || fi > rl && fj > cl ) )
		      rflag = 1;
	        }
	      HH[m1] = CC[n1];
	      II[m1] = RR[n1];
	      JJ[m1] = EE[n1];
	      WW[m1] = f;
	      XX[m1] = fi;
	      YY[m1] = fj;
	      if ( ! cflag && ( ci > rl && cj > cl || di > rl && dj > cl
			     || fi > rl && fj > cl ) )
	         cflag = 1;
	    }

	  if ( nseq == 1 && n1 == (m1 + 1) && ! rflag )
	     cflag = 0;
	  if ( cflag && n1 > 1 )	/* Compute one column */
	    { cflag = 0;
	      n1--;
	      c = 0;
	      f = - (q);
	      cj = fj = n1;
	      va = v[B[n1]];
	      if ( nseq == 2 || mm < n1 )
		{ p = 0;
	          ci = fi = pi = mm + 1;
	          pj = n1 + 1;
		  limit = mm;
		}
	      else
		{ p = HH[n1];
		  pi = II[n1];
		  pj = JJ[n1];
	          ci = fi = n1;
		  limit = n1 - 1;
		}
	      for ( i = limit; i >= m1 ; i-- )  
	        { f = f - r;
		  c = c - qr;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = HH[i] - qr; 
		  ci = II[i];
		  cj = JJ[i];
		  d = WW[i] - r;
		  di = XX[i];
		  dj = YY[i];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
	          DIAG(i, n1, c, p+va[A[i]])
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = n1; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = HH[i];
		  HH[i] = c;
		  pi = II[i];
		  pj = JJ[i];
		  II[i] = ci;
		  JJ[i] = cj;
		  WW[i] = d;
		  XX[i] = di;
		  YY[i] = dj;
		  if ( c > min )
		     flag = 1;
	          if ( ! cflag && ( ci > rl && cj > cl || di > rl && dj > cl
		               || fi > rl && fj > cl ) )
		     cflag = 1;
	        }
	      CC[n1] = HH[m1];
	      RR[n1] = II[m1];
	      EE[n1] = JJ[m1];
	      DD[n1] = f;
	      SS[n1] = fi;
	      FF[n1] = fj;
	      if ( ! rflag && ( ci > rl && cj > cl || di > rl && dj > cl
		                                 || fi > rl && fj > cl ) )
	         rflag = 1;
	    }
	}
      if ( m1 == 1 && n1 == 1 || no_cross() )
	 break;
   }
  m1--;
  n1--;
}

/* recompute the area on forward pass */
small_pass(A,B,count,nseq) char A[], B[]; int count, nseq;
{ register  int  i, j;			/* row and column indices */
  register  int  c;			/* best score at current point */
  register  int  f;			/* best score ending with insertion */
  register  int  d;			/* best score ending with deletion */
  register  int  p;			/* best score at (i-1, j-1) */
  register  int  ci, cj;		/* end-point associated with c */ 
  register  int  di, dj;		/* end-point associated with d */
  register  int  fi, fj;		/* end-point associated with f */
  register  int  pi, pj;		/* end-point associated with p */
  int  *va;				/* pointer to v(A[i], B[j]) */
  int   addnode();			/* function for inserting a node */
  int  limit;				/* lower bound on j */

	for ( j = n1 + 1; j <= nn ; j++ )
	  {  CC[j] = 0;
	     RR[j] = m1;
	     EE[j] = j;
	     DD[j] = - (q);
	     SS[j] = m1;
	     FF[j] = j;
	  }
	for ( i = m1 + 1; i <= mm; i++) 
	  {  c = 0;				/* Initialize column 0 */
	     f = - (q);
	     ci = fi = i;
	     va = v[A[i]];
	     if ( nseq == 2 || i <= n1 )
	       { p = 0;
	         pi = i - 1;
	         cj = fj = pj = n1;
		 limit = n1 + 1;
	       }
	     else
	       { p = CC[i];
		 pi = RR[i];
		 pj = EE[i];
	         cj = fj = i;
		 limit = i + 1;
	       }
	     for ( j = limit ; j <= nn ; j++ )  
	       {  f = f - r;
		  c = c - qr;
		  ORDER(f, fi, fj, c, ci, cj)
		  c = CC[j] - qr; 
		  ci = RR[j];
		  cj = EE[j];
		  d = DD[j] - r;
		  di = SS[j];
		  dj = FF[j];
		  ORDER(d, di, dj, c, ci, cj)
		  c = 0;
		  DIAG(i, j, c, p+va[B[j]])		/* diagonal */
		  if ( c <= 0 )
		    { c = 0; ci = i; cj = j; }
		  else
		    { ci = pi; cj = pj; }
		  ORDER(c, ci, cj, d, di, dj)
		  ORDER(c, ci, cj, f, fi, fj)
		  p = CC[j];
		  CC[j] = c;
		  pi = RR[j];
		  pj = EE[j];
		  RR[j] = ci;
		  EE[j] = cj;
		  DD[j] = d;
		  SS[j] = di;
		  FF[j] = dj;
		  if ( c > min )	/* add the score into list */
		    min = addnode(c, ci, cj, i, j, count, min);
	        }
	  }
}

/* Add a new node into list.  */

int addnode(c, ci, cj, i, j, K, cost)  int c, ci, cj, i, j, K, cost;
{ short found;				/* 1 if the node is in LIST */
  register int d;

  found = 0;
  if ( most != 0 && most->STARI == ci && most->STARJ == cj )
    found = 1;
  else
     for ( d = 0; d < numnode ; d++ )
	{ most = LIST[d];
	  if ( most->STARI == ci && most->STARJ == cj )
	    { found = 1;
	      break;
	    }
        }
  if ( found )
    { if ( most->SCORE < c )
        { most->SCORE = c;
          most->ENDI = i;
          most->ENDJ = j;
        }
      if ( most->TOP > i ) most->TOP = i;
      if ( most->BOT < i ) most->BOT = i;
      if ( most->LEFT > j ) most->LEFT = j;
      if ( most->RIGHT < j ) most->RIGHT = j;
    }
  else
    { if ( numnode == K )	/* list full */
	 most = low;
      else
         most = LIST[numnode++];
      most->SCORE = c;
      most->STARI = ci;
      most->STARJ = cj;
      most->ENDI = i;
      most->ENDJ = j;
      most->TOP = most->BOT = i;
      most->LEFT = most->RIGHT = j;
    }
  if ( numnode == K )
    { if ( low == most || ! low ) 
        { for ( low = LIST[0], d = 1; d < numnode ; d++ )
            if ( LIST[d]->SCORE < low->SCORE )
              low = LIST[d];
	}
      return ( low->SCORE ) ;
    }
  else
    return cost;
}

/* Find and remove the largest score in list */

vertexptr findmax()
{ vertexptr  cur;
  register int i, j,d;

  for ( j = 0, i = 1; i < numnode ; i++ )
    if ( LIST[i]->SCORE > LIST[j]->SCORE )
       j = i;
  cur = LIST[j];
  if ( j != --numnode )
    { LIST[j] = LIST[numnode];
      LIST[numnode] =  cur;
    }
  most = LIST[0];
  if ( low == cur ) low = LIST[0];
  for ( low = LIST[0], d = 1; d < numnode ; d++ )
            if ( LIST[d]->SCORE < low->SCORE )
              low = LIST[d];
        
  return ( cur );
}
// FIND ALL NON OVERLAPPING ANCHORS IN THE FLANKING REGIONS OF MAX
max_others(max,M,N) vertexptr max;int M,N;
{ 
vertexptr  cur;
register int i, j, p;
int anchors[50];
register int e1i,e1j,s2i,s2j,s1i,s1j,e2i,e2j;
register int l,b,k,t1,t2,t3,t4;
cur = max;
e1i = cur->STARI;
e1j = cur->STARJ;
s2i = cur->ENDI + 1;
s2j = cur->ENDJ + 1;
s1i = 1;
s1j = 1;
e2i = M;
e2j = N;
j = 0;
num_anch = 1;
//printf("\néééééé%d %d %d %dééééééé\n", e1i,e1j,s2i,s2j);
/* Initialize the boundary list*/
num_bnd = 2;
bounds[0]->i1 = s1i;
bounds[0]->j1 = s1j;
bounds[0]->i2 = e1i;
bounds[0]->j2 = e1j;
bounds[1]->i1 = s2i;
bounds[1]->j1 = s2j;
bounds[1]->i2 = e2i;
bounds[1]->j2 = e2j;
// Find new anchors and bounds iteratively
for (l = 0; l < num_bnd; l++)
	{
		b = best_in_bounds(bounds[l]);
		//printf("%d\n", l);
		if (b != -1)
		{
			anchors[j++] = b;
			num_anch ++;
			//printf("anchor1\n");
			//printf("%d %d\n", num_bnd, l);
			t1 = bounds[l]->i2;
			t2 = bounds[l]->j2;
			bounds[num_bnd]->i2 = t1;
			bounds[num_bnd]->j2 = t2;
			bounds[l]->i2 = LIST[b]->STARI;
			bounds[l]->j2 = LIST[b]->STARJ;
			bounds[num_bnd]->i1 = LIST[b]->ENDI + 1;
			bounds[num_bnd]->j1 = LIST[b]->ENDJ + 1;
			//printf("%d %d %d %d\n", bounds[l]->i1, bounds[l]->i2, bounds[l]->j1, bounds[l]->j2);
			num_bnd++;
			l--;
		}
		else
		{
			//printf("no anchor found %d\n", num_bnd);
			/*bounds[l]->i1 = bounds[--num_bnd]->i1;
			bounds[l]->i2 = bounds[num_bnd]->i2;
			bounds[l]->j1 = bounds[num_bnd]->j1;
			bounds[l]->j2 = bounds[num_bnd]->j2; 
			l--;
			printf("%d\n", num_bnd);*/
		}
		
	}
for (i = 1;i < num_anch;i++)
{
	k = anchors[i-1];
	cur =  LIST[k];
	LIST[k] = LIST[numnode-i];
	LIST[numnode-i] = cur;
}
}
// best anchor in a rectangular region bounded by i1,i2,j1,j2
best_in_bounds(bound) bnd bound;
{
vertexptr max;
int I1,I2,J1,J2,count,i,b;
I1 = bound->i1;
I2 = bound->i2;
J1 = bound->j1;
J2 = bound->j2;
max = low;
//printf("$$$%d %d %d %d$$$\n", I1,I2,J1,J2);
count = 0;
for(i = 0;i < numnode; i++)
        {
		//printf("%d\n", LIST[i]->SCORE);
                if (LIST[i]->STARI >= I1 && LIST[i]->ENDI <= I2 && LIST[i]->STARJ >= J1 && LIST[i]->ENDJ <= J2) /*TOP RECTANGLE*/
                {
                        if (LIST[i]->SCORE > max->SCORE)
                        {
                                max = LIST[i];
				b = i;
				count++;
                        }
                }
	}
if (count == 0) return -1;
return b;

}


/*Checking if the anchors associated with the maxi are structurally compatible*/
/*
int distance(anch_no) int anch_no
{
// Centre of main anchor: MIDI, MIDJ; can correspond to a match; 
	// get the mid points (a central match)
	// calculate distance between i1 and i2	
	f = coordinates[0][i1];
	s = coordinates[0][i2];
	d1 = sqrt(pow((f->x - s->x),2)+pow((f->y - s->y),2)+pow((f->z - s->z),2))
	f = coordinates[1][j1];
	s = coordinates[1][j2];
	d2 = sqrt(pow((f->x - s->x),2)+pow((f->y - s->y),2)+pow((f->z - s->z),2))
	
	if ((d1 - d2) > 0 && (d1 - d2) > 10)
	{
		return 0
	}
	else if ((d1 - d2) < 0 && (d1 - d2) < -10)
	{
		return 0
	}
	else return 1

}
*/

//Function to identify two anchors in the flanking boundaries of an anchor
/*vertexptr flanking_best(s1i,s1j,e1i,e1j,s2i,s2j,e2i,e2j) 
vertexptr max1,max2
int s1i,s1j,e1i,e1j,s2i,s2j,e2i,e2j, count1 =0,count2 = 0;
{
	max1 = low;
	max2 = low;
	for(i = 0;i < numnode; i++)
        {
                if (LIST[i]->STARI > s1i && LIST[i]->ENDI < e1i && LIST[i]->STARJ > s1j && LIST[i]->ENDJ < e1j) TOP RECTANGLE
                {
                        if (LIST[i]->SCORE > max1->SCORE)
                        {
                                max1 = LIST[i];
                                i1 = i;
                                count1 = 1;
                        }
                }
                if (LIST[i]->STARI > s2i && LIST[i]->STARJ > s2j and LIST[i]->ENDI < e2i && LIST[i]->ENDJ < e2j) BOTTOM RECTANGLE
                {
                        if (LIST[i]->SCORE > max2->SCORE)
                        {
                                max2 = LIST[i];
                                i2 = i;
                                count2 = 1;
                        }
                }
	}
	if count1 == 0: max1 = low;
	if count2 == 0: max2 = low;
	return(max1,max2)
}*/
// Sorting the LIST nodes based on scores: not checked
/*sortlist()
{
	vertexptr  cur;
	register int i, j, count;
	numnode_copy = numnode;
	for ( j = 0; i = 1; i < numnode_copy; i++ )
	{
		if ( LIST[i]->SCORE > LIST[j]->SCORE )
		{
			j = i;
		}
		cur = LIST[j];
		if ( j != --numnode_copy)
    			{ 
				LIST[j] = LIST[numnode_copy];
      				LIST[numnode_copy] =  cur;
				i = 1;
				j = 0;
    			}
	}
	
}
*/


/* return 1 if no node in LIST share vertices with the area */

no_cross()
{ vertexptr  cur;
  register int i;

      for ( i = 0; i < numnode; i++ )
	{ cur = LIST[i];
	  if ( cur->STARI <= mm && cur->STARJ <= nn && cur->BOT >= m1-1 && 
	       cur->RIGHT >= n1-1 && ( cur->STARI < rl || cur->STARJ < cl ))
	     { if ( cur->STARI < rl ) rl = cur->STARI;
	       if ( cur->STARJ < cl ) cl = cur->STARJ;
	       flag = 1;
	       break;
	     }
	}
      if ( i == numnode )
	return 1;
      else
	return 0;
}

/* diff(A,B,M,N,tb,te) returns the score of an optimum conversion between
   A[1..M] and B[1..N] that begins(ends) with a delete if tb(te) is zero
   and appends such a conversion to the current script.                   */

int diff(A,B,A1,B1,M,N,tb,te) char *A, *B, *A1, *B1; int M, N; float tb, te;

{ int   midi, midj, type;	/* Midpoint, type, and cost */
  int midc;

{ register int   i, j;
  register int c, e, d, s;
           int t, *va;
  	   char  *ckalloc();
	   int gap_specific();	
/* Boundary cases: M <= 1 or N == 0 */
  if (N <= 0)
    { if (M > 0) DEL(M)
      return - gap_specific(A,M); // q + r(M-1)
    }
  if (M <= 1)
    { if (M <= 0)
        { INS(N);
          return - gap_specific(B,N);
        }
      if (tb > te) tb = te;
      midc = - (tb + gap_specific(B,N) );// r removed as the initiation penalty is only q
      midj = 0;
      va = v[A[1]];
      for (j = 1; j <= N; j++)
        {  
	for ( tt = 1, z = row[I+1]; z != 0; z = z->NEXT )	
              if ( z->COL == j+J )			
	         { tt = 0; break; }
           if ( tt )			
            { c = (va[B[j]]+Vaa[A1[1]][B1[j]]) - ( gap_specific(B,j-1) + gap_specific(B+j,N-j) );
              if (c > midc)
               { midc = c;
                 midj = j;
               }
	    }
	}
      if (midj == 0)
        { INS(N) DEL(1) }
      else
        { if (midj > 1) INS(midj-1)
          REP
	  if ( A[1] == B[midj] || isdna && va[B[midj]] == MATCHSC )
	     no_mat += 1;
	  else
	     no_mis += 1;
	  /* mark (A[I],B[J]) as used: put J into list row[I] */	
          I++; J++;
	  z = ( pairptr ) ckalloc( (int) sizeof(pair));
          z->COL = J;			
          z->NEXT = row[I];				
	  row[I] = z;
          if (midj < N) INS(N-midj)
        }
      return midc;
    }
//	printf("forward pass\n");
/* Divide: Find optimum midpoint (midi,midj) of cost midc */

  midi = M/2;			/* Forward phase:                          */
  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
  t = -gin[B[1]]; // q removed
  // Initializing first row
  for (j = 1; j <= N; j++)
    { CC[j] = t;
      DD[j] = t-(gext[B[j]]); 
      t = t - gext[B[j]]; // -r removed //
    }
  t = -tb; // initiation penalty transfered from previous run
  for (i = 1; i <= midi; i++)
    { s = CC[0];  // Note that initiation penalty, considering the previous run, is passed through CC[0]
      CC[0] = c = t;// -r removed 
      e = t-gin[B[1]];	 
      t = t - gext[A[i]];
      va = v[A[i]];
      for (j = 1; j <= N; j++)
        { if ((c = c - gin[B[j]]) > (e = e - gext[B[j]])) e = c;
          if ((c = CC[j] - gin[A[i]]) > (d = DD[j] - gext[A[i]])) d = c;
	  DIAG(i+I, j+J, c, s+(va[B[j]]+Vaa[A1[i]][B1[j]]))
          if (c < d) c = d;
          if (c < e) c = e;
          s = CC[j];
          CC[j] = c;
          DD[j] = d;
        }
    }
  DD[0] = CC[0];

  RR[N] = 0;			/* Reverse phase:                          */
  t = -gin[B[N]];
//	printf("reverse pass\n");
  // Initializing last row			/*   Compute R(M/2,k) & S(M/2,k) for all k */
  for (j = N-1; j >= 0; j--)
    { RR[j] = t;
      SS[j] = t-(gext[B[j+1]]);	
      t = t - gext[B[j+1]]; // -r removed
    }
  t = -te;
  for (i = M-1; i >= midi; i--)
    { s = RR[N];
      RR[N] = c = t; // -r removed
      e = t-gin[B[N]];	
      t = t-gext[A[i+1]];
      va = v[A[i+1]];
      for (j = N-1; j >= 0; j--)
        { if ((c = c - gin[B[j+1]]) > (e = e - gext[B[j+1]])) e = c;
          if ((c = RR[j] - gin[A[i+1]]) > (d = SS[j] - gext[A[i+1]])) d = c;
	  DIAG(i+1+I, j+1+J, c, (s+va[B[j+1]]+Vaa[A1[i+1]][B1[j+1]]))
          if (c < d) c = d;
          if (c < e) c = e;
          s = RR[j];
          RR[j] = c;
          SS[j] = d;
        }
    }
  SS[N] = RR[N];

  midc = CC[0]+RR[0];		/* Find optimal midpoint */
  midj = 0;
  type = 1;
  for (j = 0; j <= N; j++)
    if ((c = CC[j] + RR[j]) >= midc)
      if (c > midc || CC[j] != DD[j] && RR[j] == SS[j])
        { midc = c;
          midj = j;
        }
  for (j = N; j >= 0; j--)
    if ((c = DD[j] + SS[j] + gin[A[midi]] - gext[A[midi]]) > midc) // -r included to consider the double initiation as an initiation and extension
      { midc = c;
        midj = j;
        type = 2;
      }
//if (count == 2){MIDI=midi+sti-1;MIDJ=midj+stj-1;TYPE=type;}//printf("*%c %c*\n",A[midi-2],B[midj-2]);}
}

// tb and te to be modified for the midpoint PB


/* Conquer: recursively around midpoint */

  if (type == 1)
    { (void) diff(A,B,A1,B1,midi,midj,gin[A[1]],gin[A[midi]]);
      (void) diff(A+midi,B+midj,A1+midi,B1+midj,M-midi,N-midj,gin[A[midi+1]],gin[A[M]]);
    }
  else
    { (void) diff(A,B,A1,B1,midi-1,midj,gin[A[1]],zero); // zero initiation penalty for reverse passes
      DEL(2);
      (void) diff(A+midi+1,B+midj,A1+midi+1,B1+midj,M-midi-1,N-midj,zero,gin[A[M]]); // zero initiation penalty for forward passes
    }
  return midc;
}

// function to extract boundary operations
operations(sti,ei) int sti,ei;
{
int i,len;
op1 = 0;
op2 = 0;	
for (i = 0; i < num_anch; i++)
 {
        len = ANCHS[i]->length;
        if (num_bnd == 0);
        if (ANCHS[i]->endi == sti - 1) // starting bound
        {
                 if (ANCHS[i]->ARi[len-1] != ANCHS[i]->ARi[len-2])
                {
                        if (ANCHS[i]->ARj[len-1] != ANCHS[i]->ARj[len-2]) op1 = 0; // match
                        else op1 = 1; // seq1 deleted 
                }
                else
                {
                        op1 = 2; // seq2 insertion
                }
        }


        if (ANCHS[i]->starti == ei + 1) // end bound anchor
        {
                if (ANCHS[i]->ARi[1] != ANCHS[i]->ARi[0])
                {
                        if (ANCHS[i]->ARj[1] != ANCHS[i]->ARj[0]) op2 = 0; // match
                        else op2 = 1;
                }
                else
                {
                        op2 = 2;
                }
        }
 }
 printf("µµ%d %dµµ\n", op1,op2);	
}

// function for aligning linker regions between anchors

int diff_L(A,B,A1,B1,M,N,tb,te,COUNT) char *A, *B, *A1, *B1; int M, N, COUNT; float tb, te;
// start and end points are called to find the corresponding anchors, global start and end points are called to perform semiglobal alignments
{ int   midi, midj, type;	/* Midpoint, type, and cost */
  int midc;

{ register int   i, j;
  register int c, e, d, s;
           int t, *va,len;
  	   char  *ckalloc();
	   char *i1,*i2,*j1,*j2;
	   
// extract the corresponding coordinates: i1,j1, i2,j2 & the operations 
//
/* Boundary cases: M <= 1 or N == 0 */
// saving linker sequence bounds 
 if (COUNT == 0){i1 = A; i2 = A + M; j1 = B; j2 = B + N;} // Storing the bounds of the linker
// Note that the end conditions involving operations from anchors are not included now !!!!!!!
 COUNT++;
  if (N <= 0)
    { if (M > 0) DEL(M)
 	if (A == i1 && op1 == 1){return (- gap_specific(A,M) + gin[A[1]] - gext[A[1]]);} // if deletion is already initiated
        if (A == IM && num_bnd != 0){return 0;} // initializing first coumn to zero, for N terminus
	return - gap_specific(A,M);
      //return - gap(M); // q + r(M-1)
    }
  if (M <= 1)
    { if (M <= 0)
        { INS(N);
 	  if (B == j1 && op1 == 2){return (- gap_specific(B,N) + gin[B[1]] - gext[B[1]]);} // if insertion is already initiated
          if (B == JN && num_bnd != 0){return 0;} // initializing first coumn to zero, for N terminus
	  return - gap_specific(B,N);	
          //return - gap(N);
        }
	// if M == 1 and N != 0 :=> not at the global boundary
      if (tb > te) tb = te;
      midc = - (tb + gap_specific(B,N));// r removed as the initiation penalty is only q
      if (B == j1 && op1 == 2) {midc = midc + gin[B[1]] - gext[B[1]];}
      if (B == JN && num_bnd != 0){midc = midc + gin[B[1]];}		
      midj = 0;
      va = v[A[1]];
      for (j = 1; j <= N; j++)
        {  for ( tt = 1, z = row[I+1]; z != 0; z = z->NEXT )
              if ( z->COL == j+J )			
	         { tt = 0; break; }
           if ( tt )			
            { c = (va[B[j]]+Vaa[A1[1]][B1[j]]) - ( gap_specific(B,j-1) + (gap_specific(B+j,N-j)) );
		if (B == j1 && op1 == 2) {c = c + gin[B[1]] - gext[B[1]];}
                if (B == JN && num_bnd != 0){c = c + gin[B[1]];}
              if (c > midc)
               { midc = c;
                 midj = j;
               }
	    }
	}
      if (midj == 0)
        { INS(N) DEL(1) }
      else
        { if (midj > 1) INS(midj-1)
          REP
	  if ( A[1] == B[midj] || isdna && va[B[midj]] == MATCHSC )
	     no_mat += 1;
	  else
	     no_mis += 1;
	  /* mark (A[I],B[J]) as used: put J into list row[I] */	
          I++; J++;
	  z = ( pairptr ) ckalloc( (int) sizeof(pair));
          z->COL = J;			
          z->NEXT = row[I];				
	  row[I] = z;
          if (midj < N) INS(N-midj)
        }
      return midc;
    }
/* Divide: Find optimum midpoint (midi,midj) of cost midc */

  midi = M/2;			/* Forward phase:                          */
  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
  t = -gin[B[1]];
// Initializing the first row
  for (j = 1; j <= N; j++)
    { 
	CC[j] = t; // -r removed
	if (A == i1 && j == 1 && op1 == 2) CC[j] = -gext[B[j]]; // if insertion is already initiated
	if (A == IM && B == JN && num_bnd != 0) {CC[j] = 0; printf("alignment in the first linker\n");}// if it is N terminus, first row is initialized to zero
	 DD[j] = t-gext[B[j]];
	t = t - gext[B[j]];
    }
  t = -tb;
  for (i = 1; i <= midi; i++)
    { s = CC[0];
	// if (A == i1 && j == j1 && o1 == 0){s = 100;} need not be considered?? 
// Initializing first column
      CC[0] = c = t;// -r removed 
	 if (A == i1 && i == 1 && op1 == 1){CC[0] = c = 0;} // if deletion is already initiated
	 if (B == JN && A == IM && num_bnd != 0){CC[0] = c = 0;} // initializing first coumn to zero, for N terminus
      e = t-gin[B[1]];
      t = t - gext[A[i]];
      va = v[A[i]];
      for (j = 1; j <= N; j++)
        {
	//if (A == i1 && i == 1 && j == 1 && o1 == 2){if ((c = c - r) > (e = e - r)) e = c;}  
	  if ((c = c - gin[B[j]]) > (e = e - gext[B[j]])) e = c; // to be altered for specific initiation preceded by insertion
	
          if ((c = CC[j] - gin[A[i]]) > (d = DD[j] - gext[A[i]])) d = c;
	  DIAG(i+I, j+J, c, (s+va[B[j]]+Vaa[A1[i]][B1[j]]))
          if (c < d) c = d;
          if (c < e) c = e;
          s = CC[j]; // previous score (i-1,j-1)
          CC[j] = c;
          DD[j] = d;
        }
    }
  DD[0] = CC[0];

  RR[N] = 0;			/* Reverse phase:                          */
  t = -gin[B[N]];			/*   Compute R(M/2,k) & S(M/2,k) for all k */
  for (j = N-1; j >= 0; j--)
    { RR[j] = t; 
        if (A+M == i2 && j == N-1 && op2 == 2) RR[j] = -gext[B[j+1]]; // if insertion is already initiated
        if (A+M == IMe && B+N == JNe && num_bnd != 0) RR[j] = 0; // if it is C terminus, first row is initialized to zero
      SS[j] = t-gext[B[j+1]];	
      t	= t - gext[B[j+1]]; // -r removed
    }
  t = -te;
  for (i = M-1; i >= midi; i--)
    { s = RR[N];
      RR[N] = c = t; // -r removed
         if (A+M == i2 && i == M-1 && op2 == 1){RR[N] = c = 0;} // if deletion is already initiated
         if (B+N == JNe && A+M == IMe && num_bnd != 0){RR[N] = c = 0;} // initializing first coumn to zero, for C terminus
      e = t-gin[B[1]];	
      t = t-gext[A[i+1]];
      va = v[A[i+1]];
      for (j = N-1; j >= 0; j--)
        { if ((c = c - gin[B[j+1]]) > (e = e - gext[B[j+1]])) e = c;
          if ((c = RR[j] - gin[A[i+1]]) > (d = SS[j] - gext[A[i+1]])) d = c;
	  DIAG(i+1+I, j+1+J, c, s+va[B[j+1]]+Vaa[A1[i+1]][B1[j+1]])
          if (c < d) c = d;
          if (c < e) c = e;
          s = RR[j];
          RR[j] = c;
          SS[j] = d;
        }
    }
  SS[N] = RR[N];

  midc = CC[0]+RR[0];		/* Find optimal midpoint */
  midj = 0;
  type = 1;
  for (j = 0; j <= N; j++)
    if ((c = CC[j] + RR[j]) >= midc)
      if (c > midc || CC[j] != DD[j] && RR[j] == SS[j])
        { midc = c;
          midj = j;
        }
  for (j = N; j >= 0; j--)
    if ((c = DD[j] + SS[j] + gin[A[midi]] - gext[A[midi]]) > midc) // -r included to consider the double initiation as an initiation and extension
      { midc = c;
        midj = j;
        type = 2;
      }
}
/* Conquer: recursively around midpoint */

  if (type == 1)
    { (void) diff_L(A,B,A1,B1,midi,midj,gin[A[1]],gin[A[midi]],COUNT);
      (void) diff_L(A+midi,B+midj,A1+midi,B1+midj,M-midi,N-midj,gin[A[midi+1]],gin[A[M]],COUNT);
    }
  else
    { (void) diff_L(A,B,A1,B1,midi-1,midj,gin[A[1]],zero,COUNT);
      DEL(2);
      (void) diff_L(A+midi+1,B+midj,A1+midi+1,B1+midj,M-midi-1,N-midj,zero,gin[A[M]],COUNT);
    }
  return midc;
}

int gap_specific(A,M) char *A;int M;
{
	int pen_op=0,pen_ext=0,pen=0,i;
//	printf("%d\n", M);
	if (M == 0) return 0;
	pen_op = gin[A[1]];
	if (M == 1) return pen_op;
//	printf("%d\n", pen_op);
	for(i=2;i<=M;i++)
	{
		pen_ext += gext[A[i]];
	}
	pen = pen_op+pen_ext;
//	printf("%d\n", pen);
	return pen;
}

// Function to align sequences using semi-global method

int diff_G(A,B,M,N,tb,te,count,sti,stj) char *A, *B; int M, N; int tb, te,count;

{ int   midi, midj, type;	/* Midpoint, type, and cost */
  int midc;

{ register int   i, j;
  register int c, e, d, s;
           int t, *va;
  	   char  *ckalloc();
// store the boundary coordinates
// if (count == 0) {IM = A; IMe = A+M; JN = B; JNe = B+N;}
/* Boundary cases: M <= 1 or N == 0 */
 count++;
  //printf("%d", count);
  if (N <= 0)
    { if (M > 0) DEL(M)
      return - gap(M); // q + r(M-1)
    }
  if (M <= 1)
    { if (M <= 0)
        { INS(N);
          return - gap(N);
        }
      if (tb > te) tb = te;
      midc = - (tb + gap(N) );// r removed as the initiation penalty is only q
      midj = 0;
      va = v[A[1]];
      for (j = 1; j <= N; j++)
        {  for ( tt = 1, z = row[I+1]; z != 0; z = z->NEXT )	
              if ( z->COL == j+J )			
	         { tt = 0; break; }		
           if ( tt )			
            { c = va[B[j]] - ( gap(j-1) + gap(N-j) );
              if (c > midc)
               { midc = c;
                 midj = j;
               }
	    }
	}
      if (midj == 0)
        { INS(N) DEL(1) }
      else
        { if (midj > 1) INS(midj-1)
          REP
	  if ( A[1] == B[midj] || isdna && va[B[midj]] == MATCHSC )
	     no_mat += 1;
	  else
	     no_mis += 1;
	  /* mark (A[I],B[J]) as used: put J into list row[I] */	
          I++; J++;
	  z = ( pairptr ) ckalloc( (int) sizeof(pair));
          z->COL = J;			
          z->NEXT = row[I];				
	  row[I] = z;
          if (midj < N) INS(N-midj)
        }
      return midc;
    }

/* Divide: Find optimum midpoint (midi,midj) of cost midc */

  midi = M/2;			/* Forward phase:                          */
  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
  t = -q;
  for (j = 1; j <= N; j++)
    { 
	CC[j] = t; // -r removed
	DD[j] = t-q;
	t = t - r;
    // if (A == IM && B == JN){CC[j] = 0;} // first row initialized to zero
    }
  t = -tb;
  for (i = 1; i <= midi; i++)
    { 
	s = CC[0];
	// if (j == j1 && o1 == 0){s = 100;} need not be considered?? 
	// CC[0] = 0;
      CC[0] = c = t;// -r removed 
	// if (A == &A[0]){CC[0] = c = 0;} // first column initialized to zero
      e = t-q;
      t = t - r;
      va = v[A[i]];
      for (j = 1; j <= N; j++)
        { 
	  if ((c = c - qr) > (e = e - r)) e = c; // to be altered for specific initiation preceded by insertion
          if ((c = CC[j] - qr) > (d = DD[j] - r)) d = c;
	  DIAG(i+I, j+J, c, s+va[B[j]])
          if (c < d) c = d;
          if (c < e) c = e;
          s = CC[j];
          CC[j] = c;
          DD[j] = d;
        }
    }
  DD[0] = CC[0];

  RR[N] = 0;			/* Reverse phase:                          */
  t = -q;			/*   Compute R(M/2,k) & S(M/2,k) for all k */
  for (j = N-1; j >= 0; j--)
    { RR[j] = t;
      SS[j] = t-q;	 
      t	= t - r; // -r removed
    }
  t = -te;
  for (i = M-1; i >= midi; i--)
    { s = RR[N];
      RR[N] = c = t; // -r removed
      e = t-q;	
      t = t-r;
      va = v[A[i+1]];
      for (j = N-1; j >= 0; j--)
        { if ((c = c - qr) > (e = e - r)) e = c;
          if ((c = RR[j] - qr) > (d = SS[j] - r)) d = c;
	  DIAG(i+1+I, j+1+J, c, s+va[B[j+1]])
          if (c < d) c = d;
          if (c < e) c = e;
          s = RR[j];
          RR[j] = c;
          SS[j] = d;
        }
    }
  SS[N] = RR[N];

  midc = CC[0]+RR[0];		/* Find optimal midpoint */
  midj = 0;
  type = 1;
  for (j = 0; j <= N; j++)
    if ((c = CC[j] + RR[j]) >= midc)
      if (c > midc || CC[j] != DD[j] && RR[j] == SS[j])
        { midc = c;
          midj = j;
        }
  for (j = N; j >= 0; j--)
    if ((c = DD[j] + SS[j] + q - r) > midc) // -r included to consider the double initiation as an initiation and extension
      { midc = c;
        midj = j;
        type = 2;
      }
if (count == 2){MIDI=midi+sti-1;MIDJ=midj+stj-1;TYPE=type;}//printf("*%c %c*\n",A[midi-2],B[midj-2]);}
}
/* Conquer: recursively around midpoint */

  if (type == 1)
    { (void) diff(A,B,midi,midj,tb,q,count,sti,stj);
      (void) diff(A+midi,B+midj,M-midi,N-midj,q,te,count,sti,stj);
    }
  else
    { (void) diff(A,B,midi-1,midj,tb,zero,count,sti,stj);
      DEL(2);
      (void) diff(A+midi+1,B+midj,M-midi-1,N-midj,zero,te,count,sti,stj);
    }
  return midc;
}



// To select list of anchors based on structural distance
anchor_selection()
{
	int i,j,a,b,k,l,flag1,flag2,i1,j1,s,e,p,flagc,m1,m2,d;
	char *ckalloc();
	int flagl,flagr,cmi,cmj,coi,coj,ct_tmp,flags,flage;
	double d1,d2;
	double distance_coordinates();
	NumBadAnch = 0;
	BadAnch = (int *) ckalloc(num_anch*sizeof(int));
	printf("anchor selection routine\n");
	// If the min number of gaps between anchors is greater than ??
	for(i = 0;i < num_anch;i++)
        {
		printf("checking for anchor%d\n",i);
		flagc = 0;
		for (p = 0;p < NumBadAnch; p++)
		{
			if (BadAnch[NumBadAnch] == i) {flagc = 1; break;}
		}	
		if (flagc == 1) continue; // if the anchor under consideration is already assigned as a bad anchor, skip
		flag1 = 0;
		flag2 = 0;
		printf("identifying anchors that are far apart\n");
		// finding the neighboring anchors
		ct_tmp = 0;
		for(j = 0;j < num_anch;j++)
        	{
			if (j == i) continue;
			ct_tmp++;
			d1 = ANCHS[i]->starti - ANCHS[j]->endi; // previous
			if(ct_tmp == 1) m1 = d1;
			if (d1 < m1 && d1 > 0)
			{
				m1 = d1;
				s = j;
				flag1 = 1;
			}
			d2 = ANCHS[j]->starti - ANCHS[i]->endi; // next
                        if(j == 0) m2 = d2;
                        if (d2 < m2 && d2 > 0)
                        {
                                m2 = d2;
                                e = j;
				flag2 = 1;
                        }
		}
		// If the anchors are far apart (> 50 res), check the score
		if (flag1 == 1)
		{
		i1 = ANCHS[i]->starti - ANCHS[s]->endi;
		j1 = ANCHS[i]->startj - ANCHS[s]->endj;
		if(i1 >  j1) d = i1 - j1; // no of gaps
		else d = j1 - i1;
		printf("distance with anchor%d is %d\n",s,d);
		flags = 0;
		if (d > 50)
		{
			if (ANCHS[s]->sc < 5000 && ANCHS[s]->sc < ANCHS[i]->sc)
			{
				for (p = 0;p < NumBadAnch; p++)
		                {
                        		if (BadAnch[NumBadAnch] == s) {flags = 1; break;}
                		}

				if(flags != 1) {BadAnch[NumBadAnch++] = s; printf("adding anchor%d as a bad anchor\n",s);}
			}
			else if (ANCHS[i]->sc < 5000 && ANCHS[i]->sc < ANCHS[s]->sc)
			{
				BadAnch[NumBadAnch++] = i;
				printf("adding anchor%d as a bad anchor\n",i);
			}
		}
		}
		flage = 0;
		if (flag2 == 1)
		{
                i1 = ANCHS[e]->starti - ANCHS[i]->endi;
                j1 = ANCHS[e]->startj - ANCHS[i]->endj;
                if(i1 > j1) d = i1 - j1;
                else d = j1 - i1;
		printf("distance with anchor%d is %d\n",e,d);
                if (d > 50)
                {
                        if (ANCHS[e]->sc < 5000 && ANCHS[e]->sc < ANCHS[i]->sc)
                        {
				for (p = 0;p < NumBadAnch; p++)
                                {
                                        if (BadAnch[NumBadAnch] == s) {flage = 1; break;}
                                }
                                if(flage != 1) {BadAnch[NumBadAnch++] = e; printf("adding anchor%d as a bad anchor\n",e);}
                        }
                        else if (ANCHS[i]->sc < 5000 && ANCHS[i]->sc < ANCHS[e]->sc)
                        {
				for (p = 0;p < NumBadAnch; p++)
                                {
                                        if (BadAnch[NumBadAnch] == i) {flage = 1; break;}
                                }
                                if(flage != 1) BadAnch[NumBadAnch++] = i;
				printf("adding anchor%d as a bad anchor\n",i);
                        }
                }
		} 
	}

	// Check consistancy with other anchors, ANCHS[0] is the main
	//
	// find the centre of max
	printf("checking inter-anchor distance\n");
	k = ANCHS[0]->length/2;
	if (ANCHS[0]->length >= 10) l = 0; // if the anchor length is more than 10, move 5 res on both sides
        else l = 2; // else move 3 res
	flagl = 1;
	flagr = 0;
	printf("identifying a central match\n");
	while (l<5)
        {
        	if (ANCHS[0]->ARi[k] != ANCHS[0]->ARi[k-1] && ANCHS[0]->ARj[k] != ANCHS[0]->ARj[k-1]) //match
                {
                	cmi = ANCHS[0]->ARi[k];
			cmj = ANCHS[0]->ARj[k]; 
			break;                      
                }
                else if (flagl == 1) // move towards left
                {
                        k--; 
                	l++; 
                }
		else if (flagr == 1) // move towards right when left movement is done
                {
                        k++;
                        l++;
                } 
		if (l == 5 && flagr == 0) // end of left move
		{
			if (ANCHS[0]->length >= 10) l = 0;
        		else l = 2;
			k = k + (5-l)+1;
			flagr = 1;
			flagl = 0;
		}
       	}
	printf("identifying a central match for other anchors\n");	
	// find the centre of others
	for(i = 1;i<num_anch;i++)
	{
		k = ANCHS[i]->length/2;
		if (ANCHS[i]->length >= 10) l = 0;
		else l = 2;
		flagl = 1;
		flagr = 0;
		while (l<5)
		{
			if (ANCHS[i]->ARi[k] != ANCHS[i]->ARi[k-1] && ANCHS[i]->ARj[k] != ANCHS[i]->ARj[k-1]) //match
			{
				coi = ANCHS[i]->ARi[k];
				coj = ANCHS[i]->ARj[k];
				break;		
			}
			else if(flagl == 1) 
			{
				k--; 
				l++; 
			}
     	 	        else if (flagr == 1)
                	{
                        	k++;
	                        l++;
        	        } 
	                if (l == 5 && flagr == 0) 
        	        {
                	        if (ANCHS[i]->length >= 10) l = 0;
                        	else l = 2;
	                        k = k + (5-l)+1;
        	                flagr = 1;
                	        flagl = 0;
                	}
		}
		printf("calculating the distance\n");
	// check the distance consistancy
		d1 = distance_coordinates(coordinates[0][coi]->x,coordinates[0][coi]->y,coordinates[0][coi]->z,coordinates[0][cmi]->x,coordinates[0][cmi]->y,coordinates[0][cmi]->z);
		d2 = distance_coordinates(coordinates[1][coj]->x,coordinates[1][coj]->y,coordinates[1][coj]->z,coordinates[1][cmj]->x,coordinates[1][cmj]->y,coordinates[1][cmj]->z);
		if (d1 > d2)
		{
			if ((d1 - d2) > 8) BadAnch[NumBadAnch++] = i;
			printf("difference in inter-anchor distance is %f\n",d1-d2);
		}
		else if ((d2-d1) > 8) 
		{
			BadAnch[NumBadAnch++] = i;
			printf("difference in inter-anchor distance is %f\n",d2-d1);
		}
		else printf("difference in inter-anchor distance is %f\n",d2-d1);
	}

}

double distance_coordinates(x1,y1,z1,x2,y2,z2) float x1,y1,z1,x2,y2,z2;
{
	double s,d;
	s = pow((x2 - x1),2) + pow((y2 - y1),2) + pow((z2 - z1),2);
	d = sqrt(s);
	return d;
}

/*Storing anchor coordinates*/
// Allocate memory for a structure that saves coordinates i,j.This has to be done after num_anch is calculated.
anchor_memory()
{
char *ckalloc();
int i;
ANCHS = (anc *) ckalloc((num_anch) * sizeof(anc));
for(i=0;i<num_anch;i++)
{
	ANCHS[i] = (anc) ckalloc(sizeof(anchor));
}
}

store_anchor(A,B,M,N,S,I,J) char A[], B[]; int M, N; int S[], I, J;
{ 
  char *ckalloc();	
  register int   i,  j, op, p, m, n; 
  int x, y, flag_mid;
  i = j = op = 0;
  anc cur;
  cur = ANCHS[NumAnchAdd];
  cur->ARi = (int *) ckalloc((m+n)*sizeof(int));
  cur->ARj = (int *) ckalloc((m+n)*sizeof(int));
  //printf("%d %d\n", I, J);
  flag_mid = 0;
  x = y = 0;		
  while (i < M || j < N)
    {
	m = i + I;
	n = j + J;
	//printf("%d %d\n", m,x);
	if (op == 0 && *S == 0) // MATCH
        { op = *S++;
	  cur->ARi[x] = m;
	  cur->ARj[y] = n;
	 /*if (NumAnchAdd == 0 && i > ((M/2) - 5) && flag_mid == 0) 
	{
		printf("hello");
		MIDI = m;
		MIDJ = n;
		flag_mid = 1;
	} */
	 i++;
	 j++;
	 x++;
	 y++;
          //*c++ = (*a == *b || isdna && v[*a][*b] == MATCHSC ) ? '|' : ' ';
        }
      else
        { if (op == 0) // INSERTION
            op = *S++;
          if (op > 0)
            {
	      cur->ARj[y] = n;
	      cur->ARi[x] = m-1; 
              op--;
		j++;
		x++;
		y++;
            }
          else
            { cur->ARi[x] = m; // DELETION
              cur->ARj[y] = n-1;
              op++;
		i++;
		x++;
		y++;	
            }
        }
}
NumAnchAdd++;
/*for(i = 0;i < x; i++)
printf("%d %d\n",cur->ARi[i],cur->ARj[i]);*/
}

/* Alignment display routine */

static char ALINE[51], BLINE[51], CLINE[51];

display(A,B,M,N,S,AP,BP,o) char A[], B[]; int M, N; int S[], AP, BP; FILE *o;
{ register char *a, *b, *c;
  register int   i,  j, op, x;
           int   lines, ap, bp;

  i = j = x = op = lines = 0;
  ap = AP;
  bp = BP;
  a = ALINE;
  b = BLINE;
  c = CLINE;
  while (i < M || j < N)
    { if (op == 0 && *S == 0)
        { op = *S++;
          *a = A[++i];
          *b = B[++j];
          *c++ = (*a == *b || isdna && v[*a][*b] == MATCHSC ) ? '|' : ' ';
	  a++;
	  b++;
        }
      else
        { if (op == 0)
            op = *S++;
          if (op > 0)
            { *a++ = ' ';
              *b++ = B[++j];
              op--;
            }
          else
            { *a++ = A[++i];
              *b++ = ' ';
              op++;
            }
          *c++ = '-';
        }
      if (a >= ALINE+50 || i >= M && j >= N)
        { *a = *b = *c = '\0';
          (void) fprintf(o,"\n%5d ",50*lines++);
          for (b = ALINE+10; b <= a; b += 10)
            (void) fprintf(o,"    .    :");
          if (b <= a+5)
            (void) fprintf(o,"    .");
          (void) fprintf(o,"\n%5d %s\n      %s\n%5d %s\n",ap,ALINE,CLINE,bp,BLINE);
	  ap = AP + i;
	  bp = BP + j;
          a = ALINE;
          b = BLINE;
          c = CLINE;
        }
    }
}

store_global(A,B,M,N,S,AP,BP,A1,B1) char A[], B[], A1[], B1[]; int M, N; int S[], AP, BP;
{
  register int   i,  j, op, x, y;
           int   lines, ap, bp;
  
  i = j = x = y = op = lines = 0;
  ap = AP;
  bp = BP;
  while (i < M || j < N)
    { if (op == 0 && *S == 0)
        { op = *S++;
	  x = ++i;
	  x += ap;
	  y = ++j;
          y += bp;			
	  seq1_out[s1++] = A[x];
	  seq2_out[s2++] = B[y];
	  //printf("\n$$$$%d$$$$\n", x);
	   global_score += (v[A[x]][B[y]] + Vaa[A1[x]][B1[y]]);
        }
      else
        { if (op == 0)
            op = *S++;
          if (op > 0)
            { 
		y = ++j;
          	y += bp;	
	      seq1_out[s1++] = '-';
              seq2_out[s2++] = B[y];
              op--;
	 if (num_bnd != 0) global_score -= 400;
	 else global_score -= 600;	
            }
          else
            { 
		x = ++i;
          	x += ap;
		seq1_out[s1++] = A[x];
              seq2_out[s2++] = '-';
              op++;
	 if (num_bnd != 0) global_score -= 400;
	 else global_score -= 600;	
            }
        }
    }
}




/* lib.c - library of C procedures. */

/* fatal - print message and die */
fatal(msg)
char *msg;
{
	(void) fprintf(stderr, "%s\n", msg);
	exit(1);
}

/* fatalf - format message, print it, and die */
fatalf(msg, val)
char *msg, *val;
{
	(void) fprintf(stderr, msg, val);
	(void) putc('\n', stderr);
	exit(1);
}
	
/* ckopen - open file; check for success */
FILE *ckopen(name, mode)
char *name, *mode;
{
	FILE *fopen(), *fp;

	if ((fp = fopen(name, mode)) == NULL)
	  fatalf("Cannot open %s.", name);
	return(fp);
}

/* ckalloc - allocate space; check for success */
char *ckalloc(amount)
int amount;
{
	char *p;

	if ((p = malloc( (unsigned) amount)) == NULL)
		fatal("Ran out of memory.");
	return(p);
}

