/*
Required data:
1) Sequences
2) Sequence Weights
3) Anchor positions and weights
4) 





 PROGRAM TO COMPUTE AN OPTIMAL MULTIPLE SEQUENCE ALIGNMENT FOR A SET OF PBs
 Input:
 1) Number of sequences
 2) file with pairwise distances
 3) file with all possible pairwise alignments: can have the name of sequences followed by the anchor bounds 
 Output
 1) Multiple PB alignment

 Weights: 
 1) For the sequences from the NJ output : multiplied with subs score
 2) For the residues in the anchor: 
        if the corresponding residues in the pairs considered is also aligned in the anchors
        
          Thus, Substitution scores = matrix score*seq_wt*anchor_wt
        
        
           Position specific gap penalties
            1) Higher penalty for the common anchor regions
            2) Accessiblity ? (Effect transfered through anchor weights), could be given a higher penalty
            3) gap penalty vs Number of sequences aligned     
        
*/




#include   <stdio.h>
#include   <string.h>
#include   <stdlib.h>

// Global and Static variables

#define  MATCHSC    10		/* match score */


static int match, mismh;		/* max and min substitution weights */
static int v[128][128];			/* substitution scores */
static int gin[128];			/*residue specific gap initiation penalty*/
static int gext[128];			/*residue specific gap extension*/
static int gi, ge, seq_comb, no_anch,diff_anch;
int M,N,K;
int **align1, **align2;			// for storing alignments within profiles
float *seq_wt;				// for storing the sequence weights
int *seq_len;				// for storing the sequence lengths
char **sequences;			// to store the PB sequences
int *Seq1,*Seq2;			// to store the component sequences of a profile
int **SS;				// array with profile substitution scores

typedef struct PROF_ALN // for storing the alignment in the form of a tree
		{
			int s1,s2; //  sequences/profiles no:
			int d; // distance between s1 and s2 in the tree
			int len;
			int *a1,*a2; // alignment of the two profiles
		} prof, *PA;


typedef struct ANCH_WT
		{
			int s1,s2; // seq1 & 2
			int len; // no of matches
			int *wt1;  // array of positions from seq1 in anchors
			int *wt2;  // array of positions from seq2 in anchors
			int *wt; // array of position specific weights
		} anch, *anc_aln;

PA *alignments;
anc_aln *anch_weight;
int P1,P2;
// FUNCTION MAIN

main(argc, argv) int argc; char *argv[];
{ 				/* Sequence lengths and k     */
  int  symbol,a,d,no_anch,ini,l,m,c1,c2,count1,count2,l1,l2;				/* The next character	      */
  int  ms, q, r;			/* User-supplied weights      */
  FILE *gu,*anc, *sw, *seq, *Sp,*ginit, *ckopen();	/* File Pointers */
  char *ckalloc();			/* space-allocating function  */
  char *ckrealloc();			/* reallocating */
  register int i, j ,k ,u ;		/* Looping variables */
  char  alph[129], *s;			/* alphabet */
  int  size,flag,flag_1,flag_2;		/* size of alphabet & flags*/
//  PA *alignments;			/* structure array for storing profile alignments*/
//  anc_aln *anch_weight;			/* structure array for storing anchor matches for all seq comb*/	
  PA aln,aln1,aln2;
  char c;	
  float seqwt;
  int s1,s2,len,flag_aln;
	
if ( argc != 9 && argc != 11 && argc != 12)
{ 
  fprintf(stderr,"Usage: %s k Seq1 [Seq2] mismatch gap_open gap_extend\n\n", argv[0]);
  fprintf(stderr,"k           number of sequences \n");

  fprintf(stderr,"pw_file     file with all pairwise alignments\n");
  fprintf(stderr,"dist        file with the pairwise distances\n");

  fprintf(stderr,"gap_openL   gap open penalty for anchors, a non-negative integer \n");
  fprintf(stderr,"gap_extendL gap extension penalty for anchors, a positive integer \n");
  fprintf(stderr,"gap_openG   gap open penalty for linkers, a positive integer \n");
  fprintf(stderr,"gap_extendG gap extension penalty for linkers, a positive integer \n");
  fprintf(stderr,"gap_openPB  gap open penalty specific for PBs, a positive integer \n");
  fprintf(stderr,"gap_extPB   gap extension penalty specific for PBs, a positive integer \n");
  /*exit(1);*/
}

	



// (1)
// Number of sequences

        /* ***read k: the number of sequences*** */
	(void) sscanf(argv[1],"%d", &K);


	// allocating memory for storing anchor alignments: (K*(K-1))/2 * (M + N) 
	seq_comb = (K*(K-1))/2;
	anch_weight = (anc_aln *) ckalloc(seq_comb*sizeof(anc_aln));
	for (i = 0;i < seq_comb; i++)
	{
		anch_weight[i] = (anc_aln) ckalloc(sizeof(anch));
	}



//


// (2)
// File with Sequence Weights and Sequences

        seq = fopen(argv[2],"r");
        sequences = (char **) ckalloc(K*sizeof(char *));
        for (i = 0; i < K; i++)
        {
                if (c = fgetc(seq) == '>')
                {
                        fscanf(seq,"%d %d %f", &a, &d, &seqwt);
                        seq_len[i] = d;
			seq_wt[i] = seqwt;
                }
                sequences[i] = (char *) ckalloc(d*sizeof(char)); // INDEX: 0 - (K-1)
                while(c != '\n') c = fgetc(seq);
                for(j=0;j<d;j++)
                {
                        fscanf(seq,"%c",&c);
                        sequences[i][j] = c;
                }
        }
        fclose(seq);

//
//


// (3)


	/* open the file with the order of tree alignment and */
	gu = ckopen(argv[3],"r");

//



// (?)

	/* opening the file with pairwise alignments */
	/*
	Format:
	>Seq1 Seq2
	Aln1 ..... 
	Aln2 .....
 	*/
//	pw = ckopen(argv[4], "r");


	// number of alignments = seq_comb
/*
	if ( (symbol = getc(pw) ) != '>' )
	  fatal("Sequence one must be in FASTA format, starting with '>'");
	for (size = 2; ( symbol = getc(Ap)) != EOF ; size++ )
	  if ( symbol == '\n' )
	    break;
*/

//



// (4)

	  // ***Store the substitution matrix (scaled to positive values) in a 2D array	***
	    Sp = ckopen(argv[4],'r');
	    (void) fscanf(Sp, "%s", alph);
	    size = strlen(alph);
	    match = mismh = 0;
	    for ( i = 0; i < size ; i++ )
	      for ( j = 0; j <= i ; j++ )
		{ (void) fscanf(Sp, "%d", &ms);
		  v[alph[i]][alph[j]] = v[alph[j]][alph[i]] = ms;
		  if ( ms > match ) match = ms;
		  if ( ms < mismh ) mismh = ms;
		}
	   fclose(Sp);
		

//
//





//(5)

	// ***Open the file with the anchor coordinates for sequence pairs***
	/*
	Format:
	>Seq1 Seq2 ;(anch bounds from Seq1);(anch bounds from Seq2)
	Anch position aln1 .... 
	Anch position aln2 ....
	Anch Weights  wt   ....
	Storing:
	array of structures such that array index = Sum(N - i) - 1, i = 1,...,Seq no:,N = no of seq
	*/
	anc = ckopen(argv[5],"r");
	no_anch = 0;
	// Parse the anchor file
	while(c = fgetc(anc) != EOF)
	{
		if (c == '>') //alignment header
		{
			fscanf(anc,"%d %d %d", &s1,&s2,&len); // to read first four details in the header
			anch_weight[j++]->s1 = s1;
			anch_weight[j]->s2 = s2;
			//anch_weight[j]->wt = wt;
			anch_weight[j]->len = len;
			while (c != '\n' && c != EOF)
			{
				c = fgetc(anc);
			}
			if (c == '\n') flag_aln = 1;
			no_anch++;
		
			//allocate memory for anch_weight->wt
			anch_weight[j]->wt1 = (int *) ckalloc(len*sizeof(int));
			anch_weight[j]->wt2 = (int *) ckalloc(len*sizeof(int));
			anch_weight[j]->wt = (int *) ckalloc(len*sizeof(int));
			// Read alignments
			for(i = 0; i < len; i++)
			{
				fscanf(anc,"%d",&a);
				anch_weight[j]->wt1[i] = a;
			}
			while(c != '\n') c = fgetc(anc);
			for(i = 0; i < len; i++)
                	{
                        	fscanf(anc,"%d",&a);
         	               anch_weight[j]->wt2[i] = a;
               	 	}
			while(c != '\n') c = fgetc(anc);
			for(i = 0; i < len; i++)
                        {
                                fscanf(anc,"%f",&c);
                               anch_weight[j]->wt[i] = c;
                        }
		
		}
	}
	diff_anch = seq_comb - no_anch;
	fclose(anc);
	



/* (7)

	 Read PB specific gap initiation and extension penalties from file
	
         If there are two extra arguments */
        if (argc == 12)
	{
 		ginit = ckopen(argv[argc-1],"r");
		for ( i = 0; i < size ; i++ )
		{
			(void) fscanf(ginit, "%d", &ini);
			gin[alph[i]] = q*ini;	// mean of ini could be 1 and variance can be min
		}/*
		gexten = ckopen(argv[argc-2],"r");
		for ( i = 0; i < size ; i++ )
       		{
       	        	(void) fscanf(ginit, "%d", &ini);
       	        	gext[alph[i]] = r*ini;   // mean ini = 1 and min variance
       	 	}*/
	}


//
//

	Seq1 = (int *) ckalloc(K*sizeof(int));
	Seq2 = (int *) ckalloc(K*sizeof(int));






// ALIGNMENT PROCESS

// Allocate memory for alignment structure arrays
	alignments = (PA *) ckalloc((K-1)*sizeof(PA));
	for (i = 0;i < K-1; i++)
	{
		alignments[i] = (PA) ckalloc(sizeof(prof));

	}
// read lines from file with guide tree; each line is a node
	for(i=0;i < K-1;i++)
	{
		flag_1 = flag_2 = 0;
		while (fgetc(gu) != '>') fgetc(gu);
	
		fscanf(gu,"%d %d %d %d %d", &l,&m,&d,&c1,&c2); // l = seq1, m = seq2, d = distance
		alignments[i]->s1 = l;	// Seq1
		alignments[i]->s2 = m;  // Seq2
		alignments[i]->d = d;	// distance
		count1 = c1;		// Number of components for Seq1
		count2 = c2;		// Number of components for Seq2
		aln = alignments[i];
		if (l > K || m > K) // if the alignment involves a node
		{
			if (l > K) // if first sequence is a profile
			{
				aln1 = alignments[l - K - 1];  // sequence of first component 
				l1 = aln1->len;
				flag_1 = 1;
				P1 = count1;
		
			        Seq1 = (int *) ckalloc(K*sizeof(int));

	
				// to allocate memory for storing the profile alignment ina  2D array
				align1 = (int **) ckalloc(count1*sizeof(int *));
				for (i=0;i<count1;i++)
				{
					align1[i] = (int *) ckalloc(l1*sizeof(int));
				}

				// store alignment in each column of the profile 
				for(i=0;i<l1;i++)
				{
					store_column(aln1,align1,i,-1,i,Seq1); // storing the alignment, columwise
				}
			}
			else // if it is a sequence
			{	
				l1 =  seq_len[l-1];// sequence length
				align1 = (int **) ckalloc(sizeof(int *));
                                align1[0] = (int *) ckalloc(l1*sizeof(int));
				for(i=0;i<l1;i++) align1[0][i] = sequences[l-1][i];
			}
			if (m > K) // if second sequence is a profile
			{
				aln2 = alignments[m - K - 1];
				l2 = aln2->len;
				flag_2 = 1;
				P2 = count2;

				Seq2 = (int *) ckalloc(K*sizeof(int));

				align2 = (int **) ckalloc(count2*sizeof(int *));
                                for (i=0;i<count2;i++)
                                {
                                        align2[i] = (int *) ckalloc(l1*sizeof(int));
                                }
				for(i=0;i<l2;i++)
                                {
                                        store_column(aln2,align2,i,-1,i,Seq2); // storing the alignment, columnwise
                                }
				
			}
			else 
			{
				l2 = seq_len[m-1];
				align2 = (int **) ckalloc(sizeof(int *));
                                align2[0] = (int *) ckalloc(l2*sizeof(int));
                                for(i=0;i<l2;i++) align2[0][i] = sequences[m-1][i];
			} 
			/* Allocate memory for storing the alignment */
			aln->a1 = (int *) ckalloc((l1+l2)*sizeof(int));
			aln->a2 = (int *) ckalloc((l1+l2)*sizeof(int));
			M = l1; /* length of profile1 */
			N = l2;	/* length of profile2 */
			/* generate the matrix of substitution scores */
			substitution_score();

			if (flag_1 == 1)
			{
				if (flag_2 == 1)
					diff(aln1,aln2,1,1); /* both inputs are profiles */
				else diff(aln1,al2,1,1);     /* first is a profile */
			}
			else 
			{
				diff(al1,aln2,0,1); 	    // seconds sequence is a profile
			}
			store_output(aln); // to store the output in aln 
		}
		else   // get the alignments from the guide file
		{
			c = fgetc(gu);
			while (c != '\n' and c!= EOF)
				c = fgetc(gu);
			l1 = seq_len[l-1]; 
			l2 = seq_len[m-1];
			aln->a1 = (int *) ckalloc((l1+l2)*sizeof(int));
                        aln->a2 = (int *) ckalloc((l1+l2)*sizeof(int));
			p = 0;
			while ((c = fgetc(gu)) != '\n' && c != EOF)
				aln->a1[p] = c;
			p = 0;
                        while ((c = fgetc(gu)) != '\n' && c != EOF)
                                aln->a2[p] = c;
			
		}
	}
	fclose(gu);
}

/*

// for finding the sequence components of a profile
recursive_profile(al,p,ct_seq,Seq) int p,ct_seq;int *Seq, 
{
	int l1,m1;
	l1 = al->s1 - K - 1;
	m1 = al->s2 - K - 1;
        if (p == 1)
	{
                while(l1 >= 0 || m1 >= 0) // until both the profile components are extracted completely
		{
                        l1 = al->s1 - K - 1;
                        m1 = al->s2 - K - 1;
                        if (l1 >= 0)
			{
                                aln1 = alignments[l1];
				(void) recursive_profile(aln1,1,ct_seq,Seq);
			}
			else
				// store the seq name
			{
				Seq[ct_seq++] = al->s1;
				p = 0;
			}
                        if (m1 >= 0)
			{
                                aln2 = alignments[m1];
				(void) recursive_profile(aln2,1,ct_seq,Seq);
			}
			else
			{ 
				// store the seq name
				Seq[ct_seq++] = al->s2;
				p = 0;
			}
		}
	return ct_seq;
	}
	else return 1;
}

*/

// to store each column of the alignment
store_column(al,align,index,ct,index1,Seq) int ct;PA al; int index,index1;int **align;int *Seq; 
{
	
	int i,l1,m1,pos;
	PA aln1,aln2;
	// Component sequences
		while(l1 >= 0 || m1 >= 0) // until both the profile components are extracted completely
		{
        		l1 = al->s1 - K - 1;
                	m1 = al->s2 - K - 1;
                	if (l1 >= 0)
			{
                		aln1 = alignments[l1];
				if (index != -1) // if the column index doesnt correspond to that of a parent gap 
				{
					pos = al->a1[index];
					if (pos == -1) store_column(aln1,align,-1,ct,index1); // if the position has a gap (-1), pass an index of -1
					else store_column(aln1,align,pos-1,ct,index1); // for a non gap position, pass an index of pos - 1
				}
				else store_column(aln1,align,-1,ct,index1); // if a position corresponds to a parent gap
				 
			}
                	else // if the profile is a single sequence
			{
				if (index == -1) align[++ct][index1] = -1; // parent gap
				else align[++ct][index1] = al->a1[index]; // filling the column
				if (index1 == 0) Seq[ct] = l1; 
			}

                	if (m1 >= 0)
			{
                        	aln2 = alignments[m1];
				if (index != -1)
				{
					pos = al->a2[index];
					if (pos == -1) store_column(aln2,align,-1,ct,index1);
					else store_column(aln2,align,pos-1,ct,index1);
				}
				else store_column(aln2,align,-1,ct,index1);
			}
                	else
			{
				if (index == -1) align[++ct][index1] = -1;
				else align[++ct][index1] = al->a2[index]; // filling the column
				if (index1 == 0) Seq[ct] = m1;
			}
                               
		}
}

/*
// to store anchor weights for different combinations of sequence components; currently all combinations stored 
anchor_weights()
{

// for el in seq_set1
	for(i = 0; i < ct_s1;i++)
	{
	// for el in seq_set2
		for(j = 0; j < ct_s2;j++)
		{
			ANCH_POS[]->s1 = 
			ANCH_POS[]->s2 = 
			// read corresponding lines from anchor file and store anchor coordinates in a temp array
				
			// extract the match positions and store in structures
			for (k = 0; k < array_size; k++)
			{
				if (anch1[k] != '-' && anch2[k] != '-') // matches
				{	
					ANCH_POS[]->wt1 = anch1[k];
					ANCH_POS[]->wt2 = anch1[k]; 
				}
			}
		}
	}

}
*/


//SS = (SS(Mat)*(1+Anch))*Seq1*Seq2
substitution_score()
{
	int *a1,*a2,seq; // for comparing columnwise
	int i,j,k,l,flag1,p,u,index;
	float ss,anchor;
	char *ckalloc();
//reallocate memory for SS
SS = (int **) ckalloc(M*sizeof(int *));
for(i=0;i<N;i++) SS[i] = (int *) ckalloc(N*sizeof(int));
//
for(i =0; i < M; i++) // for each position in align1
{
	for (j = 0; j < N;j++) // for each position in align2
	{
		ss = 0.0;
		// pairwise combinations
		for (k = 0; k < P1; k++) /* check if order of seq in Seq & align are same */
		{
			for(l = 0; l < P2; l++)
			{
				flag1 = 0;
				// finding the index in the anchors structure array
				if (Seq1[k] < Seq2[l]) 
				{	
					seq = Seq1[k];
					flag1 = 1;
				}
				else seq = Seq2[l];
				index = (seq*K - seq*(seq+1)/2) - diff_anch*K;
				if (index < 0) index = 0;
				for(p = index; p < no_anch; p++)
				{
					if (flag1 == 1)
					{
						if (anch_weight[p]->s1 == Seq1[k] && anch_weight[p]->s2 == Seq2[l])
						{
							index = p;
							break;
						}
					}
					else
					{ 
						if (anch_weight[p]->s1 == Seq2[l] && anch_weight[p]->s2 == Seq1[k])
						{
							index = p;
							break;
						}
					}
				}
				
				// checking if the coordinate is present as an anchor match
				if (flag1 == 1)
				{
					for(u=0;u<anch_weight[index]->len;u++)
					{
						if (anch_weight[index]->wt1[u] == align1[k][i])
						{
							if (anch_weight[index]->wt2[u] == align2[l][j]) 
								anchor = anch_weight[index]->wt[u];
							else anchor = 0;
						}
					}

				}
				/* Filling the position specific score matrix */
				if (align1[k][i] == -1 || align2[l][j] == -1) ss = 0; /* those involving gaps */
				else ss +=  v[sequences[Seq1[k]][align1[k][i]]][sequences[Seq2[l]][align2[l][j]]](1+anchor)*seq_wt(Seq1[k])*seq_wt(Seq2[l]);
			}
		}
		SS[M][N] = ss/(P1*P2);
	}
}
}


static int q, r;			/* gap penalties */
static int qr;				/* qr = q + r */

typedef struct ONE { int COL ;  struct ONE  *NEXT ;} pair, *pairptr;


pairptr *row, z; 			/* for saving used aligned pairs */
static short tt;

vertexptr  low = 0;			/* lowest score node in LIST */
vertexptr  most = 0;			/* latestly accessed node in LIST */
static int numnode;			/* the number of nodes in LIST */

static int *CC, *DD;			/* saving matrix scores */
static int *HH, *WW;		 	/* saving matrix scores */
static int  m1, mm, n1, nn;		/* boundaries of recomputed area */
static int  rl, cl;			/* left and top boundaries */
static int  min;			/* minimum score in LIST */
static short flag;			/* indicate if recomputation necessary*/
static int MIDI, MIDJ,TYPE;
static FILE *o1,*o2;
static char *seq1_out, *seq2_out;

/* DIAG() assigns value to x if (ii,jj) is never used before */
#define DIAG(ii, jj, x, value)				\
{ for ( tt = 1, z = row[(ii)]; z != 0; z = z->NEXT )	\
    if ( z->COL == (jj) )				\
      { tt = 0; break; }				\
  if ( tt )						\
    x = ( value );					\
}

/* replace (ss1, xx1, yy1) by (ss2, xx2, yy2) if the latter is large */
#define ORDER(ss1, xx1, yy1, ss2, xx2, yy2)		\
{ if ( ss1 < ss2 )					\
    { ss1 = ss2; xx1 = xx2; yy1 = yy2; }		\
  else							\
    if ( ss1 == ss2 )					\
      { if ( xx1 < xx2 )				\
	  { xx1 = xx2; yy1 = yy2; }			\
	else						\
	  if ( xx1 == xx2 && yy1 < yy2 )		\
	    yy1 = yy2;					\
      }							\
}

/* The following definitions are for function diff() */

int diff();
static int  zero = 0;				/* int type zero        */

#define gap(k)  ((k) <= 0 ? 0 : q+r*(k-1))	/* k-symbol indel score ; altered*/

static int *sapp;				/* Current script append ptr */
static int  last;				/* Last script op appended */

static int I, J;				/* current positions of A ,B */
static int no_mat; 				/* number of matches */ 
static int no_mis; 				/* number of mismatches */ 
static int al_len; 				/* length of alignment */
						/* Append "Delete k" op */
#define DEL(k)				\
{ I += k;				\
  al_len += k;				\
  if (last < 0)				\
    last = sapp[-1] -= (k);		\
  else					\
    last = *sapp++ = -(k);		\
}
						/* Append "Insert k" op */
#define INS(k)				\
{ J += k;				\
  al_len += k;				\
  if (last < 0)				\
    { sapp[-1] = (k); *sapp++ = last; }	\
  else					\
    last = *sapp++ = (k);		\
}

						/* Append "Replace" op */
#define REP 				\
{ last = *sapp++ = 0; 			\
  al_len += 1;				\
}

	FF = ( int * ) ckalloc(j);
	i = (M + 1) * sizeof(int);
	HH = ( int * ) ckalloc(i);
	WW = ( int * ) ckalloc(i);
	II = ( int * ) ckalloc(i);
	JJ = ( int * ) ckalloc(i);
	XX = ( int * ) ckalloc(i);
	YY = ( int * ) ckalloc(i);
	S = ( int * ) ckalloc(i + j);
	row = ( pairptr * ) ckalloc( (M + 1) * sizeof(pairptr));

	/* set up list for each row */
	for ( i = 1; i <= M; i++ )
	  if ( nseq == 2 )
	     row[i] = 0;
	  else
	    { row[i] = z = ( pairptr ) ckalloc( (int) sizeof(pair));
              z->COL = i;			
              z->NEXT = 0;
	    }

	q = Q;
	r = R;
	qr = q; // gap penalty = q + r(k-1)
	bounds = ( bnd * ) ckalloc(30 * sizeof(bnd));
	for (i = 0;i < 30; i++)
	bounds[i] = ( bnd ) ckalloc( (int) sizeof(bd));
	/*output memory allocation*/
	seq1_out = (char *) ckalloc((M+N)*sizeof(char));
	seq2_out = (char *) ckalloc((M+N)*sizeof(char)); 

 (void) printf("Match   Mismatch   Gap-Open Penalty   Gap-Extension Penalty\n");
 (void) printf(" %d      %d            %d                 %d\n\n",
			match, mismh, q, r);
	if ( nseq == 2 )
	  { (void) printf("Upper Sequence: %s\n", dhead1);
	    (void) printf("        Length: %d\n", M);
	    (void) printf("Lower Sequence: %s\n", dhead2);
	    (void) printf("        Length: %d\n", N);
	  }
	else
	  { (void) printf("Single Sequence: %s\n", dhead1);
	    (void) printf("         Length: %d\n", M);
	  }
	flag_end = 0;
	flag_optim = 0;
	o1 = fopen("anchors.aln","w");
        o2 = fopen("pb_alignment.aln","w");
	while(flag_end == 0)
	{

	// Allocating memory for storing nodes of scores
	numnode = min = 0;
	NumAnchAdd = 0;
	MIDI = MIDJ = 0;
	flag_opt = 0;
	s1 = s2 = 0;

	big_pass(A,B,M,N,K,nseq); // finding sub-optimal alignment regions

	count = 0;


        /* Report the K best alignments one by one. After each alignment is
           output, recompute part of the matrix. First determine the size
	   of the area to be recomputed, then do the recomputation         */


int diff(A,B,M,N,tb,te) char *A, *B int M, N; float tb, te;

{ int   midi, midj, type;	/* Midpoint, type, and cost */
  int midc;

{ register int   i, j;
  register int c, e, d, s;
           int t, *va;
  	   char  *ckalloc();
	   int gap_specific();	
/* Boundary cases: M <= 1 or N == 0 */
  if (N <= 0)
    { if (M > 0) DEL(M)
      return - gap_specific(A,M);
    }
  if (M <= 1)
    { if (M <= 0)
        { INS(N);
          return - gap_specific(B,N);
        }
      if (tb > te) tb = te;
      midc = - (tb + gap_specific(B,N) );
      midj = 0;
      va = v[A[1]];
      for (j = 1; j <= N; j++)
        {  
	for ( tt = 1, z = row[I+1]; z != 0; z = z->NEXT )	
              if ( z->COL == j+J )			
	         { tt = 0; break; }
           if ( tt )			
            { c = (va[B[j]]) - ( gap_specific(B,j-1) + gap_specific(B+j,N-j) );
              if (c > midc)
               { midc = c;
                 midj = j;
               }
	    }
	}
      if (midj == 0)
        { INS(N) DEL(1) }
      else
        { if (midj > 1) INS(midj-1)
          REP
	  if ( A[1] == B[midj] || isdna && va[B[midj]] == MATCHSC )
	     no_mat += 1;
	  else
	     no_mis += 1;
	  /* mark (A[I],B[J]) as used: put J into list row[I] */	
          I++; J++;
	  z = ( pairptr ) ckalloc( (int) sizeof(pair));
          z->COL = J;			
          z->NEXT = row[I];				
	  row[I] = z;
          if (midj < N) INS(N-midj)
        }
      return midc;
    }
/* Divide: Find optimum midpoint (midi,midj) of cost midc */

  midi = M/2;			/* Forward phase:                          */
  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
  t = -gin[B[1]];
 for (j = 1; j <= N; j++)
    { CC[j] = t;
      DD[j] = t-(gext[B[j]]); 
      t = t - gext[B[j]];
    }
  t = -tb; 
  for (i = 1; i <= midi; i++)
    { s = CC[0];
      CC[0] = c = t;
      e = t-gin[B[1]];	 
      t = t - gext[A[i]];
      va = v[A[i]];
      for (j = 1; j <= N; j++)
        { if ((c = c - gin[B[j]]) > (e = e - gext[B[j]])) e = c;
          if ((c = CC[j] - gin[A[i]]) > (d = DD[j] - gext[A[i]])) d = c;
	  DIAG(i+I, j+J, c, s+(va[B[j]]))
          if (c < d) c = d;
          if (c < e) c = e;
          s = CC[j];
          CC[j] = c;
          DD[j] = d;
        }
    }
  DD[0] = CC[0];

  RR[N] = 0;			/* Reverse phase:                          */
  t = -gin[B[N]];
  for (j = N-1; j >= 0; j--)
    { RR[j] = t;
      SS[j] = t-(gext[B[j+1]]);	
      t = t - gext[B[j+1]];
    }
  t = -te;
  for (i = M-1; i >= midi; i--)
    { s = RR[N];
      RR[N] = c = t;
      e = t-gin[B[N]];	
      t = t-gext[A[i+1]];
      va = v[A[i+1]];
      for (j = N-1; j >= 0; j--)
        { if ((c = c - gin[B[j+1]]) > (e = e - gext[B[j+1]])) e = c;
          if ((c = RR[j] - gin[A[i+1]]) > (d = SS[j] - gext[A[i+1]])) d = c;
	  DIAG(i+1+I, j+1+J, c, (s+va[B[j+1]]))
          if (c < d) c = d;
          if (c < e) c = e;
          s = RR[j];
          RR[j] = c;
          SS[j] = d;
        }
    }
  SS[N] = RR[N];

  midc = CC[0]+RR[0];		/* Find optimal midpoint */
  midj = 0;
  type = 1;
  for (j = 0; j <= N; j++)
    if ((c = CC[j] + RR[j]) >= midc)
      if (c > midc || CC[j] != DD[j] && RR[j] == SS[j])
        { midc = c;
          midj = j;
        }
  for (j = N; j >= 0; j--)
    if ((c = DD[j] + SS[j] + gin[A[midi]] - gext[A[midi]]) > midc) 
      { midc = c;
        midj = j;
        type = 2;
      }
}

/* Conquer: recursively around midpoint */

  if (type == 1)
    { (void) diff(A,B,midi,midj,gin[A[1]],gin[A[midi]]);
      (void) diff(A+midi,B+midj,M-midi,N-midj,gin[A[midi+1]],gin[A[M]]);
    }
  else
    { (void) diff(A,B,midi-1,midj,gin[A[1]],zero); // zero initiation penalty for reverse passes
      DEL(2);
      (void) diff(A+midi+1,B+midj,M-midi-1,N-midj,zero,gin[A[M]]); // zero initiation penalty for forward passes
    }
  return midc;
}



/* Alignment display routine */

static char ALINE[51], BLINE[51], CLINE[51];

display(A,B,M,N,S,AP,BP,o) char A[], B[]; int M, N; int S[], AP, BP; FILE *o;
{ register char *a, *b, *c;
  register int   i,  j, op, x;
           int   lines, ap, bp;

  i = j = x = op = lines = 0;
  ap = AP;
  bp = BP;
  a = ALINE;
  b = BLINE;
  c = CLINE;
  while (i < M || j < N)
    { if (op == 0 && *S == 0)
        { op = *S++;
          *a = A[++i];
          *b = B[++j];
          *c++ = (*a == *b || isdna && v[*a][*b] == MATCHSC ) ? '|' : ' ';
	  a++;
	  b++;
        }
      else
        { if (op == 0)
            op = *S++;
          if (op > 0)
            { *a++ = ' ';
              *b++ = B[++j];
              op--;
            }
          else
            { *a++ = A[++i];
              *b++ = ' ';
              op++;
            }
          *c++ = '-';
        }
      if (a >= ALINE+50 || i >= M && j >= N)
        { *a = *b = *c = '\0';
          (void) fprintf(o,"\n%5d ",50*lines++);
          for (b = ALINE+10; b <= a; b += 10)
            (void) fprintf(o,"    .    :");
          if (b <= a+5)
            (void) fprintf(o,"    .");
          (void) fprintf(o,"\n%5d %s\n      %s\n%5d %s\n",ap,ALINE,CLINE,bp,BLINE);
	  ap = AP + i;
	  bp = BP + j;
          a = ALINE;
          b = BLINE;
          c = CLINE;
        }
    }
}

store_global(A,B,M,N,S,AP,BP,ind) char A[], B[]; int M, N; int S[], AP, BP;
{
  register int   i,  j, op, x, y;
           int   lines, ap, bp;
  
  i = j = x = y = op = lines = 0;
  ap = AP;
  bp = BP;
  while (i < M || j < N)
    { if (op == 0 && *S == 0)
        { op = *S++;
	  x = ++i;
	  x += ap;
	  y = ++j;
          y += bp;			
	  alignments[ind]->a1[s1++] = A[x];
	  alignments[ind]->a2[s2++] = B[y];
	  //printf("\n$$$$%d$$$$\n", x);
        }
      else
        { if (op == 0)
            op = *S++;
          if (op > 0)
            { 
		y = ++j;
          	y += bp;	
	      alignments[ind]->a1[s1++] = -1;
              alignments[ind]->a2[s2++] = B[y];
              op--;
            }
          else
            { 
		x = ++i;
          	x += ap;
		alignments[ind]->a1[s1++] = A[x];
                alignments[ind]->a2[s2++] = -1;
              op++;
            }
        }
    }
}




/* lib.c - library of C procedures. */

/* fatal - print message and die */
fatal(msg)
char *msg;
{
	(void) fprintf(stderr, "%s\n", msg);
	exit(1);
}

/* fatalf - format message, print it, and die */
fatalf(msg, val)
char *msg, *val;
{
	(void) fprintf(stderr, msg, val);
	(void) putc('\n', stderr);
	exit(1);
}
	
/* ckopen - open file; check for success */
FILE *ckopen(name, mode)
char *name, *mode;
{
	FILE *fopen(), *fp;

	if ((fp = fopen(name, mode)) == NULL)
	  fatalf("Cannot open %s.", name);
	return(fp);
}

/* ckalloc - allocate space; check for success */
char *ckalloc(amount)
int amount;
{
	char *p;

	if ((p = malloc( (unsigned) amount)) == NULL)
		fatal("Ran out of memory.");
	return(p);
}

/* ckrealloc - reallocate space; check for success */
char *ckrealloc(amount)
int amount;
{
        char *p;

        if ((p = realloc( (unsigned) amount)) == NULL)
                fatal("Ran out of memory.");
        return(p);
}


