import PDBParser
import ProteineObj
import ResiduObj
import sys
from math import *
from os import path
from os import mkdir
from copy import *	

__doc__= """Module for obtaining statistics of pdb structure quality"""

#-------------------------------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------------------------------
class EchecParsing (Exception):
        def __init__(self ):
                print >>sys.stderr, "Echec parsing dans la prise en compte des breaks"


#-------------------------------------------------------------------------------------------------------------
class StructBuilder :

	
	
	def __init__(self, NomFichierPDB,verbose=1):
		"""Methode d'initialisation du constructeur de structure"""
		#Initialisation d'un objet Proteine
		self.Proteine = ProteineObj.Proteine(NomFichierPDB, verbose)
		#Initialisation d'un parseur du PDB
		self._PDBParser = PDBParser.PDBParser (NomFichierPDB)
		
	def Build_Chains (self, stat_file=None, naccess=0, dssp=0, stride=0, xtlsstr=0, promotif=0, segno=0, kaksi=0, store_out=0, out_dir=None, zip=0, breaks=0, rama_plot = False, chn=None, verbose = 1):
		"""Methode permettant d'appeler la construction d'une nouvelle chaine (objet) et d'initialiser ses champs:
		sequence 3 lettres, 1 lettre et sequence de structure (objet) de residus notamment.
		L'initialisation d'une chaine se fait d'apres les chaines vues dans SEQRES. Si ATOM n'a pas fourni les memes
		noms pour les chaines: une exception est lancee"""
		
		
		#Recuperation du dictionnaires contenant les chaines prot sous forme de seq d'aa en 3 lettres
		Dict_SEQRES_3lettres = self._PDBParser.SEQRES_Parser()
		#print Dict_SEQRES_3lettres
		#print [len(Dict_SEQRES_3lettres[ident]) for ident in Dict_SEQRES_3lettres] #**

		dict_modres = {}
		dict_breakcount = {}
		percent_missing = {}
		#print Dict_SEQRES_3lettres
		Dict_ATOM = self._PDBParser.ATOM_Parser()
		pdb_id = self.Proteine.IDpdb
		Resolution = self._PDBParser.ResolutionParser()
		if Resolution == None:
			Resolution = 77	
		TypeExperiment = self._PDBParser.ExperimentTypeParser()
		r_factor = float(self._PDBParser.rfactor_parser())
		dict_modres = self._PDBParser.Modresparser()[0]
		dict_missing = self._PDBParser.Missingparser()
		lines_seqres = self._PDBParser._DictSEQRES
		pdb_stat = []
		pdb_stat_list = {}	
#		f=open('statistics.txt','w')
		pdb_stats = []
		number_model = 0
		calpha_only = []
		dictscop_merge = {}
		dictscop_order = {}
		chain_prev = []
		chain_list = {}	
		if out_dir != None:
			if not path.exists(out_dir):
				mkdir(out_dir)
#                if stat_file != None:
#	        	stat=open(stat_file,'w')
 #                       stat.write("ID: PDB ID \nExp: Experimental Method\nRes: Resolution\nRF: R factor\nMod: Number of models\nChn: Chain ID\nres: Number of residues\nres_str: Number of residues with atom coordinates\natoms: Total number of atoms\nmodres: Number of residues modified (includes PTMs)\nbrks: Number of breaks (includes terminal breaks)\nmiss: % of residues missing(in breaks)\nmiss_bb: Percentage of residues with missing backbone atoms\nmiss_sc: Percentage of residues with missing side chain atoms\nPoor_BF: Percentage of atoms with bfactors > 40\nConfalt_CA: Percentage of residues with alt confs for CA\nConfalt_sc: Percentage of residues with alt conf for side chain\nSC : percent of atom pairs with short contacts\nrama_out: percent of residues with disallowed conf\nBL: percent of main chain bonds with non-standard bond lengths\nBA: Percent of main chain angles that deviate from the standard\n\n\n\nID   Exp Res  RF   Mod Chn res  res_str atoms modres brks miss    miss_bb miss_sc Poor_BF ConA_ca ConA_sc SC  ram_out BL      BA     \n\n")


		#Verification de la compatibilite des chaines repertoriees dans SEQRES et ATOM --> si probleme : renvoie une exception
		if len(Dict_SEQRES_3lettres) != 0 : #On verifie avant si le champs SEQRES existe
			self._PDBParser.Verif_EquiChaine_SEQRESetATOM()
		
		#Iteration sur les modeles
		#print Dict_ATOM	
		IterateurModel = iter(Dict_ATOM)
		#print IterateurModel
		#print "Taille Dict_ATOM = ", len(Dict_ATOM)
		while 1:
			try:
				ModelPDBcomplet = IterateurModel.next() #Modele PDB : contient un dico des chaines
				#print "Iter Modeles"
				#print ModelPDBcomplet
				#print len(ModelPDBcomplet)
				self.Proteine.CompteModeles ()
			
				number_model += 1
				#Iteration sur les chaines (SEQRES_Parser() renvoie le dictionnaire des lignes SEQRES par chaine. L'iterateur renvoie donc le nom des chaines)
				#print Dict_SEQRES_3lettres #**
				Iterateur_Chaines = iter(ModelPDBcomplet)
			
				while 1:
					try :
 
						#Nom de la chaine en cours
						Chaine = Iterateur_Chaines.next()
						chain_ID = Chaine
						if chn != None and chn.upper() != Chaine:
							continue
						if number_model != 1: continue
						if chain_ID == " ":
							chain_ID = "_"
						chain_name = pdb_id+"_"+`number_model`+"_"+chain_ID
						print chain_name
						if verbose ==1 : pass#print "Chaine", Chaine
						#Creation de la chaine dans l'objet Proteine 
						self.Proteine.add_Chaine (Chaine)
						#Initialisation du champ resolution et methode experiementale
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						self.Proteine.set_Resolution (Resolution)
						self.Proteine.set_TypeExperiment (TypeExperiment)
						chain_record = ModelPDBcomplet[Chaine]
						count_poor_bfactor = 0
						count_poor_occupancy = 0
						number_confalt_sc = 0
						number_confalt_bb = 0
						count_poor_occupancy_ca = 0
						list_phi_psi = {}
						count_res_break = 0
						#Initialisation des champs sequence 3 lettres et 1 lettre de la chaine
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						if len(Dict_SEQRES_3lettres) != 0 :
							Sequence_3lettres = Dict_SEQRES_3lettres[Chaine]
				#			print Sequence_3lettres
							number_residues = len(Sequence_3lettres)
							self.Proteine.Chaines[Chaine].setSequence_3lettres (Sequence_3lettres ) #Liste
							#print self.Proteine.Chaines[Chaine].Seq_3lettres #***
							Sequence_1lettre = self.Proteine.Chaines[Chaine].Seq3lettresTO1lettre(Sequence_3lettres)
							#print Sequence_1lettre
							self.Proteine.Chaines[Chaine].setSequence_1lettres (Sequence_1lettre)
							#print self.Proteine.Chaines[Chaine].Seq_1lettre #***

						#Initialisation du champ sequence de structures de residus
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						Sequence_Residus = ModelPDBcomplet[Chaine]
					#	print ModelPDBcomplet[Chaine] #***
						if dict_missing != None:
							try:
								dict_miss = dict_missing[Chaine]
							except KeyError:
								dict_miss = []
						else: 
							dict_miss = []
						self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_Seq_Struct(ModelPDBcomplet[Chaine], dict_miss)

						if out_dir != None:
							pdb_dir = out_dir+"/"+chain_name
						else: pdb_dir = chain_name
						
						if not path.exists(pdb_dir):
							mkdir(pdb_dir)
							self.Proteine.Chaines[Chaine].ModelsStruct[-1].write_pdb_chains(pdb_dir, pdb_id, number_model, Chaine, lines_seqres[Chaine])
						elif not path.isfile(pdb_dir+"/"+chain_name+".pdb"):
							self.Proteine.Chaines[Chaine].ModelsStruct[-1].write_pdb_chains(pdb_dir, pdb_id, number_model, Chaine, lines_seqres[Chaine])	
						else: pass	

                                                #------------------------------------------------------------------#
#						self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIParRes ("pdb_id"," ","temp", "temp" )
						if stat_file != None:
							number_SC = self.Proteine.Chaines[Chaine].ModelsStruct[-1].calculate_short_contact_wo_hb(ModelPDBcomplet[Chaine])
							count_poor_bfactor = self.Proteine.Chaines[Chaine].ModelsStruct[-1].poor_BF
							count_atoms = self.Proteine.Chaines[Chaine].ModelsStruct[-1].count_atoms

						#------------------------------------------------------------------#

							percent_missing[Chaine] = self.Proteine.Chaines[Chaine].ModelsStruct[-1].PourcResidusEntiersBBetSC()
	
							percent_missing_bb = percent_missing[Chaine][0]
	
							percent_missing_sc = percent_missing[Chaine][2]
	
							number_seq_struct = percent_missing[Chaine][4]
						#-------------------------------------------------------------------#

                                                #----------------------------------------------------------------#
	
						#NB: !!!! La proteine contient un dict de chaines. Une chaine est une liste de modele.( Un modele est une liste de dict d'atome)
						#Ici nous enregistrons, un ou plusieurs modele structuraux dans une chaine: on ajout donc la liste de dico d'atome dans le dernier objet modele cree [-1].

#						if summ_dir != None:
							
						#Prise en compte des breaks eventuels dans la structure
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						self.Proteine.Chaines[Chaine].ModelsStruct[-1].PriseEnCompteDesBreaks(Chaine,self.Proteine.Chaines[Chaine].Seq_3lettres,self.Proteine.Chaines[Chaine].Seq_1lettre,verbose)

                                                #-----------------------------------------------------------#

                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_AnglesPhiParRes()

                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_AnglesPsiParRes()

                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_PBsParRes("/home/lamarck/joseph/data/PBs_AnglesDiedres.data")
                                                #---------------------------------------------------------------#

                                                if naccess == 1:
                                                        acc_file = pdb_dir+"/"+"naccess.tar.gz"
							acc_dir = pdb_dir+"/"+"naccess"
							if not out_dir == None:
								temp_acc = out_dir + "/temp_naccess"
							else: temp_acc = "temp_naccess"
							if store_out == 1:
                                                        	if not path.isfile(acc_file) and not path.exists(acc_dir):
                                                                	self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_accessParRes(pdb_id, number_model, Chaine, pdb_dir, acc_dir)
								elif path.exists(acc_dir) and not path.isfile(acc_dir+"/"+chain_name+".rsa"):
									self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_accessParRes(pdb_id, number_model, Chaine, pdb_dir, acc_dir)
							else: self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_accessParRes(pdb_id, number_model, Chaine, pdb_dir, temp_acc)

#                                                else: self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_accessParRes(pdb_id, number_model, Chaine, "temp_chains", "temp_naccess")
                                                if dssp == 1:
                                                        dssp_file = pdb_dir + "/" + pdb_id +`number_model`+Chaine+".dssp.tar.gz"
							if not out_dir == None:
								temp_dssp = out_dir + "/temp_dssp"
							else: temp_dssp = "temp_dssp"
							if store_out == 1:
                                                        	if not path.isfile(dssp_file[:-7]) and not path.isfile(dssp_file):
                                                                	self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIParRes(pdb_id, number_model, Chaine, pdb_dir,pdb_dir)
	                                                else: self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIParRes(pdb_id, number_model, Chaine, pdb_dir, temp_dssp)

						if stride == 1:
	                                                stride_file = pdb_dir + "/" + pdb_id +`number_model`+Chaine+".str.tar.gz"

                                                        if not out_dir == None:
                                                                temp_stride = out_dir + "/temp_stride"
                                                        else: temp_stride = "temp_stride"
							if store_out == 1:
								if not path.isfile(stride_file) and not path.isfile(stride_file[:-7]):
									self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIStride(pdb_id, number_model, Chaine, pdb_dir,pdb_dir)
							else: 
								self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIStride(pdb_id, number_model, Chaine, pdb_dir,temp_stride)
						if xtlsstr == 1:
                                                        xtl_file = pdb_dir+"/"+"xtlsstr.tar.gz"
                                                        xtl_dir = pdb_dir+"/"+"xtlsstr"
                                                        if not out_dir == None:
                                                                temp_xtl = out_dir + "/temp_xtlsstr"
                                                        else: temp_xtl = "temp_xtlsstr"
                                                        if store_out == 1:
								if not path.exists(xtl_dir) and not path.isfile(xtl_file):
									self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIXtlsstr(pdb_id, number_model, Chaine, pdb_dir,xtl_dir)
								elif path.exists(xtl_dir) and not path.isfile(xtl_dir+"/pdbXXXX.vtr"):
									self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIXtlsstr(pdb_id, number_model, Chaine, pdb_dir,xtl_dir)
							else:
								self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIXtlsstr(pdb_id, number_model, Chaine, pdb_dir,temp_xtl)
						if promotif == 1:
							PM_file = pdb_dir+"/"+"promotif.tar.gz"
							PM_dir = pdb_dir+"/"+"promotif"
							if not out_dir == None:
								temp_PM = out_dir + "/temp_promotif"
							else: temp_PM = "temp_promotif"
                                                        if store_out == 1:
                                                                if not path.exists(PM_dir) and not path.isfile(PM_file):
                                                                        self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIPM(pdb_id, number_model, Chaine, pdb_dir,PM_dir)
                                                                elif path.exists(PM_dir) and not path.isfile(PM_dir+"/"+chain_name+".sst"):
                                                                        self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIPM(pdb_id, number_model, Chaine, pdb_dir,PM_dir)
                                                        else:
                                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIPM(pdb_id, number_model, Chaine, pdb_dir,temp_PM)

#							
#							self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIPM(pdb_id, number_model, Chaine, pdb_dir,"temp_PM")
                                                if segno == 1:
                                                        segno_file = pdb_dir + "/" + pdb_id +`number_model`+Chaine+".seg.tar.gz"

                                                        if not out_dir == None:
                                                                temp_segno = out_dir + "/temp_segno"
                                                        else: temp_segno = "temp_segno"
                                                        if store_out == 1:
                                                                if not path.isfile(segno_file) and not path.isfile(segno_file[:-7]):
                                                                        self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIISEGNO(pdb_id, number_model, Chaine, pdb_dir,pdb_dir)
                                                        else:
                                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIISEGNO(pdb_id, number_model, Chaine, pdb_dir,temp_segno)

                                                if kaksi == 1:
                                                        kaksi_file = pdb_dir + "/" + pdb_id +`number_model`+Chaine+".kak.tar.gz"

                                                        if not out_dir == None:
                                                                temp_kaksi = out_dir + "/temp_kaksi"
                                                        else: temp_kaksi = "temp_kaksi"
                                                        if store_out == 1:
                                                                if not path.isfile(kaksi_file) and not path.isfile(kaksi_file[:-7]):
                                                                        self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIKAKSI(pdb_id, number_model, Chaine, pdb_dir,pdb_dir)
                                                        else:
                                                                self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_SIIKAKSI(pdb_id, number_model, Chaine, pdb_dir,temp_kaksi)



						#-------------------------------------------------------------------#
						chain_prev.append(Chaine)

                                                #---------------------------------------------------------------#

                                                if rama_plot == True:
                                                        self.Proteine.Chaines[Chaine].ModelsStruct[-1].ramachandran_plot()
                                                #----------------------------------------------------------------# 
                                                #----------------------------------------------------------------#
                                                #scoplist = self.Proteine.Chaines[Chaine].ModelsStruct[-1].assign_scop("dir.des.scop.txt_1.73", pdb_id,Chaine)
                                                #scop_list = scoplist[0]
                                                #scopid_list = scoplist[1]

###########
						if number_model == 1:
							chain_details = deepcopy(self.Proteine.Chaines[Chaine].ModelsStruct[-1].chain_details(pdb_id+chain_ID))
                                                #----------------------------------------------------------------#


                                                #--------------------------------------------------------------------#


						if stat_file != None:						

	                                                bond_details = self.Proteine.Chaines[Chaine].ModelsStruct[-1].main_chain_checks()
        	                                        bad_bond = bond_details[0]
                	                                total_bond = bond_details[2]
                        	                        bad_angle = bond_details[1]
                                	                try: percent_BB = (float(bad_bond)/float(total_bond))*100
							except ZeroDivisionError: percent_BB = -77
                                                	try: percent_BA = (float(bad_angle)/float(total_bond))*100
							except ZeroDivisionError: percent_BA = -77


						#-----------------------------------------------------------#	

						#if number_model == 1:
						#	chain_id = Chaine
						#	if chain_id == "":
						#		chain_id = "_"
						#	file = pdb_dir + "/" + pdb_id + "_" + chain_id + ".fas"
						#	fasta = open(file, "w")
						#	fasta.writelines(self.Proteine.Chaines[Chaine].ChaineToFasta(pdb_id))
						#	fasta.close()
						if breaks == 1 and number_model == 1:
                                                        chain_id = Chaine
                                                        if chain_id == "":
                                                                chain_id = "_"
                                                        filename = pdb_dir + "/" + pdb_id + "_" + chain_id + ".brk"
							brk = open(filename, "w")
							brk.writelines(break_dict_list[0])
							brk.close()
						#--------------------------------------------------------------#
						if stat_file != None:
							if dict_modres == None:
								number_modres = 0
							else:
								try:
									number_modres = len(dict_modres[Chaine])
								except KeyError:
									number_modres = 0
#							try:
#								number_break = dict_breakcount[Chaine][-1]
#							except KeyError:
#								number_break = 0
													
							percent_res_break = (float(count_res_break)/float(number_residues))*100
							percent_poor_bfactor = (float(count_poor_bfactor)/float(count_atoms))*100
							percent_confalt_bb = (float(number_confalt_bb)/float(number_seq_struct))*100
							percent_confalt_sc = (float(number_confalt_sc)/float(number_seq_struct))*100
						#----------------------------------------------------------------#
	
						if stat_file != None:
							pdb_stats += ["%-4s %-3s %-4.2f %-4.2f %-3d %-3s %-4d %-7d %-5d %-6d %-4d %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %-3d %7.4f %7.4f %7.4f\n"%(pdb_id, TypeExperiment, Resolution, r_factor, number_model, Chaine, number_residues, number_seq_struct, count_atoms, number_modres, breakcount, percent_res_break, percent_missing_bb, percent_missing_sc, percent_poor_bfactor, percent_confalt_bb,percent_confalt_sc, number_SC, percent_outliers, percent_BB, percent_BA)]
#						stat.writelines(pdb_stats)
#						pdb_stat = [pdb_id, TypeExperiment, Resolution, Chaine, number_residues, number_seq_struct, count_atoms, number_modres, number_break, count_res_break, percent_missing_bb, percent_missing_sc, count_poor_bfactor, number_confalt_sc,number_confalt_bb, count_poor_occupancy_ca]
						if chn == None: break
					except StopIteration:
						break
				#print self.Proteine.Chaines
				#print self.Proteine.Resolution
			except StopIteration:
				break
		#print "******************************************************************************************"
				
#		print pdb_stats
		#return pdb_stats, calpha_only
		return chain_details	
#		if stat_file != None
#			f.writelines(pdb_stats)
#		f.close()


#-------------------------------------------------------------------------------------------------------------



# TEST ----------------------		
def main ():

	PBsLettres = {0:"a",1:"b",2:"c",3:"d",4:"e",5:"f",6:"g",7:"h",8:"i",9:"j",10:"k",11:"l",12:"m",13:"n",14:"o",15:"p",999:"Z"}
	print "PDB1"
#	try:
#	S1 = StructBuilder("essaiPDB_CA.pdb")
#	S1 = StructBuilder("pdbtest2.pdb")
#	S1 = StructBuilder("PDBs/pdb2icy.ent")
#	S1 = StructBuilder("testPDB3.pdb")
	S1 = StructBuilder("pdb1w6k.ent")
#	S1 = StructBuilder("PDBs/pdb1ifc.ent")
#	S1 = StructBuilder("PDBs/pdb1v8c.ent")
#	S1 = StructBuilder("PDBs/pdb2icy.ent")
#	S1 = StructBuilder("PDBs/pdb1v3u.ent")
#	S1 = StructBuilder("testPDB4.pdb")
#	S1 = StructBuilder("/home/bergman/bornot/data/NV_PROTO_MOY_formates/Proto_Pos0.pdb")
#	S1 = StructBuilder("PDBs/pdb1ee8.ent")
	S1.Build_Chains()
	#print S1.Proteine.MethodeExp
#	print S1.Proteine.ProtTOString_PDB()
#	f=open('testPDB.pdb','w')
#	f.writelines(S1.Proteine.ProtTOString_PDB())
#	f.close()
	#print S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct
#	chaine = 'A'
#	for i in range(len(S1.Proteine.Chaines[chaine].ModelsStruct)):
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].setMemoConformationsAlternatives_Ca()
#		print "ConformationAlt"
#		print S1.Proteine.Chaines[chaine].ModelsStruct[i].memoConformationsALT_CA
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].getIndicesDebFinPartie()
		
		
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesAlphasParRes() 
		#print [res.AngleAlpha for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesPhiParRes ()
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesPsiParRes ()
		#print [res.AnglesPhiPsi for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		#print [res.AnglesPhiPsi['Phi'] for res in S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct]
		#print [res.AnglesPhiPsi['Psi'] for res in S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct]
		
		#print "Protos"
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_ProtoParRes ("/home/etoile/bornot/data/NV_PROTO_MOY_formates/")
#		print [res.Proto for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
		#print "PBs"
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_PBsParRes ("/home/lamarck/joseph/data/PBs_AnglesDiedres.data")
#		print [PBsLettres[res.PB[0]] for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
		
		#print "SII"
#		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_SIIParRes ("10mh"," ","temp/", "temp/" )
		#print [res.DSSP for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
	#print S1.Proteine.Chaines[' '].ChaineTOListOfString_Banque (stringList=[], CodePDB="1ifc",TypeFich="BanqueGenerale")
	#f=open('testBanque.db','w')
	#f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Banque (stringList=[], CodePDB="2icy",TypeFich="ParChaine"))
	#f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Banque (stringList=[], CodePDB="10mh",TypeFich="BanqueGenerale"))
	#f.close()
#	f=open('testheader.db','w')
#	f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Header (S1.Proteine.IDpdb, S1.Proteine.MethodeExp , S1.Proteine.Resolution))
#	f.close()
#	f=open('testfasta.db','w')
#	f.writelines(S1.Proteine.Chaines[chaine].ChaineToFasta ("10MH"))
#	f.close()
#	f=open('testfastaBreaks.db','w')
#	f.writelines(S1.Proteine.Chaines[chaine].ChaineBreaksToFasta ("pdb1a83.ent")[0])
#	f.close()
#	except Exception,msg:
#		print >>sys.stderr, msg
#		pass

#	print "PDB2"
#	try:
#	S2 = StructBuilder("PDBs/pdb1ee8.ent")
#	S2.Build_Chains()
		
#	except Exception,msg:
#		print >>sys.stderr, msg
#		pass

if __name__ == "__main__":
	main()
