import PDBParser
import ProteineObj

import sys

__doc__= """Module permettant de construire un objet Proteine a partir du parsage d'un pdb"""

#-------------------------------------------------------------------------------------------------------------
class EchecParsing (Exception):
	def __init__(self ):
		print >>sys.stderr, "Echec parsing dans la prise en compte des breaks"
#-------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------
class StructBuilder :

	
	
	def __init__(self, NomFichierPDB,verbose=1):
		"""Methode d'initialisation du constructeur de structure"""
		#Initialisation d'un objet Proteine
		self.Proteine = ProteineObj.Proteine(NomFichierPDB, verbose)
		#Initialisation d'un parseur du PDB
		self._PDBParser = PDBParser.PDBParser (NomFichierPDB)
		
	def Build_Chains (self, verbose=1)	:
		"""Methode permettant d'appeler la construction d'une nouvelle chaine (objet) et d'initialiser ses champs:
		sequence 3 lettres, 1 lettre et sequence de structure (objet) de residus notamment.
		L'initialisation d'une chaine se fait d'apres les chaines vues dans SEQRES. Si ATOM n'a pas fourni les memes
		noms pour les chaines: une exception est lancee"""
		
		#Recuperation du dictionnaires contenant les chaines prot sous forme de seq d'aa en 3 lettres
		Dict_SEQRES_3lettres = self._PDBParser.SEQRES_Parser()
		#print Dict_SEQRES_3lettres
		#print [len(Dict_SEQRES_3lettres[ident]) for ident in Dict_SEQRES_3lettres] #**

		#print Dict_SEQRES_3lettres
		Dict_ATOM = self._PDBParser.ATOM_Parser()
		Resolution = self._PDBParser.ResolutionParser()
		TypeExperiment = self._PDBParser.ExperimentTypeParser()

		#Verification de la compatibilite des chaines repertoriees dans SEQRES et ATOM --> si probleme : renvoie une exception
		if len(Dict_SEQRES_3lettres) != 0 : #On verifie avant si le champs SEQRES existe
			self._PDBParser.Verif_EquiChaine_SEQRESetATOM()
		
		#Iteration sur les modeles
		IterateurModel = iter(Dict_ATOM)
		#print "Taille Dict_ATOM = ", len(Dict_ATOM)
		while 1:
			try:
				ModelPDBcomplet = IterateurModel.next() #Modele PDB : contient un dico des chaines
				#print "Iter Modeles"
				#print ModelPDBcomplet
				#print len(ModelPDBcomplet)
				self.Proteine.CompteModeles ()
			
				#Iteration sur les chaines (SEQRES_Parser() renvoie le dictionnaire des lignes SEQRES par chaine. L'iterateur renvoie donc le nom des chaines)
				#print Dict_SEQRES_3lettres #**
				Iterateur_Chaines = iter(ModelPDBcomplet)
			
				while 1:
					try :
						#Nom de la chaine en cours
						Chaine = Iterateur_Chaines.next()
						if verbose ==1 : print "Chaine", Chaine
						#Creation de la chaine dans l'objet Proteine 
						self.Proteine.add_Chaine (Chaine)
						
						#Initialisation du champ resolution et methode experiementale
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						self.Proteine.set_Resolution (Resolution)
						self.Proteine.set_TypeExperiment (TypeExperiment)
						if verbose ==1 : print self.Proteine.MethodeExp
						
						#Initialisation des champs sequence 3 lettres et 1 lettre de la chaine
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						if len(Dict_SEQRES_3lettres) != 0 :
							Sequence_3lettres = Dict_SEQRES_3lettres[Chaine]
							self.Proteine.Chaines[Chaine].setSequence_3lettres (Sequence_3lettres ) #Liste
							#print self.Proteine.Chaines[Chaine].Seq_3lettres #***
							Sequence_1lettre = self.Proteine.Chaines[Chaine].Seq3lettresTO1lettre(Sequence_3lettres)
							self.Proteine.Chaines[Chaine].setSequence_1lettres (Sequence_1lettre)
							#print self.Proteine.Chaines[Chaine].Seq_1lettre #***

						#Initialisation du champ sequence de structures de residus
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						Sequence_Residus = ModelPDBcomplet[Chaine]
						#print ModelPDBcomplet[Chaine] #***
						self.Proteine.Chaines[Chaine].ModelsStruct[-1].set_Seq_Struct(ModelPDBcomplet[Chaine])
						#NB: !!!! La proteine contient un dict de chaines. Une chaine est une liste de modele.( Un modele est une liste de dict d'atome)
						#Ici nous enregistrons, un ou plusieurs modele structuraux dans une chaine: on ajout donc la liste de dico d'atome dans le dernier objet modele cree [-1].

						#Prise en compte des breaks eventuels dans la structure
						#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						try :
							self.Proteine.Chaines[Chaine].ModelsStruct[-1].PriseEnCompteDesBreaks(Chaine,self.Proteine.Chaines[Chaine].Seq_3lettres,self.Proteine.Chaines[Chaine].Seq_1lettre,verbose)
						except Exception,msg:
							print msg
							self.Proteine.EchecParsing = 1
							raise EchecParsing
							
				
					except StopIteration:
						break
				#print self.Proteine.Chaines
				#print self.Proteine.Resolution
			except StopIteration:
				break
				


#-------------------------------------------------------------------------------------------------------------



# TEST ----------------------		
def main ():

	PBsLettres = {0:"a",1:"b",2:"c",3:"d",4:"e",5:"f",6:"g",7:"h",8:"i",9:"j",10:"k",11:"l",12:"m",13:"n",14:"o",15:"p",999:"Z"}
	print "PDB1"
#	try:
#	S1 = StructBuilder("essaiPDB_CA.pdb")
#	S1 = StructBuilder("pdbtest2.pdb")
#	S1 = StructBuilder("PDBs/pdb2icy.ent")
#	S1 = StructBuilder("testPDB3.pdb")
	S1 = StructBuilder("10MH.pdb")
#	S1 = StructBuilder("PDBs/pdb1ifc.ent")
#	S1 = StructBuilder("PDBs/pdb1v8c.ent")
#	S1 = StructBuilder("PDBs/pdb2icy.ent")
#	S1 = StructBuilder("PDBs/pdb1v3u.ent")
#	S1 = StructBuilder("testPDB4.pdb")
#	S1 = StructBuilder("/home/bergman/bornot/data/NV_PROTO_MOY_formates/Proto_Pos0.pdb")
#	S1 = StructBuilder("PDBs/pdb1ee8.ent")
	S1.Build_Chains()
	#print S1.Proteine.MethodeExp
#	print S1.Proteine.ProtTOString_PDB()
	f=open('testPDB.pdb','w')
	f.writelines(S1.Proteine.ProtTOString_PDB())
	f.close()
	#print S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct
	chaine = 'A'
	for i in range(len(S1.Proteine.Chaines[chaine].ModelsStruct)):
		S1.Proteine.Chaines[chaine].ModelsStruct[i].setMemoConformationsAlternatives_Ca()
		print "ConformationAlt"
		print S1.Proteine.Chaines[chaine].ModelsStruct[i].memoConformationsALT_CA
		S1.Proteine.Chaines[chaine].ModelsStruct[i].getIndicesDebFinPartie()
		
		
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesAlphasParRes() 
		print [res.AngleAlpha for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesPhiParRes ()
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_AnglesPsiParRes ()
		print [res.AnglesPhiPsi for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		#print [res.AnglesPhiPsi['Phi'] for res in S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct]
		#print [res.AnglesPhiPsi['Psi'] for res in S1.Proteine.Chaines['A'].ModelsStruct[0].Seq_Struct]
		
		print "Protos"
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_ProtoParRes ("/home/bergman/bornot/data/NV_PROTO_MOY_formates/")
		print [res.Proto for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
		print "PBs"
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_PBsParRes ("/home/bergman/bornot/data/PBs/PBs_AnglesDiedres.data")
		print [PBsLettres[res.PB[0]] for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
		print "SII"
		S1.Proteine.Chaines[chaine].ModelsStruct[i].set_SIIParRes ("10mh"," ","temp/", "temp/" )
		print [res.DSSP for res in S1.Proteine.Chaines[chaine].ModelsStruct[i].Seq_Struct]
		
	#print S1.Proteine.Chaines[' '].ChaineTOListOfString_Banque (stringList=[], CodePDB="1ifc",TypeFich="BanqueGenerale")
	f=open('testBanque.db','w')
	#f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Banque (stringList=[], CodePDB="2icy",TypeFich="ParChaine"))
	f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Banque (stringList=[], CodePDB="10mh",TypeFich="BanqueGenerale"))
	f.close()
	f=open('testheader.db','w')
	f.writelines(S1.Proteine.Chaines[chaine].ChaineTOListOfString_Header (S1.Proteine.IDpdb, S1.Proteine.MethodeExp , S1.Proteine.Resolution))
	f.close()
	f=open('testfasta.db','w')
	f.writelines(S1.Proteine.Chaines[chaine].ChaineToFasta ("2icy"))
	f.close()
	f=open('testfastaBreaks.db','w')
	f.writelines(S1.Proteine.Chaines[chaine].ChaineBreaksToFasta ("2icy"))
	f.close()
#	except Exception,msg:
#		print >>sys.stderr, msg
#		pass

	print "PDB2"
#	try:
#	S2 = StructBuilder("PDBs/pdb1ee8.ent")
#	S2.Build_Chains()
		
#	except Exception,msg:
#		print >>sys.stderr, msg
#		pass

if __name__ == "__main__":
	main()
